@@ -3,7 +3,7 @@
 # tests the -V switch in fim
 # TODO : should be integrated in the vim-like test suite.
 
-f=src/fim 
+f=${FIM_EXE-src/fim} # for e.g. Debian's post-installation tests
 
 fail()
 {
