@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2016-09-15 17:21:32 +0200 (Thu, 15 Sep 2016) $ */
+/* $LastChangedDate: 2024-03-30 12:48:15 +0100 (Sat, 30 Mar 2024) $ */
 /*
  AADevice.cpp : aalib device Fim driver file
 
- (c) 2008-2016 Michele Martone
+ (c) 2008-2024 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ static bool aainvalid;
 	)
 	{
 		/* output screen variables */
-		fim_byte_t PIXELVAL = (fim_byte_t) PIXELCOL;
+		const fim_byte_t PIXELVAL = (fim_byte_t) PIXELCOL;
 		fim_coo_t 
 			oi,// output image row index
 			oj;// output image columns index
@@ -137,7 +137,7 @@ static bool aainvalid;
 		fim_byte_t*srcp;
 		fim_coo_t idr,idc,lor,loc;
     		
-		fim_flags_t mirror=flags&FIM_FLAG_MIRROR, flip=flags&FIM_FLAG_FLIP;//STILL UNUSED : FIXME
+		const fim_flags_t mirror=flags&FIM_FLAG_MIRROR, flip=flags&FIM_FLAG_FLIP;//STILL UNUSED : FIXME
 
 		if ( !src ) return FIM_ERR_GENERIC;
 	
@@ -234,8 +234,8 @@ static bool aainvalid;
 //#define height() aa_scrheight(ascii_context_)
 
 	fim_err_t AADevice::display(
-		//struct ida_image *img, // source image structure
-		void *ida_image_img, // source image structure
+		//const struct ida_image *img, // source image structure
+		const void *ida_image_img, // source image structure
 		//void* rgb,// source rgb array
 		fim_coo_t iroff,fim_coo_t icoff, // row and column offset of the first input pixel
 		fim_coo_t irows,fim_coo_t icols,// rows and columns in the input image
@@ -253,7 +253,7 @@ static bool aainvalid;
 		 * FIXME : centering mechanisms missing here; an intermediate function
 		 * shareable with FramebufferDevice would be nice, if implemented in AADevice.
 		 * */
-		void* rgb = ida_image_img?((struct ida_image*)ida_image_img)->data:FIM_NULL;// source rgb array
+		void* rgb = ida_image_img?((const struct ida_image*)ida_image_img)->data:FIM_NULL;// source rgb array
 		if ( !rgb ) return FIM_ERR_GENERIC;
 	
 		if( iroff <0 ) return -2;
@@ -419,21 +419,32 @@ static bool aainvalid;
 		}*/
 		aa_hidecursor (ascii_context_);
 
-#if 0
 		/* won't help in the ill X11-windowed aalib shipped with many distros case */
-		sym_keys["Left" ]=AA_LEFT;
-		sym_keys["Right"]=AA_RIGHT;
-		sym_keys["Up"   ]=AA_UP;
-		sym_keys["Down" ]=AA_DOWN;
-		sym_keys["Esc"  ]=AA_ESC;
-#endif
+		dktorlkm_[AA_LEFT]=FIM_KKE_LEFT;
+		dktorlkm_[AA_RIGHT]=FIM_KKE_RIGHT;
+		dktorlkm_[AA_UP]=FIM_KKE_UP;
+		dktorlkm_[AA_DOWN]=FIM_KKE_DOWN;
+		dktorlkm_[AA_ESC]=FIM_KKE_ESC;
+		const int aa_f0 = 65870 - 1;
+		dktorlkm_[aa_f0+ 1]=FIM_KKE_F1;
+		dktorlkm_[aa_f0+ 2]=FIM_KKE_F2;
+		dktorlkm_[aa_f0+ 3]=FIM_KKE_F3;
+		dktorlkm_[aa_f0+ 4]=FIM_KKE_F4;
+		dktorlkm_[aa_f0+ 5]=FIM_KKE_F5;
+		dktorlkm_[aa_f0+ 6]=FIM_KKE_F6;
+		dktorlkm_[aa_f0+ 7]=FIM_KKE_F7;
+		dktorlkm_[aa_f0+ 8]=FIM_KKE_F8;
+		dktorlkm_[aa_f0+ 9]=FIM_KKE_F9;
+		dktorlkm_[aa_f0+10]=FIM_KKE_F10;
+		dktorlkm_[aa_f0+11]=FIM_KKE_F11;
+		dktorlkm_[aa_f0+12]=FIM_KKE_F12;
 
 	/*	The mulx and muly are not reliable */
 #if 0
 		if(ascii_context_->mulx>0 && ascii_context_->muly>0)
-			cc.setVariable("aascale",(((fim_scale_t)ascii_context_->muly)/((fim_scale_t)ascii_context_->mulx)));
-		cc.setVariable("aamuly",((fim_int)(ascii_context_->muly)));
-		cc.setVariable("aamulx",((fim_int)(ascii_context_->mulx)));
+			cc.setVariable("aascale",FIM_INT_SCALE_FRAC(ascii_context_->muly,static_cast<fim_scale_t>(ascii_context_->mulx)));
+		cc.setVariable("aamuly",(ascii_context_->muly));
+		cc.setVariable("aamulx",(ascii_context_->mulx));
 #endif
 		return FIM_ERR_NO_ERROR;
 	}
@@ -443,14 +454,14 @@ static bool aainvalid;
 		finalized_=true;
 		aa_close(ascii_context_);
 	}
-	int AADevice::get_chars_per_line(){return aa_scrwidth(ascii_context_);}
-	int AADevice::get_chars_per_column(){return aa_scrheight(ascii_context_);}
-	fim_coo_t AADevice::txt_width() { return aa_scrwidth(ascii_context_ ) ;}
-	fim_coo_t AADevice::txt_height(){ return aa_scrheight(ascii_context_) ;}
-	fim_coo_t AADevice::width() { return aa_imgwidth(ascii_context_ ) ;}
-	fim_coo_t AADevice::height(){ return aa_imgheight(ascii_context_) ;}
-
-	fim_err_t AADevice::init_console(void)
+	fim_coo_t AADevice::get_chars_per_line(void)const{return aa_scrwidth(ascii_context_);}
+	fim_coo_t AADevice::get_chars_per_column(void)const{return aa_scrheight(ascii_context_);}
+	fim_coo_t AADevice::txt_width(void)const { return aa_scrwidth(ascii_context_ ) ;}
+	fim_coo_t AADevice::txt_height(void)const{ return aa_scrheight(ascii_context_) ;}
+	fim_coo_t AADevice::width(void)const { return aa_imgwidth(ascii_context_ ) ;}
+	fim_coo_t AADevice::height(void)const{ return aa_imgheight(ascii_context_) ;}
+#if 0
+	fim_err_t AADevice::format_console(void)
 	{
 #ifndef FIM_WANT_NO_OUTPUT_CONSOLE
 		//mc_.setRows ( -height()/2);
@@ -459,7 +470,7 @@ static bool aainvalid;
 #endif /* FIM_WANT_NO_OUTPUT_CONSOLE */
 		return FIM_ERR_NO_ERROR;
 	}
-
+#endif
 	fim_err_t AADevice::fs_puts(struct fs_font *f, fim_coo_t x, fim_coo_t y, const fim_char_t *str)
 	{
 #if (!FIM_AALIB_DRIVER_DEBUG)
@@ -496,7 +507,7 @@ static bool aainvalid;
 
 	fim_err_t AADevice::status_line(const fim_char_t *msg)
 	{
-		fim_coo_t th=txt_height();
+		const fim_coo_t th=txt_height();
 		if(th<1)
 			goto err;
 #if (!FIM_AALIB_DRIVER_DEBUG)
@@ -517,48 +528,42 @@ err:
 	static aa_context *fim_aa_ascii_context; /* This was a global variable just for the use in fim_aa_get_input. Now one it's of no use.  */
 	static fim_sys_int fim_aa_get_input(fim_key_t * c, bool want_poll, bool * srp)
 	{
+		fim_key_t e = 0;
+
 		if(srp)
 		       	*srp = true;
 		if(!c)
 			return 0;
-		*c = 0x0;	/* blank */
-		*c = aa_getevent(fim_aa_ascii_context,0);/* 1 if want to receive AA_RELEASE events, too */
+		*c = FIM_SYM_NULL_KEY;
+		*c = aa_getevent(fim_aa_ascii_context,0);/* 1 for blocking wait */
 		if(*c==AA_UNKNOWN)
 			*c=0;
 		if(*c)
 			std::cout << "";/* FIXME : removing this breaks things. console-related problem, I guess */
 		if(!*c)
 			return 0;
-#if 0
-		if(*c==AA_UP   ){*c=272;return 1;}
-		if(*c==AA_DOWN ){*c=274;return 1;}
-		if(*c==AA_LEFT ){*c=276;return 1;}
-		if(*c==AA_RIGHT){*c=275;return 1;}
-#else
-		if(*c==AA_UP   ){*c=AA_UP;return 1;}
-		if(*c==AA_DOWN ){*c=AA_DOWN;return 1;}
-		if(*c==AA_LEFT ){*c=AA_LEFT;return 1;}
-		if(*c==AA_RIGHT){*c=AA_RIGHT;return 1;}
-#endif
-		/* FIXME : see defaultConfiguration.cpp */
-		if(*c==AA_UP   ){*c=4283163;return 1;}
-		if(*c==AA_DOWN ){*c=4348699;return 1;}
-		if(*c==AA_LEFT ){*c=4479771;return 1;}
-		if(*c==AA_RIGHT){*c=4414235;return 1;}
-		if(*c==AA_BACKSPACE){*c=127;return 1;}
 
-		/* FIXME : these five bindings work only under X .. */
-		if(*c==65765){*c=2117491483;return 1;}/* pageup   (arbitrary) */
-		if(*c==65766){*c=2117425947;return 1;}/* pagedown (arbitrary) */
-		if(*c==65779){*c=2117229339;return 1;}/* ins  (arbitrary) */
-		if(*c==65760){*c=2117163803;return 1;}/* home (arbitrary) */
-		if(*c==65767){*c=2117360411;return 1;}/* end  (arbitrary) */
-
-		if(*c==AA_ESC)
+		switch (*c)
 		{
-			*c=27;
+			case (AA_UP   ):     e=FIM_KKE_UP; break;
+			case (AA_DOWN ):     e=FIM_KKE_DOWN;break;
+			case (AA_LEFT ):     e=FIM_KKE_LEFT;break;
+			case (AA_RIGHT):     e=FIM_KKE_RIGHT;break;
+			case (AA_BACKSPACE): e=FIM_KKE_BACKSPACE;break;
+
+			/* Note: these five bindings work only under X11 .. */
+			case (65765): e=FIM_KKE_PAGE_UP;break;
+			case (65766): e=FIM_KKE_PAGE_DOWN;break;
+			case (65779): e=FIM_KKE_INSERT;break;
+			case (65760): e=FIM_KKE_HOME;break;
+			case (65767): e=FIM_KKE_END;break;
+
+			case (AA_ESC): e=FIM_KKE_ESC; break;
+		}
+		if (e)
+		{
+			*c = e;
 			return 1;
-			/* esc  */
 		}
 		if(srp)
 			*srp = false;
@@ -576,54 +581,50 @@ err:
 		if(sr)
 			return rc;
 
-		if(*c==65907)
+		rc = 1;
+		switch (*c)
 		{
-			status_line((const fim_char_t*)"control key not yet supported in aa. sorry!");
-			return 1;
-			/* left ctrl (arbitrary) */
-		}
-		if(*c==65908)
-		{
-			status_line((const fim_char_t*)"control key not yet supported in aa. sorry!");
-			return 1;
-			/* right ctrl (arbitrary) */
-		}
-		if(*c==65909)
-		{
-			status_line((const fim_char_t*)"lock key not yet supported in aa. sorry!");
-			return 1;
-			/* right ctrl (arbitrary) */
-		}
-		if(*c==65906)
-		{
-			status_line((const fim_char_t*)"shift key not yet supported in aa. sorry!");
-			return 1;
-			/* shift (arbitrary) */
-		}
-		if(*c==AA_MOUSE)
-		{
-			status_line((const fim_char_t *)"mouse events not yet supported in aa. sorry!");
-			return 1;
-			/* */
-		}
-		if(*c==AA_RESIZE )
-		{
-			//status_line((const fim_char_t *)"window resizing not yet supported. sorry!");
-			cc.resize(0,0);
-			/*aa_resize(ascii_context_);*//*we are not yet ready : the FimWindow and Viewport stuff .. */
-			return 0;
-			/* */
+			case (65907):
+				status_line((const fim_char_t*)"control key not yet supported in aa. sorry!");
+				/* left ctrl (arbitrary) */
+			break;
+			case (65908):
+				status_line((const fim_char_t*)"control key not yet supported in aa. sorry!");
+				/* right ctrl (arbitrary) */
+			break;
+			case (65909):
+				status_line((const fim_char_t*)"lock key not yet supported in aa. sorry!");
+				/* right ctrl (arbitrary) */
+			break;
+			case (65906):
+				status_line((const fim_char_t*)"shift key not yet supported in aa. sorry!");
+				/* shift (arbitrary) */
+			break;
+			case (AA_MOUSE):
+				status_line((const fim_char_t *)"mouse events not yet supported in aa. sorry!");
+				/* */
+			break;
+			case (AA_RESIZE):
+				//status_line((const fim_char_t *)"window resizing not yet supported. sorry!");
+				cc.display_resize(0,0);
+				/*aa_resize(ascii_context_);*//*we are not yet ready : the FimWindow and Viewport stuff .. */
+				rc = 0;
+				/* */
+			break;
 		}
 
+		if (dktorlkm_.find(*c)!=dktorlkm_.end())
+			*c = dktorlkm_[*c];
+
 		//std::cout << "event : " << *c << "\n";
 		//if(*c<0x80) return 1;
-		return 1;
+		return rc;
 	}
 
 	fim_err_t AADevice::resize(fim_coo_t w, fim_coo_t h)
 	{
 		/* resize is handled by aalib */
-		const bool want_resize_=true;
+		FIM_CONSTEXPR bool want_resize_=true;
 #if AA_LIB_VERSIONCODE>=104000
 		/* aalib version 104000 calls exit(-1) on zero-width/height contexts. this is simply stupid.
 		   the code we have here is not completely clean, but it catches the situation and proposes
@@ -650,8 +651,8 @@ err:
 
 		if(1==aa_resize(ascii_context_))
 		{
-			cc.setVariable(FIM_VID_SCREEN_WIDTH, (fim_int)width() );
-			cc.setVariable(FIM_VID_SCREEN_HEIGHT,(fim_int)height());
+			cc.setVariable(FIM_VID_SCREEN_WIDTH, width() );
+			cc.setVariable(FIM_VID_SCREEN_HEIGHT,height());
 			return FIM_ERR_NO_ERROR;
 		}
 		return FIM_ERR_GENERIC;
@@ -659,11 +660,16 @@ err:
 
 	fim_coo_t AADevice::status_line_height(void)const
 	{
-		return 1;
+		return FIM_AALIB_FONT_HEIGHT;
+	}
+
+	fim_coo_t AADevice::font_height(void)const
+	{
+		return FIM_AALIB_FONT_HEIGHT;
 	}
 
 #ifndef FIM_WANT_NO_OUTPUT_CONSOLE
-	AADevice::AADevice(MiniConsole & mc_, fim::string opts ):DisplayDevice(mc_),
+	AADevice::AADevice(MiniConsole& mc_, fim::string opts ):DisplayDevice(mc_),
 #else /* FIM_WANT_NO_OUTPUT_CONSOLE */
 	AADevice( fim::string opts ):DisplayDevice(),
 #endif /* FIM_WANT_NO_OUTPUT_CONSOLE */
