@@ -1,8 +1,8 @@
-/* $Id: Arg.h 115 2008-01-31 16:39:52Z dezperado $ */
+/* $LastChangedDate: 2024-01-07 12:46:19 +0100 (Sun, 07 Jan 2024) $ */
 /*
- Arg.h : Argument class
+ Arg.h : argument lists
 
- (c) 2007 Michele Martone
+ (c) 2007-2024 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,24 +18,17 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
-#ifndef ARG_FBVI_H
-#define ARG_FBVI_H
-#include "fim.h"
+#ifndef FIM_ARG_H
+#define FIM_ARG_H
+#include <vector>
+#include "fim_string.h"
 namespace fim
 {
-class Arg
-{
-	public:
-	fim::string val;
-	Arg(const fim::string &s):val(s){}
-	Arg(const Arg &arg):val(arg.val){}
-	bool operator==(int i){int v=atoi(val.c_str());return v==i;}
-	bool operator<=(int i){int v=atoi(val.c_str());return v<=i;}
-	bool operator>=(int i){int v=atoi(val.c_str());return v>=i;}
-	bool operator <(int i){int v=atoi(val.c_str());return v <i;}
-	bool operator >(int i){int v=atoi(val.c_str());return v >i;}
-};
-
-
+typedef std::vector<fim::string> args_t;
+int fim_args_opt_count(const args_t& args, const char oc);
+bool fim_args_opt_have(const args_t& args, fim::string optname);
+std::ostream& operator<<(std::ostream& os, const args_t& v);
+args_t fim_shell_args_escape(const args_t & args, bool quoted=true);
+fim::string fim_args_as_cmd(const args_t & args);
 }
-#endif
+#endif /* FIM_ARG_H */
