@@ -1,4 +1,4 @@
-/* $LastChangedDate: 2013-11-06 19:33:57 +0100 (Wed, 06 Nov 2013) $ */
+/* $LastChangedDate: 2024-03-25 02:17:19 +0100 (Mon, 25 Mar 2024) $ */
 /*
  Benchmarkable.h.h : header file for benchmarkable classes
 
@@ -24,10 +24,8 @@
 #if FIM_WANT_BENCHMARKS
 	class Benchmarkable{
 		private:
-		fim_int benchmarkstate;
+		// fim_int benchmarkstate{0};
 	       	public:
-		Benchmarkable(void){benchmarkstate=0;}
-		virtual ~Benchmarkable(void){}
 		virtual fim_int get_n_qbenchmarks(void)const=0;
 		virtual void quickbench(fim_int qbi)=0;
 		virtual string get_bresults_string(fim_int qbi, fim_int qbtimes, fim_fms_t qbttime)const=0;
