@@ -1,4 +1,4 @@
-/* $LastChangedDate: 2024-04-08 00:08:57 +0200 (Mon, 08 Apr 2024) $ */
+/* $LastChangedDate: 2024-05-11 20:15:30 +0200 (Sat, 11 May 2024) $ */
 /*
  Browser.cpp : Fim image browser
 
@@ -97,6 +97,12 @@ namespace fim
 	       	return flist_.cf();
 	}
 
+	fim_int Browser::current_p(void)const
+	{
+		// TODO: needs proper diffusion in the code...
+		return getGlobalIntVariable(FIM_VID_PAGE);
+	}
+
 #if FIM_WANT_PIC_LBFL
 	fim_cxr Browser::limit_to_variables(size_t min_vals, bool expanded) const
 	{
@@ -970,8 +976,8 @@ nostat:
 #endif
 		if ( ! ( dir = opendir(dn.c_str() ) ))
 			goto ret;
-
-		dn += FIM_CNS_DIRSEP_STRING;
+		if ( dn.back() != FIM_CNS_DIRSEP_CHAR )
+			dn += FIM_CNS_DIRSEP_STRING;
 	       	re = getGlobalStringVariable(FIM_VID_PUSHDIR_RE);
 		if( re == FIM_CNS_EMPTY_STRING )
 			re = FIM_CNS_PUSHDIR_RE;
@@ -1402,8 +1408,11 @@ ret:
 		if( args.size() > 0 )
 		{
 			const fim_int cfi = current_n();
-			for(size_t i = 0; i<args.size() && current_n() == cfi ; ++i)
+			const fim_int cpi = current_p();
+			for(size_t i = 0; i<args.size() && current_n() == cfi && current_p() == cpi ; ++i)
+			{
 				result = goto_image_internal(args[i].c_str(),FIM_X_NULL);
+			}
 		}
 		else
 			result = goto_image_internal(FIM_NULL,FIM_X_NULL);
@@ -2705,7 +2714,7 @@ void flist_t::erase_at_bitset(const fim_bitset_t& bs, fim_bool_t negative)
 	for(auto fit=bit;fit!=eit;++fit)
 		if(bs.at(fit-bit) != negative)
 			this->erase(fit-ecount),ecount++;
-	// TODO: updating cf_ here would make sense, but is redundant
+	cf_=size()?std::min(size()-1,cf_):0;// TODO: need tests and connection with adj_cf and copy_from_bitset
 }
 	fim_bool_t flist_t::pop_current(void)
 	{
