@@ -1,4 +1,4 @@
-/* $LastChangedDate: 2024-04-08 00:08:57 +0200 (Mon, 08 Apr 2024) $ */
+/* $LastChangedDate: 2024-05-11 14:46:02 +0200 (Sat, 11 May 2024) $ */
 /*
  Browser.h : Image browser header file
 
@@ -159,6 +159,7 @@ class Browser FIM_FINAL
 	public:
 	flist_t get_file_list(void)const { return flist_; }
 	int current_n(void)const;
+	fim_int current_p(void)const;
 	fim::string get_next_filename(int n)const; // FIXME: should be private
 	fim::string last_regexp_; // was private
 	fim_int last_src_dir_;
