@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2017-04-17 14:54:21 +0200 (Mon, 17 Apr 2017) $ */
+/* $LastChangedDate: 2024-02-23 00:49:56 +0100 (Fri, 23 Feb 2024) $ */
 /*
  Cache.cpp : Cache manager source file
 
- (c) 2007-2017 Michele Martone
+ (c) 2007-2024 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,14 +26,24 @@
 	#include <time.h>
 #endif /* HAVE_SYS_TIME_H */
 
+#define FIM_CACHE_MIPMAP 1
 #define FIM_CACHE_INSPECT 0
+#define FIM_PRINT_REPORT(X)   printf("CACHE:%c:%20s:%s",X,__func__,getReport(FIM_CR_CD).c_str());
 #if FIM_CACHE_INSPECT
-#define FIM_PR(X) printf("CACHE:%c:%20s:%s",X,__func__,getReport(FIM_CR_CD).c_str());
+#define FIM_PR(X)   FIM_PRINT_REPORT(X)
+//#define FIM_PS(X,S) FIM_PR(X); std::cout << " " << __func__ << "(" << S << ")\n";
 #else /* FIM_CACHE_INSPECT */
 #define FIM_PR(X) 
+//#define FIM_Ps(X) 
 #endif /* FIM_CACHE_INSPECT */
 
-//#define FIM_CACHE_DEBUG 0
+#define FIM_VERBOSE_CACHE 1
+
+#ifdef FIM_VERBOSE_CACHE
+#define FIM_ALLOW_CACHE_DEBUG getGlobalVariable(FIM_VID_DBG_COMMANDS).find('C') >= 0
+#else  /* FIM_VERBOSE_CACHE */
+#define FIM_ALLOW_CACHE_DEBUG 0
+#endif /* FIM_VERBOSE_CACHE */
 
 #if 0
 #define FIM_LOUD_CACHE_STUFF FIM_PR(-10); FIM_LINE_COUT
@@ -45,49 +55,52 @@
 	extern CommandConsole cc;
 namespace fim
 {
-	static fim_time_t fim_time(void) /* stand-alone function */
+	std::ostream& operator<<(std::ostream& os, const cache_key_t & key)
+	{
+		os << ( key.second.second == FIM_E_FILE ? "file " : "stdin " );
+		os << fim_basename_of(key.first) << " at page ";
+		os << key.second.first << "";
+		return os;
+	}
+
+	static fim_time_t fim_time(void) FIM_NOEXCEPT /* stand-alone function */
 	{
 #ifdef HAVE_SYS_TIME_H
 		struct timeval tv;
-		const fim_time_t prec = 1000; /* fraction of second precision */
-		gettimeofday(&tv, NULL);
+		FIM_CONSTEXPR fim_time_t prec = 1000; /* fraction of second precision */
+		gettimeofday(&tv, FIM_NULL);
 		return tv.tv_sec * prec + tv.tv_usec / ( 1000000 / prec );
 #else /* HAVE_SYS_TIME_H */
-		return time(NULL);
+		return time(FIM_NULL);
 #endif /* HAVE_SYS_TIME_H */
 	}
 
-	fim_time_t Cache::reltime(void)const
+	fim_time_t Cache::get_reltime(void)const
 	{
 		return fim_time()-time0_;
 	}
 
 	Cache::Cache(void)
 		:Namespace(&cc)
-#if FIM_WANT_BDI
-		,dummy_img_()
-#endif	/* FIM_WANT_BDI */
+		,time0_(fim_time())
 	{
-		/*	FIXME : potential flaw ?	*/
 		FIM_LOUD_CACHE_STUFF;
-		lru_.erase(lru_.begin(),lru_.end());
-		time0_ = fim_time();
 	}
 
 	int Cache::cached_elements(void)const
 	{
+		/*	the count of cached images	*/
 		FIM_LOUD_CACHE_STUFF;
 		return imageCache_.size();
 	}
 
-	Image* Cache::get_lru( bool unused )const
+	ImagePtr Cache::get_lru( bool unused )const
 	{
+		/* get the least recently used element.
+		 * if unused is true, only an unused image will be returned, _if any_*/
 		lru_t::const_iterator lrui;
-
-		/* warning : syscall ! */
-		fim_time_t m_time;
-		m_time = reltime();
-		Image*  l_img=NULL;
+		fim_time_t m_time(get_reltime()), l_time=0;
+		ImagePtr  l_img(FIM_NULL);
 		cachels_t::const_iterator ci;
 		FIM_LOUD_CACHE_STUFF;
 
@@ -96,35 +109,20 @@ namespace fim
 
 		for( ci=imageCache_.begin();ci!=imageCache_.end();++ci)
 		if( ci->second /* <- so we can call this function in some intermediate states .. */
-			 && last_used(ci->first) < m_time  &&  (  (! unused) || (used_image(ci->first)<=0)  ) )
+			 && (l_time=last_used(ci->first)) < m_time  &&  (  (! unused) || (used_image(ci->first)<=0)  ) )
 		{
 			l_img  = ci->second;
-			m_time = last_used(ci->first);
+			m_time = l_time;
 		}
 ret:
 		return l_img;
 	}
 
-	bool Cache::free_all(void)
-	{
-		/*
-		 * free all unused elements from the cache
-		 */
-		rcachels_t rcc = reverseCache_;
-
-		FIM_LOUD_CACHE_STUFF;
-		FIM_PR(' ');
-                for(    rcachels_t::const_iterator rcci=rcc.begin(); rcci!=rcc.end();++rcci )
-			if(usageCounter_[rcci->first->getKey()]==0)
-				erase( rcci->first );
-		return true;
-	}
-
 	int Cache::free_some_lru(void)
 	{
 		/*
-		 * this triggering deletion (and memory freeing) of cached elements
-		 * (yes, it is a sort of garbage collector, with its pros and cons)
+		 * trigger deletion (and memory free) of cached elements
+		 * (a sort of garbage collector)
 		 */
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR(' ');
@@ -133,34 +131,35 @@ ret:
 		return erase( get_lru(true)  );
 	}
 
-	int Cache::erase_clone(fim::Image* oi)
+	int Cache::erase_clone(fim::ImagePtr oi)
 	{
+		/*	erases the image clone from the cache	*/
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR(' ');
-#ifdef FIM_CACHE_DEBUG
-		std::cout << "erasing clone " << fim_basename_of(oi->getName()) << "\n";
-#endif /* FIM_CACHE_DEBUG */
-		cloneUsageCounter_.erase(oi);
+		if(FIM_ALLOW_CACHE_DEBUG)
+			std::cout << FIM_CNS_DBG_CMDS_PFX << "erasing clone " << fim_basename_of(oi->getName()) << "\n";
+		//cloneUsageCounter_.erase(oi);
+#if FIM_IMG_NAKED_PTRS
 		delete oi;
+#else /* FIM_IMG_NAKED_PTRS */
+		// TBD
+#endif /* FIM_IMG_NAKED_PTRS */
 		clone_pool_.erase(oi);
 		return 0;
 	}
 
 	bool Cache::need_free(void)const
 	{
-		/*	temporary : we free elements for more than some cached images	*/
-
+		/*	whether we should free some cache ..	*/
 		/*
 		struct mallinfo mi = mallinfo();
 		cout << "allocated : " <<  mi.uordblks << "\n";
 		if( mi.uordblks > getIntGlobalVariable(FIM_VID_MAX_CACHED_MEMORY) )
 			return true;
-
-		these are not the values we want ..
 		*/
-		int mci = getGlobalIntVariable(FIM_VID_MAX_CACHED_IMAGES);
-		int mcm = getGlobalIntVariable(FIM_VID_MAX_CACHED_MEMORY); /* getIntGlobalVariable */
-		size_t smcm = mcm > 0 ? mcm : 0;
+		const fim_int mci = getGlobalIntVariable(FIM_VID_MAX_CACHED_IMAGES);
+		const fim_int mcm = getGlobalIntVariable(FIM_VID_MAX_CACHED_MEMORY); /* getIntGlobalVariable */
+		const size_t smcm = mcm > 0 ? mcm : 0;
 
 	       	if( smcm > 0 && img_byte_size()/FIM_CNS_CSU > smcm )
 			goto rt;
@@ -172,26 +171,24 @@ ret:
 		if(mci > 0 && cached_elements() > mci)
 			goto rt;
 rf:
+		FIM_PR('n');
 		return false;
 rt:
+		FIM_PR('y');
 		return true;
 	}
 
 	int Cache::used_image(cache_key_t key)const
 	{
-		/*	acca' nun stimm'a'ppazzia'	*/
-		//return usageCounter_[key] ;
 		FIM_LOUD_CACHE_STUFF;
 		return ( usageCounter_.find(key)!=usageCounter_.end() ) ?  (*(usageCounter_.find(key))).second : 0;
 	}
 
-	bool Cache::is_in_clone_cache(fim::Image* oi)const
+	bool Cache::is_in_clone_cache(fim::ImagePtr oi)const
 	{
-		/*	acca' nun stimm'a'ppazzia'	*/
 		FIM_LOUD_CACHE_STUFF;
 		if(!oi)
-			return -1;
-		//return *(clone_pool_.find(oi))==oi;
+			return false;
 		return ( clone_pool_.find(oi)!=clone_pool_.end() )	
 			&&
 			((*clone_pool_.find(oi)) == oi );
@@ -199,65 +196,18 @@ rt:
 
 	bool Cache::is_in_cache(cache_key_t key)const
 	{
-		/*	acca' nun stimm'a'ppazzia'	*/
-		//return imageCache_[key]!=NULL;
 		FIM_LOUD_CACHE_STUFF;
-		return ( imageCache_.find(key)!=imageCache_.end() )
+		const bool iic = ( imageCache_.find(key) != imageCache_.end() )
 			&&
-			((*(imageCache_.find(key))).second!=NULL) ;
+			((*(imageCache_.find(key))).second!=FIM_NULL) ;
+		return iic;
 	}
 
-	bool Cache::is_in_cache(fim::Image* oi)const
-	{
-		/*	acca' nun stimm'a'ppazzia'	*/
-		FIM_LOUD_CACHE_STUFF;
-		if(!oi)
-			return -1;
-		return ( reverseCache_.find(oi)!=reverseCache_.end() )	
-			&&
-			( (*(reverseCache_.find(oi))).second.first.c_str()== oi->getKey().first );
-	}
-
-#if 0
-	int Cache::free(fim::Image* oi)
-	{
-		/*	acca' nun stimm'a'ppazzia'	*/
-		if(!oi)
-			return -1;
-
-		if(!is_in_cache(oi))
-		{
-#if 0
-			/* if the image is not already one of ours, it 
-			 * is probably a cloned one, and the caller 
-			 * didn't know this.
-			 *
-			 * in this case we keep it in the cache, 
-			 * so it could be useful in the future.
-			 * */
-			if( oi->revertToLoaded() )//removes internal scale caches
-				cacheImage( oi ); //FIXME : validity should be checked ..
-#else
-			delete oi;
-#endif
-			return 0;
-		}
-
-		/*
-		 * fixme : we should explicitly mark for deletion someday.. 
-		 * */
-
-		//if(need_free())return erase(oi);
-		/*	careful here !!	*/
-		//if(need_free())free_some_lru();
-		else return 0;	/* no free needed */
-	}
-#endif
-
 	int Cache::prefetch(cache_key_t key)
 	{
 		int retval = 0;
 		FIM_PR('*');
+		//FIM_PS('*',key);
 
 		FIM_LOUD_CACHE_STUFF;
 		if(is_in_cache(key))
@@ -279,9 +229,8 @@ rt:
 			FIM_PR('s');
 			goto ret;// just a fix in the case the browser is still lame
 		}
-#ifdef FIM_CACHE_DEBUG
-		std::cout << "prefetch request for "<< key.first << " \n";
-#endif /* FIM_CACHE_DEBUG */
+		if(FIM_ALLOW_CACHE_DEBUG)
+			std::cout << FIM_CNS_DBG_CMDS_PFX << "cache prefetch req     " << key << "\n";
 
     		if( regexp_match(key.first.c_str(),FIM_CNS_ARCHIVE_RE,1) )
 		{
@@ -290,125 +239,121 @@ rt:
 			goto ret;
 		}
 
-		if(!loadNewImage(key,0,true))
+		if(!loadNewImage(key,true))
 		{
 			retval = -1;
-#ifdef FIM_CACHE_DEBUG
-			std::cout << "loading failed\n";
-#endif /* FIM_CACHE_DEBUG */
+			if(FIM_ALLOW_CACHE_DEBUG)
+				std::cout << FIM_CNS_DBG_CMDS_PFX << "loading failed\n";
 			goto ret;
 		}
 		else
 		{
 			FIM_PR('l');
 		}
-		setGlobalVariable(FIM_VID_CACHED_IMAGES,(fim_int)cached_elements());
-		setGlobalVariable(FIM_VID_CACHE_STATUS,getReport().c_str());
+		setGlobalVariable(FIM_VID_CACHED_IMAGES,cached_elements());
+		setGlobalVariable(FIM_VID_CACHE_STATUS,getReport());
 ret:
 		FIM_PR('.');
 		return retval;
 	}
 
-	Image * Cache::loadNewImage(cache_key_t key, fim_page_t page, fim_bool_t delnc)
+	int Cache::prefetch(fid_t fid)
 	{
-		Image *ni = NULL;
+		return prefetch(cache_key_t{fid,{FIM_CNS_FIRST_PAGE,FIM_E_FILE}});
+	}
+
+	ImagePtr Cache::loadNewImage(cache_key_t key, fim_bool_t delnc)
+	{
+		const fim_page_t page {key.second.first};
+		ImagePtr ni = FIM_NULL;
 		FIM_PR('*');
 
 		FIM_LOUD_CACHE_STUFF;
 		/*	load attempt as alternative approach	*/
 		try
 		{
-		if( ( ni = new Image(key.first.c_str(), NULL, page) ) )
+		if( ( ni = ImagePtr( new Image(key.first.c_str(), FIM_NULL, page) ) ) )
 		{
-#ifdef FIM_CACHE_DEBUG
-			std::cout << "loadNewImage("<<key.first.c_str()<<")\n";
-#endif /* FIM_CACHE_DEBUG */
+			if(FIM_ALLOW_CACHE_DEBUG)
+				std::cout << FIM_CNS_DBG_CMDS_PFX << "cache loaded           " << key << "\n";
 			if( ni->cacheable() )
 				cacheNewImage( ni );
 			else
 				if (delnc) // delete non cacheable
 				{
+#if FIM_IMG_NAKED_PTRS
 					delete ni;
-					ni = NULL;
+#else /* FIM_IMG_NAKED_PTRS */
+					// TBD
+#endif /* FIM_IMG_NAKED_PTRS */
+					ni = FIM_NULL;
 				}
 		}
 		}
 		catch(FimException e)
 		{
 			FIM_PR('E');
-			ni = NULL; /* not a big problem */
+			ni = FIM_NULL; /* not a big problem */
 //			if( e != FIM_E_NO_IMAGE )throw FIM_E_TRAGIC;  /* hope this never occurs :P */
 		}
-ret:
-		FIM_PR(' ');
+		FIM_PR('.');
 		return ni;
 	}
 	
-	Image * Cache::getCachedImage(cache_key_t key)
+	ImagePtr Cache::getCachedImage(cache_key_t key)
 	{
-		/*
-		 * returns an image if already in cache ..
-		 * */
-		Image *ni = NULL;
+		/* returns an image if already in cache. */
+		ImagePtr ni = FIM_NULL;
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR(' ');
 	
-		/*	acca' nun stimm'a'ppazzia'	*/
-		//if(!key.first)return ni;
-
-		/*	cache lookup */
-		//this->cached_elements();
 		if( ( ni = this->imageCache_[key]) )
-		{
 			this->lru_touch(key);
-		}
 		return ni;
 	}
 
-	bool Cache::cacheNewImage( fim::Image* ni )
+	bool Cache::cacheNewImage( fim::ImagePtr ni )
 	{
+		const cache_key_t key = ni->getKey();
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR(' ');
-#ifdef FIM_CACHE_DEBUG
-		std::cout << "going to cache: "<< ni << "\n";
-#endif /* FIM_CACHE_DEBUG */
-		this->imageCache_[ni->getKey()]=ni;
-		this->reverseCache_[ni]= ni->getKey();
-		lru_touch( ni->getKey() );
-		usageCounter_[ ni->getKey()]=0; // we yet don't assume any usage
-		setGlobalVariable(FIM_VID_CACHED_IMAGES,(fim_int)cached_elements());
+		//if(FIM_ALLOW_CACHE_DEBUG)
+		//	std::cout << FIM_CNS_DBG_CMDS_PFX << "going to cache: "<< *ni /*<< " [" << ni << "]" */<< "\n";
+		if(FIM_ALLOW_CACHE_DEBUG)
+			std::cout << FIM_CNS_DBG_CMDS_PFX << "going to cache: "<< key << "\n";
+		this->imageCache_[key]=ni;
+		lru_touch( key );
+		usageCounter_[key]=0; // we don't assume any usage yet
+		setGlobalVariable(FIM_VID_CACHED_IMAGES,cached_elements());
 		return true;
 	}
 	
-	int Cache::erase(fim::Image* oi)
+	int Cache::erase(fim::ImagePtr oi)
 	{
-		/*
-		 * erases the image from the image cache
-		 * */
-		/*	acca' nun stimm'a'ppazzia'	*/
+		/*	erases the image from the cache	*/
 		int retval=-1;
+		cache_key_t key;
 		FIM_PR(' ');
 
 		FIM_LOUD_CACHE_STUFF;
 		if(!oi)
-		{
 			goto ret;
-		}
 
-		if(is_in_cache(oi) )
+		if( is_in_cache(key = oi->getKey()) )
 		{
-			usageCounter_[oi->getKey()]=0;
+			usageCounter_[key]=0;
 			/* NOTE : the user should call usageCounter_.erase(key) after this ! */
-#ifdef FIM_CACHE_DEBUG
-			std::cout << "will erase  "<< oi << " " <<  fim_basename_of(oi->getName()) << " time:"<< lru_[oi] << "\n";
-			std::cout << "erasing original " << fim_basename_of(oi->getName()) << "\n";
-#endif /* FIM_CACHE_DEBUG */
-			lru_.erase(oi);
-			imageCache_.erase(reverseCache_[oi]);
-			reverseCache_.erase(oi);
-//			delete imageCache_[reverseCache_[oi]];
+			if(FIM_ALLOW_CACHE_DEBUG)
+				std::cout << FIM_CNS_DBG_CMDS_PFX << "cache erases           " << (oi->getKey()) << "\n"; /*<< " with timestamp " << lru_[oi->getKey()] */
+			lru_.erase(key);
+			imageCache_.erase(key);
+			usageCounter_.erase(key);
+#if FIM_IMG_NAKED_PTRS
 			delete oi;
-			setGlobalVariable(FIM_VID_CACHED_IMAGES,(fim_int)cached_elements());
+#else /* FIM_IMG_NAKED_PTRS */
+			// TBD
+#endif /* FIM_IMG_NAKED_PTRS */
+			setGlobalVariable(FIM_VID_CACHED_IMAGES,cached_elements());
 			retval = 0;
 		}
 ret:
@@ -422,9 +367,9 @@ ret:
 		FIM_LOUD_CACHE_STUFF;
 		if(imageCache_.find(key)==imageCache_.end())
 			goto ret;
-		if(lru_.find(imageCache_.find(key)->second )==lru_.end())
+		if(lru_.find(key)==lru_.end())
 			goto ret;
-		retval = lru_.find(imageCache_.find(key)->second )->second;
+		retval = lru_.find(key)->second;
 ret:
 		return retval;
 	}
@@ -432,60 +377,71 @@ ret:
 	int Cache::lru_touch(cache_key_t key)
 	{
 		/*
-		 * if the specified file is cached, in this way it is marked as used, too
-		 *
-		 * NOTE : the usage count is not affected, 
+		 * if the specified file is cached, in this way it is marked as used, too.
+		 * the usage count is not affected, 
 		 * */
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR(' ');
-//		std::cout << lru_[imageCache_[key]] << " -> ";
-		lru_[imageCache_[key]]= reltime();
-//		std::cout << lru_[imageCache_[key]] << "\n";
+//		std::cout << lru_[key] << " -> ";
+		lru_[key]= get_reltime();
+//		std::cout << lru_[key] << "\n";
 		return 0;
 	}
 
-	bool Cache::freeCachedImage(Image *image, const ViewportState *vsp)
+	bool Cache::freeCachedImage(ImagePtr image, const ViewportState *vsp, bool force)
 	{
 		/*
-		 * TODO: rename to free().
-		 *
 		 * If the supplied image is cached as a master image of a clone, it is freed and deregistered.
 		 * If not, no action is performed.
 		 * */
-		// WARNING : FIXME : DANGER !!
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR('*');
 
 		if( !image )
 			goto err;
 
-//		if( is_in_cache(image) && usageCounter_[image->getKey()]==1 )
 		if(vsp)
-		{
 			viewportInfo_[image->getKey()] = *vsp;
-		}
 		if( is_in_clone_cache(image) )
 		{
-			usageCounter_[image->getKey()]--;
 			FIM_PR('c');
+			if(FIM_ALLOW_CACHE_DEBUG)
+				std::cout << FIM_CNS_DBG_CMDS_PFX << "decrement clones count " << image->getKey() << "\n";
+			usageCounter_[image->getKey()]--;
 			erase_clone(image);	// we _always_ immediately delete clones
-			setGlobalVariable(FIM_VID_CACHE_STATUS,getReport().c_str());
+			setGlobalVariable(FIM_VID_CACHE_STATUS,getReport());
 			goto ret;
 		}
 		else
-		if( is_in_cache(image) )
+		if(!is_in_cache(image->getKey()) )
+		{
+			FIM_PR('n');
+			if(FIM_ALLOW_CACHE_DEBUG)
+				std::cout << FIM_CNS_DBG_CMDS_PFX << "can't unuse uncached!! " << image->getKey() << "\n";
+		}
+		else
+		if( is_in_cache(image->getKey()) )
 		{
 			FIM_PR('-');
+			if(FIM_ALLOW_CACHE_DEBUG)
+				std::cout << FIM_CNS_DBG_CMDS_PFX
+					<< "unuse  " << image->getKey()
+					<< "  force:" << force
+					<< "  usageCounter_():" << (usageCounter_[image->getKey()])
+					<< "  cached_elements():" << cached_elements() << "\n";
 			lru_touch( image->getKey() ); // we have been using it until now
 			usageCounter_[image->getKey()]--;
-#if FIM_WANT_EXPERIMENTAL_MIPMAPS
-			image->mm_free();
-#endif /* FIM_WANT_EXPERIMENTAL_MIPMAPS */
+#if FIM_WANT_MIPMAPS
+			if( getGlobalStringVariable(FIM_VID_CACHE_CONTROL).find('M') == 0 )
+				image->mm_free();
+#endif /* FIM_WANT_MIPMAPS */
 			if(
-				(usageCounter_[image->getKey()])==0 && 
-				image->getKey().second!=FIM_E_STDIN 
-				)
+				( (usageCounter_[image->getKey()])==0 && 
+				image->getKey().second.second!=FIM_E_STDIN  )
+				 || force)
 			{
+				const fim_int minciv = getGlobalIntVariable(FIM_VID_MIN_CACHED_IMAGES);
+				const fim_int minci = ( minciv < 1 ) ? 4 : minciv;
 #if 0
 				if( need_free() && image->getKey().second!=FIM_E_STDIN )
 				{
@@ -494,27 +450,38 @@ ret:
 					usageCounter_.erase(key);
 				}
 #else
-				/* doing it here is dangerous : */
-				if( need_free() )
+				if( ( need_free() && cached_elements() > minci ) || force )
 				{
-					Image * lrui = get_lru(true);
+					ImagePtr lrui;
+					if(FIM_ALLOW_CACHE_DEBUG)
+						std::cout << FIM_CNS_DBG_CMDS_PFX << "cache suggested free  " << image->getKey() << (force?"  forced=1":"  forced=0") << "\n";
+					if(force)
+						lrui = image;
+					else
+						lrui = get_lru(true);
+//					if(FIM_ALLOW_CACHE_DEBUG && force)
+//						std::cout << FIM_CNS_DBG_CMDS_PFX << "cache forced to free  " << lrui->getName() << "\n";
 					FIM_PR('o');
 					if( lrui ) 
 					{
-						cache_key_t key = lrui->getKey();
-						if( FIM_VCBS(viewportInfo_) > FIM_CNS_VICSZ )
+						const cache_key_t key = lrui->getKey();
+						if(FIM_ALLOW_CACHE_DEBUG)
+							std::cout << FIM_CNS_DBG_CMDS_PFX << "cache to free          " << key << (force?"  forced=1":"  forced=0") << "\n";
+						if( ( FIM_VCBS(viewportInfo_) > FIM_CNS_VICSZ ) || force )
 							viewportInfo_.erase(key);
-						if(( key.second != FIM_E_STDIN ))
+						if(( key.second.second != FIM_E_STDIN ))
 						{	
 							this->erase( lrui );
-							usageCounter_.erase(key);
 						}
 					}
+					else
+						if(FIM_ALLOW_CACHE_DEBUG)
+							std::cout << FIM_CNS_DBG_CMDS_PFX << "nothing to free: (" << cached_elements() << " cached elements)\n";
 					// missing usageCounter_.erase()..
 				}
 #endif
 			}
-			setGlobalVariable(FIM_VID_CACHE_STATUS,getReport().c_str());
+			setGlobalVariable(FIM_VID_CACHE_STATUS,getReport());
 			goto ret;
 		}
 err:
@@ -525,10 +492,10 @@ ret:
 		return true;
 	}
 
-	Image * Cache::useCachedImage(cache_key_t key, ViewportState *vsp, fim_page_t page)
+	ImagePtr Cache::useCachedImage(cache_key_t key, ViewportState *vsp)
 	{
 		/*
-		 * TODO: rename to get().
+		 * Shall rename to get().
 		 *
 		 * The caller invokes this member function to obtain an Image object pointer.
 		 * If the object is cached and it already used, a clone is built and returned.
@@ -540,21 +507,20 @@ ret:
 		 * A freeImage action will do the converse operation (and delete).
 		 * If the image is not already cached, it is loaded, if possible.
 		 *
-		 * So, if there is no such image, NULL is returned
+		 * So, if there is no such image, FIM_NULL is returned
 		 * */
-		Image * image = NULL;
+		ImagePtr image = FIM_NULL;
 
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR('*');
-#ifdef FIM_CACHE_DEBUG
-		std::cout << "  useCachedImage(\""<<fim_basename_of(key.first.c_str())<<" of type "<< ( key.second == FIM_E_FILE ? " file ": " stdin ")<<"\")\n";
-#endif /* FIM_CACHE_DEBUG */
+
+		if(FIM_ALLOW_CACHE_DEBUG)
+			std::cout << FIM_CNS_DBG_CMDS_PFX << "cache check for " << ( key.second.second == FIM_E_FILE ? " file  ": " stdin ")<<key.first<<" at page " << key.second.first << "\n";
 		if(!is_in_cache(key)) 
 		{
-#ifdef FIM_CACHE_DEBUG
-			std::cout << "not in the cache: "<< key.first << " \n";
-#endif /* FIM_CACHE_DEBUG */
-			image = loadNewImage(key,page,false);
+			if(FIM_ALLOW_CACHE_DEBUG)
+				std::cout << FIM_CNS_DBG_CMDS_PFX << "cache does not have    " << key << "\n";
+			image = loadNewImage(key,false);
 			if(!image)
 				goto ret; // bad luck!
 			if(!image->cacheable())
@@ -566,48 +532,43 @@ ret:
 			image = getCachedImage(key);// in this way we update the LRU cache :)
 			if(!image)
 			{
-#ifdef FIM_CACHE_DEBUG
-				cout << "critical internal cache error!\n";
-#endif /* FIM_CACHE_DEBUG */
+				if(FIM_ALLOW_CACHE_DEBUG)
+					std::cout << FIM_CNS_DBG_CMDS_PFX << "critical internal cache error!\n";
 				goto done;
 			}
 			if( used_image( key ) )
 			{
 				// if the image was already used, cloning occurs
-//				image = image->getClone(); // EVIL !!
 				try
 				{
-#ifdef FIM_CACHE_DEBUG
-					Image * oi=image;
-#endif /* FIM_CACHE_DEBUG */
-					image = new Image(*image); // cloning
-#ifdef FIM_CACHE_DEBUG
-					std::cout << "  cloned image: \"" <<fim_basename_of(image->getName())<< "\" "<< image << " from \""<<fim_basename_of(oi->getName()) <<"\" " << oi << "\n";
-#endif /* FIM_CACHE_DEBUG */
+					//ImagePtr oi=image;
+					image = ImagePtr ( new Image(*image) ); // cloning
+					if(FIM_ALLOW_CACHE_DEBUG)
+						std::cout << FIM_CNS_DBG_CMDS_PFX << "  cloned image: \"" << key << "\" "<< image << "\n";
+						//std::cout << "  cloned image: \"" <<fim_basename_of(image->getName())<< "\" "<< image << " from \""<<fim_basename_of(oi->getName()) <<"\" " << oi << "\n";
 				}
 				catch(FimException e)
 				{
 					FIM_PR('E');
 					/* we will survive :P */
-					image = NULL; /* we make sure no taint remains */
+					image = FIM_NULL; /* we make sure no taint remains */
 //					if( e != FIM_E_NO_IMAGE )throw FIM_E_TRAGIC;  /* hope this never occurs :P */
 				}
 				if(!image)
 					goto ret; //means that cloning failed.
 				clone_pool_.insert(image); // we have a clone
-				cloneUsageCounter_[image]=1;
+				//cloneUsageCounter_[image]=1;
 			}
-#if FIM_WANT_EXPERIMENTAL_MIPMAPS
-			if(getGlobalIntVariable(FIM_VID_WANT_MIPMAPS)>0)
-				if(!image->has_mm())
-					image->mm_make();
-#endif /* FIM_WANT_EXPERIMENTAL_MIPMAPS */
+#if FIM_WANT_MIPMAPS
+			if( getGlobalStringVariable(FIM_VID_CACHE_CONTROL).find('M') == 0 )
+				image->mm_make();
+#endif /* FIM_WANT_MIPMAPS */
 			lru_touch( key );
 			// if loading and eventual cloning succeeded, we count the image as used of course
 			usageCounter_[key]++;
 		}
 done:
-		setGlobalVariable(FIM_VID_CACHE_STATUS,getReport().c_str());
+		setGlobalVariable(FIM_VID_CACHE_STATUS,getReport());
 ret:
 		if(vsp && image)
 		{
@@ -618,11 +579,12 @@ ret:
 		return image;
 	}
 
-	Image * Cache::setAndCacheStdinCachedImage(Image * image)
+	ImagePtr Cache::setAndCacheStdinCachedImage(ImagePtr image)
 	{
-		/* FIXME : document me
+		/* Cache an image coming from stdin (that is, not reloadable).
 		 * */
-		cache_key_t key(FIM_STDIN_IMAGE_NAME,FIM_E_STDIN);
+		const fim_page_t page = FIM_CNS_FIRST_PAGE;
+		const cache_key_t key{FIM_STDIN_IMAGE_NAME,{page,FIM_E_STDIN}};
 		FIM_LOUD_CACHE_STUFF;
 		FIM_PR('*');
 
@@ -631,25 +593,20 @@ ret:
 		
 		try
 		{
-#ifdef FIM_CACHE_DEBUG
-			Image * oi=image;
-#endif /* FIM_CACHE_DEBUG */
-			image = new Image(*image); // cloning
+			image = ImagePtr ( new Image(*image) ); // cloning
 			if(image)
-			{
 				cacheNewImage( image );
-			}
 		}
 		catch(FimException e)
 		{
 			FIM_PR('E');
 			/* we will survive :P */
-			image = NULL; /* we make sure no taint remains */
+			image = FIM_NULL; /* we make sure no taint remains */
 //			if( e != FIM_E_NO_IMAGE )throw FIM_E_TRAGIC;  /* hope this never occurs :P */
 		}
 		if(!image)
 			goto ret; //means that cloning failed.
-		setGlobalVariable(FIM_VID_CACHE_STATUS,getReport().c_str());
+		setGlobalVariable(FIM_VID_CACHE_STATUS,getReport());
 ret:
 		return image;	//so, it could be a clone..
 	}
@@ -657,77 +614,62 @@ ret:
 	fim::string Cache::getReport(int type)const
 	{
 		/* TODO: rename to info(). */
-		fim::string cache_report;
+		std::ostringstream cache_report;
 
 		if(type == FIM_CR_CN || type == FIM_CR_CD)
 		{
 			fim_char_t buf[FIM_PRINTFNUM_BUFSIZE];
-			int mci = getGlobalIntVariable(FIM_VID_MAX_CACHED_IMAGES);
-			int mcm = getGlobalIntVariable(FIM_VID_MAX_CACHED_MEMORY);
+			const fim_int mci = getGlobalIntVariable(FIM_VID_MAX_CACHED_IMAGES);
+			fim_int mcm = getGlobalIntVariable(FIM_VID_MAX_CACHED_MEMORY);
 			mcm = mcm >= 0 ? mcm*FIM_CNS_CSU:0;
-			cache_report  = " ";
-			cache_report += "count:";
-			cache_report += fim::string(cached_elements());
-			cache_report += "/";
-			cache_report += fim::string(mci);
-			cache_report += " ";
-			cache_report += "occupation:";
-			fim_snprintf_XB(buf, sizeof(buf), img_byte_size());cache_report += buf;
-			cache_report += "/";
-			fim_snprintf_XB(buf, sizeof(buf), mcm);cache_report += buf;
-			cache_report += " ";
+			cache_report << " " << "count:" << cached_elements() << "/" << mci << " occupation:";
+			fim_snprintf_XB(buf, sizeof(buf), img_byte_size());
+			cache_report << buf << "/";
+			fim_snprintf_XB(buf, sizeof(buf), mcm);
+			cache_report << buf << " ";
 			for(ccachels_t::const_iterator ci=usageCounter_.begin();ci!=usageCounter_.end();++ci)
 			if(
 			  ( type == FIM_CR_CD && ( imageCache_.find(ci->first) != imageCache_.end() ) ) ||
 			  ( type == FIM_CR_CN && ( imageCache_.find(ci->first) != imageCache_.end()  && ci->second) )
 			  )
 			{
-				cache_report+= fim_basename_of((*ci).first.first.c_str());
-				cache_report+=":";
-				cache_report+=fim::string((*ci).second);
-				cache_report+=":";
+				cache_report << "f=" << fim_basename_of((*ci).first.first) << ":";
+				cache_report << "p=" << (*ci).first.second.first << ":";
+				cache_report << "t=" << (*ci).second << ":";
 				fim_snprintf_XB(buf, sizeof(buf), imageCache_.find(ci->first)->second->byte_size());
-				cache_report += buf;
-				cache_report+="@";
-				cache_report+=fim::string((size_t)last_used(ci->first));
-				cache_report+=" ";
+				cache_report << buf << "@" << last_used(ci->first) << " ";
 			}
-			cache_report += "\n";
+			cache_report << "\n";
 			goto ret;
 		}
-		cache_report = "cache contents : \n";
+		cache_report << "cache contents : \n";
 		FIM_LOUD_CACHE_STUFF;
 #if 0
 		cachels_t::const_iterator ci;
 		for( ci=imageCache_.begin();ci!=imageCache_.end();++ci)
 		{	
-			cache_report+=((*ci).first);
-			cache_report+=" ";
-			cache_report+=fim::string(usageCounter_[((*ci).first)]);
-			cache_report+="\n";
+			cache_report << ((*ci).first) << " " << fim::string(usageCounter_[((*ci).first)]) << "\n";
 		}
 #else
 		for(ccachels_t::const_iterator ci=usageCounter_.begin();ci!=usageCounter_.end();++ci)
 		{	
-			cache_report+=((*ci).first.first);
-			cache_report+=":";
-			cache_report+=fim::string((*ci).first.second);
-			cache_report+=" ,usage:";
-			cache_report+=fim::string((*ci).second);
-			cache_report+="\n";
+			cache_report<<((*ci).first.first) <<":" <<((*ci).first.second.first) <<":" <<((*ci).first.second.second) <<" ,usage:" <<((*ci).second) <<"\n";
 		}
-		cache_report += "clone pool contents : \n";
-		for(std::set< fim::Image* >::const_iterator  cpi=clone_pool_.begin();cpi!=clone_pool_.end();++cpi)
+		cache_report << "clone pool contents : \n";
+		for(std::set< fim::ImagePtr >::const_iterator  cpi=clone_pool_.begin();cpi!=clone_pool_.end();++cpi)
 		{	
-			cache_report+=fim_basename_of((*cpi)->getName());
-			cache_report+=" " ; 
-			cache_report+= string((int*)(*cpi)) ; 
-			cache_report+=",";
+			cache_report << fim_basename_of((*cpi)->getName()) <<" " 
+#if FIM_IMG_NAKED_PTRS
+				<< string((int*)(*cpi))
+#else /* FIM_IMG_NAKED_PTRS */
+				<< ((int*)((*cpi).get()))
+#endif /* FIM_IMG_NAKED_PTRS */
+				<<",";
 		}
-		cache_report+="\n";
+		cache_report << "\n";
 #endif
 ret:
-		return cache_report;
+		return cache_report.str();
 	}
 
 	Cache::~Cache(void)
@@ -737,10 +679,13 @@ ret:
 		for( ci=imageCache_.begin();ci!=imageCache_.end();++ci)
 			if(ci->second)
 			{
-#ifdef FIM_CACHE_DEBUG
-				std::cout << "about to free " << (ci->first.first) << "\n";
-#endif /* FIM_CACHE_DEBUG */
+				if(FIM_ALLOW_CACHE_DEBUG)
+					std::cout << FIM_CNS_DBG_CMDS_PFX << "cache erases also      " << (ci->first) << "\n";
+#if FIM_IMG_NAKED_PTRS
 				delete ci->second;
+#else /* FIM_IMG_NAKED_PTRS */
+				// TBD
+#endif /* FIM_IMG_NAKED_PTRS */
 			}
 		imageCache_.clear(); /* destroy Image objects */
 	}
@@ -767,8 +712,8 @@ ret:
 		bs += FIM_VCBS(viewportInfo_);
 		/* 
 		bs += sizeof(usageCounter_);
-		bs += sizeof(reverseCache_); */
-		/* TODO: incomplete ... */
+		TODO: incomplete ...
+		*/
 		return bs;
 	}
 
@@ -781,7 +726,6 @@ ret:
 
 	void Cache::desc_update(void)
 	{
-		/* TODO: report error code */
 #if FIM_WANT_PIC_CMTS
 		cachels_t::const_iterator ci;
 
@@ -791,5 +735,4 @@ ret:
 				ci->second->desc_update();
 #endif /* FIM_WANT_PIC_CMTS */
 	}
-}
-
+} /* namespace */
