@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2017-07-14 21:46:19 +0200 (Fri, 14 Jul 2017) $ */
+/* $LastChangedDate: 2024-03-30 12:54:10 +0100 (Sat, 30 Mar 2024) $ */
 /*
  CommandConsole-cmd.cpp : Fim console commands
 
- (c) 2009-2015 Michele Martone
+ (c) 2009-2024 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,40 +18,26 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
-
-
-
 #include "fim.h"
 #ifdef HAVE_LIBGEN_H
 #include <libgen.h>
 #endif /* HAVE_LIBGEN_H */
 
 #define FIM_WANT_SYSTEM_CALL_DEBUG 0
+#define FIM_CMD_RET_HELP_MSG(CMD) "See " FIM_FLT_HELP " " CMD " for its invocation.\n"
 
 namespace fim
 {
-	fim::string CommandConsole::fcmd_bind(const args_t& args)
+	fim_key_t CommandConsole::kstr_to_key(const fim_char_t * kstr)const
 	{
-		/*
-		 *	this is the interactive bind command
-		 *	the user supplies a string with the key combination, and if valid, its keycode
-		 *	is associated to the user supplied action (be it a command, alias, etc..)
-		 *	FIX ME
-		 */
-		const fim_char_t *kerr=FIM_FLT_BIND" : invalid key argument (should be one of : k, C-k, K, <Left..> }\n";
-		const fim_char_t*kstr=FIM_NULL;
 		int l;
-		fim_key_t key=FIM_SYM_NULL_KEY;
+		fim_key_t key = FIM_SYM_NULL_KEY;
 
-		if(args.size()==0)
-			return getBindingsList();
-
-		kstr=(args[0].c_str());
 		if(!kstr)
-			return kerr;
+			goto err;
 		l=strlen(kstr);
 		if(!l)
-			return kerr;
+			goto err;
 
 #ifdef FIM_WANT_RAW_KEYS_BINDING
 		if(l>=2 && isdigit(kstr[0]) && isdigit(kstr[1]))
@@ -63,8 +49,43 @@ namespace fim
 		else
 		{
 			if(sym_keys_.find(kstr)!= sym_keys_.end())
-				key=sym_keys_[args[0]];
+				key=sym_keys_.find(fim::string(kstr))->second;
 		}
+err:
+		return key;
+	}
+
+	fim_cxr CommandConsole::fcmd_bind(const args_t& args)
+	{
+		/*
+		 *	this is the interactive bind command
+		 *	the user supplies a string with the key combination, and if valid, its keycode
+		 *	is associated to the user supplied action (be it a command, alias, etc..)
+		 */
+		const fim_char_t* kerr=FIM_FLT_BIND ": invalid key argument (should be one of: k, C-k, K, <Left..> }\n";
+		const fim_char_t* kstr=FIM_NULL;
+		fim_key_t key=FIM_SYM_NULL_KEY;
+
+		if(args.size()==0)
+			return getBindingsList();
+
+		kstr=(args[0].c_str());
+		if(!kstr || 0 == strlen(kstr))
+		{
+#if FIM_WANT_BIND_ALL
+			if(args.size()>1)
+			{
+				for (auto bi=bindings_.begin(); bi != bindings_.end(); ++bi)
+					bindings_[bi->first]=args[1];
+				return "Re-bound all keys to " + args[1];
+			}
+			else
+#endif
+			return kerr;
+		}
+
+		key = kstr_to_key(args[0].c_str());
+
 		if(key==FIM_SYM_NULL_KEY)
 		{
 			return "supplied key "+args[0]+" is invalid.\n";
@@ -77,38 +98,34 @@ namespace fim
 		if(args.size()==1)
 		{
 			//first arg should be a valid key code
-			fim::string binding_expanded;
-			binding_expanded+=FIM_FLT_BIND" '";
-			binding_expanded+=args[0];
-			binding_expanded+="' '";
-			binding_expanded+=bindings_[key];
-			binding_expanded+="'\n";
-			return binding_expanded;
+			std::ostringstream oss;
+			oss << FIM_FLT_BIND " " << fim_auto_quote(args[0],3) << " " << fim_auto_quote(bindings_[key],3) << "\n";
+			return oss.str();
 		}
 		if(args.size()<2)
 		       	return kerr;
 		if(args[1]==FIM_CNS_EMPTY_STRING)
-		       	return unbind(args[0]);
-
+		       	return fcmd_unbind({args[0]});
 		if(args.size()>=3)
-			bindings_help_[key]=args[2]; /* TODO: shall move to bind() */
-		return bind(key,args[1]);
+			return bind(key,args[1],args[2]);
+		else
+			return bind(key,args[1]);
 	}
 
-	fim::string CommandConsole::fcmd_unbind(const args_t& args)
+	fim_cxr CommandConsole::fcmd_unbind(const args_t& args)
 	{
 		/*
-		 * 	unbinds the action eventually bound to the first key name specified in args..
-		 *	IDEAS : multiple unbindings ?
-		 *	maybe you should made surjective the binding_keys mapping..
+		 * 	unbinds the action possibly bound to the first key name specified in args..
+		 * 	what about multiple unbindings ?
+		 *	maybe you should make the binding_keys mapping become surjective...
 		 */
 		if(args.size()!=1)
-			return FIM_FLT_UNBIND" : specify the key to unbind\n";
+			return FIM_FLT_UNBIND ": specify the key to unbind\n";
 		return unbind(args[0]);
 	}
 
 #if 0
-	fim::string CommandConsole::fcmd_setenv(const args_t& args)
+	fim_cxr CommandConsole::fcmd_setenv(const args_t& args)
 	{
 		/*
 		 *	
@@ -120,128 +137,32 @@ namespace fim
 	}
 #endif
 
-	fim::string CommandConsole::fcmd_help(const args_t &args)
+	fim_cxr CommandConsole::fcmd_help(const args_t& args)
 	{	
-		Command *cmd = FIM_NULL;
+		fim_cxr retval;
 
-		if(!args.empty() && args[0].length()>0 )
+		if( !args.empty() )
 		{
-			if((args[0].at(0))==FIM_CNS_SLASH_CHAR)
+			const auto a0l = args[0].length();
+			if( 0 == a0l ) // help '' # special 'hint' form, which I may perhaps better move into FIM_ACM_PREEXECUTIONCYCLE, which occurs after the config...
 			{
-				/* FIXME: this shall be regexp-based. */
-				fim::string hstr,cstr,astr,bstr,ptn;
-
-				ptn = args[0].substr(1,args[0].size());
-				for(size_t i=0;i<commands_.size();++i) 
-					if(commands_[i] && commands_[i]->help_.find(ptn) != commands_[i]->help_.npos)
-					{
-						cstr += commands_[i]->cmd_;
-						cstr += " ";
-					}
-
-				for( aliases_t::const_iterator ai=aliases_.begin();ai!=aliases_.end();++ai)
-				{
-					if(ai->first.find(ptn) != ai->first.npos)
-					{	
-						astr +=((*ai).first);
-						astr += " ";
-					}
-					else
-					if(ai->second.first.find(ptn) != ai->second.first.npos)
-					{	
-						astr +=((*ai).first);
-						astr += " ";
-					}
-					else
-					if(ai->second.second.find(ptn) != ai->second.second.npos)
-					{	
-						astr +=((*ai).first);
-						astr += " ";
-					}
-				}
-
-				for(bindings_t::const_iterator  bi=bindings_.begin();bi!=bindings_.end();++bi)
-				{
-					key_syms_t::const_iterator ikbi=key_syms_.find(((*bi).first));
-					bindings_help_t::const_iterator  bhi=bindings_help_.find((*bi).first);
-
-					if(ikbi!=key_syms_.end())
-					{
-						if(ikbi->second.find(ptn) != ikbi->second.npos)
-						{
-							bstr += ikbi->second;
-						       	bstr += " ";
-							// std::cout << "key : " << ikbi->second << "\n";
-						}
-						else
-						if(bi->second.find(ptn) != bi->second.npos)
-						{
-							bstr += ikbi->second;
-						       	bstr += " ";
-							// std::cout << "def : " << ikbi->second << "\n";
-						}
-						if(bhi != bindings_help_.end() )
-						if(bhi->second.find(ptn) != bhi->second.npos)
-						{
-							bstr += ikbi->second;
-						       	bstr += " ";
-							// std::cout << "dsc : " << ikbi->second << "\n";
-						}
-					}
-				}
-				/* FIXME: may extend to autocmd's */
-
-				if(!cstr.empty())
-					hstr+="Commands (type 'help' and one quoted command name to get more info):\n " + cstr + "\n";
-				if(!astr.empty())
-					hstr+="Aliases (type 'alias' and one quoted alias to get more info):\n " + astr + "\n";
-				if(!bstr.empty())
-					hstr+="Bindings (type 'bind' and one quoted binding to get more info):\n " + bstr + "\n";
-				if(!hstr.empty())
-					return "The following help items matched \"" + ptn + "\":\n" + hstr;
+				if(FIM_CNS_INVALID_KEY_VAL!=getIntVariable(FIM_VID_CONSOLE_KEY))
+					retval = "WELCOME: to switch to the command line interface press ':'.\n";
 				else
-					return "No item matched \"" + ptn + "\"\n";
+					retval = "WELCOME: the command line is disabled (and therefore access to online help).\n";
+				return retval; // no FIM_VID_DISPLAY_CONSOLE setting
 			}
-
-		{
-			fim::string sws;
-			sws = fim_help_opt(args[0].c_str());
-			if(sws != FIM_CNS_EMPTY_STRING)
-				return sws+string("\n");
+			if( 0 < a0l )
+				retval = get_help(args[0],'l');
 		}
-			cmd=findCommand(args[0]);
-			if(cmd)
-				return
-					string("\"")+(args[0]+string("\" is a command, documented:\n"))+
-				      	cmd->getHelp()+string("\n");
-			else
-			if(aliasRecall(fim::string(args[0]))!=FIM_CNS_EMPTY_STRING)
-				return
-					string("\"")+(args[0]+string("\" is an alias, and was declared as:\n"))+
-					get_alias_info(args[0]);
-			else
-			{
-				if(isVariable(args[0]))
-				{
-					fim::string hs;
-					hs+=fim::string("\"");
-					hs+=args[0] + fim::string( "\" is a variable, with value:\n" );
-					hs+=getStringVariable(args[0]);
-					hs+=fim::string("\nand description:\n");
-					hs+=fim_var_help_db_query(args[0]);
-					hs+=fim::string("\n");
-					return hs;
-				}
-				else
-					cout << args[0] << " : no such command, alias, or variable.\n";
-			}
+		this->setVariable(FIM_VID_DISPLAY_CONSOLE,1);
 
-		}
-		this->setVariable(FIM_VID_DISPLAY_CONSOLE,(fim_int)1);
-		return "" FIM_FLT_HELP " " FIM_CNS_EX_ID_STRING ": provides help for " FIM_CNS_EX_ID_STRING ", if it is a variable, alias, or command. Use " FIM_KBD_TAB " in commandline mode to get a list of commands. Command line mode can be entered with the default key '" FIM_SYM_CONSOLE_KEY_STR "', and left pressing " FIM_KBD_ENTER ".\n";
+		if(retval.empty())
+			retval = "" FIM_FLT_HELP " " FIM_CNS_EX_ID_STRING ": provides help for " FIM_CNS_EX_ID_STRING ", if it is a variable, alias, key, or command. Use " FIM_KBD_TAB " in commandline mode to get a list of aliases and commands. When command line mode is available, the default key to access it is '" FIM_SYM_CONSOLE_KEY_STR "', and to leave it is " FIM_KBD_ENTER ".\n";
+		return fim_man_to_text(retval, true);
 	}
 
-	fim::string CommandConsole::fcmd_quit(const args_t &args)
+	fim_cxr CommandConsole::fcmd_quit(const args_t& args)
 	{
 		/*
 		 * now the postcycle execution autocommands are enabled !
@@ -255,18 +176,16 @@ namespace fim
 	}
 
 #ifndef FIM_WANT_NOSCRIPTING
-	fim::string CommandConsole::fcmd_executeFile(const args_t &args)
+	fim_cxr CommandConsole::fcmd_executeFile(const args_t& args)
 	{
-		/*
-		 * TODO : catch exceptions
-		 * */
 		for(size_t i=0;i<args.size();++i)
-			executeFile(args[i].c_str());
+			if( executeFile(args[i].c_str()) != FIM_ERR_NO_ERROR )
+				; // might be verbose ...
 		return FIM_CNS_EMPTY_RESULT;
 	}
 #endif /* FIM_WANT_NOSCRIPTING */
 	
-	fim::string CommandConsole::fcmd_foo(const args_t &args)
+	fim_cxr CommandConsole::fcmd_foo(const args_t& args)
 	{
 		/*
 		 * useful function for bogus commands, but autocompletable (like language constructs)
@@ -274,25 +193,12 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-#if 0
-	fim::string CommandConsole::get_expr_type(const args_t &args);
-	{
-		/*
-		 * a command to echo arguments types, for debug and learning purposes
-		 */
-		if(args.size()==0)fim::cout<<"type command\n";
-		for(size_t i=0;i<args.size();++i)fim::cout << (args[i].c_str()) << "\n";
-		return FIM_CNS_EMPTY_RESULT;
-
-	}
-#endif
-
-	fim::string CommandConsole::fcmd_echo(const args_t &args)
+	fim_cxr CommandConsole::fcmd_echo(const args_t& args)
 	{
 		return do_echo(args);
 	}
 
-	fim::string CommandConsole::do_echo(const args_t &args)const
+	fim::string CommandConsole::do_echo(const args_t& args)const
 	{
 		/*
 		 * a command to echo arguments, for debug and learning purposes
@@ -304,20 +210,32 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd__stdout(const args_t &args)
+	fim_cxr CommandConsole::fcmd__stderr(const args_t& args)
 	{
 		/*
 		 * a command to echo to stdout arguments, for debug and learning purposes
 		 */
 		if(args.size()==0)
-			std::cout<<"echo command\n";
+			std::cerr<<"echo command\n";
+		for(size_t i=0;i<args.size();++i)
+			std::cerr << (args[i].c_str()) << "\n";
+		return FIM_CNS_EMPTY_RESULT;
+	}
+
+	fim_cxr CommandConsole::fcmd__stdout(const args_t& args)
+	{
+		/*
+		 * a command to echo to stdout arguments, for debug and learning purposes
+		 */
+		if(args.size()==0)
+			std::cout<<"stdout command\n";
 		for(size_t i=0;i<args.size();++i)
 			std::cout << (args[i].c_str()) << "\n";
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
 #ifdef FIM_AUTOCMDS
-	fim::string CommandConsole::fcmd_autocmd(const args_t& args)
+	fim_cxr CommandConsole::fcmd_autocmd(const args_t& args)
 	{
 		/*
 		 * associates an action to a certain event in certain circumstances
@@ -330,12 +248,12 @@ namespace fim
 		}
 		if(args.size()==1)
 		{
-			/* autocmd Event : should list all autocmds for the given Event */
+			/* autocmd Event: list all autocmds for Event */
 			return autocmds_list(args[0],"");
 		}
 		if(args.size()==2)
 		{
-			/* autocmd Event Pattern : should list all autocmds for the given Event Pattern */
+			/* autocmd Event Pattern: list all autocmds for Event Pattern */
 			return autocmds_list(args[0],args[1]);
 		}
 		if(args.size()==3)
@@ -345,7 +263,7 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_autocmd_del(const args_t& args)
+	fim_cxr CommandConsole::fcmd_autocmd_del(const args_t& args)
 	{
 		/*
 		 * deletes one or more autocommands
@@ -357,12 +275,12 @@ namespace fim
 		}
 		if(args.size()==1)
 		{
-			/* autocmd Event : should delete all autocmds for the given Event */
+			/* autocmd Event: delete all autocmds for Event */
 			return autocmd_del(args[0],"","");
 		}
 		if(args.size()==2)
 		{
-			/* autocmd Event Pattern : should delete all autocmds for the given Event Pattern */
+			/* autocmd Event Pattern: delete all autocmds for Event Pattern */
 			return autocmd_del(args[0],args[1],"");
 		}
 		if(args.size()==3)
@@ -373,7 +291,7 @@ namespace fim
 	}
 #endif /* FIM_AUTOCMDS */
 
-	fim::string CommandConsole::fcmd_set_in_console(const args_t& args)
+	fim_cxr CommandConsole::fcmd_set_in_console(const args_t& args)
 	{
 		/*
 		 * EXPERIMENTAL !!
@@ -384,7 +302,7 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_set_interactive_mode(const args_t& args)
+	fim_cxr CommandConsole::fcmd_set_interactive_mode(const args_t& args)
 	{
 #ifdef FIM_USE_READLINE
 		ic_=-1;set_status_bar("",FIM_NULL);
@@ -395,11 +313,12 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_sys_popen(const args_t& args)
+	fim_cxr CommandConsole::fcmd_sys_popen(const args_t& args)
 	{
 		/*
 		 *
 		 * */
+#if FIM_WANT_POPEN_CALLS
 		for(size_t i=0;i<args.size();++i)
 		{
 			FILE* fd=popen(args[i].c_str(),"r");
@@ -411,14 +330,15 @@ namespace fim
 			executeStdFileDescriptor(fd);
 			pclose(fd);
 		}
+#endif /* FIM_WANT_POPEN_CALLS */
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
 	fim_err_t CommandConsole::fpush(FILE *tfd)
 	{
-			/* todo : read errno in case of error and print some report.. */
+			/* todo: read errno in case of error and print some report.. */
 	
-			/* pipes are not seekable : this breaks down all the Fim file handling mechanism */
+			/* pipes are not seekable: this is incompatible with Fim's file handling mechanism */
 			/*
 			while( (rc=read(0,buf,FIM_PIPE_BUFSIZE))>0 ) fwrite(buf,rc,1,tfd);
 			rewind(tfd);
@@ -428,19 +348,16 @@ namespace fim
 			 * but it would require to rewrite much of the file loading stuff
 			 * (which is quite fbi's untouched stuff right now)
 			 * */
-			Image* stream_image=FIM_NULL;
+			ImagePtr stream_image=FIM_NULL;
 			if(!tfd)
 				return FIM_ERR_GENERIC;
-			try{ stream_image=new Image(FIM_STDIN_IMAGE_NAME,fim_fread_tmpfile(tfd)); }
+			try{ stream_image=ImagePtr ( new Image(FIM_STDIN_IMAGE_NAME,fim_fread_tmpfile(tfd)) ); }
 			catch (FimException e){/* write me */}
 #ifdef FIM_READ_STDIN_IMAGE
-			// DANGEROUS TRICK!
 			if(stream_image)
 			{
-				browser_.set_default_image(stream_image);
-					if(!cc.browser_.cache_.setAndCacheStdinCachedImage(stream_image))
-						std::cerr << FIM_EMSG_CACHING_STDIN;// FIXME
-				browser_.push(FIM_STDIN_IMAGE_NAME);
+				browser_.set_default_image(ImagePtr(std::move(stream_image)));
+				browser_.push_path(FIM_STDIN_IMAGE_NAME);
 			}
 #else /* FIM_READ_STDIN_IMAGE */
 			/* FIXME: this point should be better not reached */
@@ -449,13 +366,13 @@ namespace fim
 			//pclose(tfd);
 	}
 
-#ifdef FIM_PIPE_IMAGE_READ
 	/*
 	 * FBI/FIM FILE PROBING MECHANISMS ARE NOT THOUGHT WITH PIPES IN MIND!
 	 * THEREFORE WE MUST FIND A SMARTER TRICK TO IMPLEMENT THIS
 	 * */
-	fim::string CommandConsole::fcmd_pread(const args_t& args)
+	fim_cxr CommandConsole::fcmd_pread(const args_t& args)
 	{
+#if FIM_PIPE_IMAGE_READ
 		/*
 		 * we read a whole image file from pipe
 		 * */
@@ -473,87 +390,52 @@ namespace fim
 			 * error handling
 			 * */
 		}
+#endif /* FIM_PIPE_IMAGE_READ */
 		return FIM_CNS_EMPTY_RESULT;
 	}
-#endif /* FIM_PIPE_IMAGE_READ */
 
-	fim::string CommandConsole::fcmd_cd(const args_t& args)
+	fim_cxr CommandConsole::fcmd_cd(const args_t& args)
 	{
 		/*
 		 * change working directory
 		 * */
-		static fim::string oldpwd=fcmd_pwd(args_t());
 		for(size_t i=0;i<args.size();++i)
 		{
 			fim::string dir=args[i];
-			if(dir=="-")dir=oldpwd;
-			oldpwd=fcmd_pwd(args_t());
+			if(dir=="-")
+				dir=oldcwd_;
+			oldcwd_=fim_getcwd();
 #ifdef HAVE_LIBGEN_H
 			if(!is_dir(dir))
-			{
 				dir=fim_dirname(dir);
-			}
 #endif /* HAVE_LIBGEN_H */
-			int ret = chdir(dir.c_str());
-#if 1
-			if(ret)
-			       	return (fim::string("cd error : ")+fim::string(strerror(errno)));
-#else
-			// deprecated
-			if(ret)
-			       	return (fim::string("cd error : ")+fim::string(sys_errlist[errno]));
-#endif
+			if( chdir(dir.c_str()) )
+			       	return (fim::string("cd error: ")+fim::string(strerror(errno)));
 		}
-		setVariable(FIM_VID_PWD,fcmd_pwd(args_t()).c_str());
+		setVariable(FIM_VID_PWD,fim_getcwd());
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_basename(const args_t& args)
+	fim_cxr CommandConsole::fcmd_basename(const args_t& args)
 	{
-		/*
-		 * */
-		static fim::string res="";
+		fim::string res;
 		for(size_t i=0;i<args.size();++i)
-		{
-			fim::string arg=args[i];
-			res+=basename((fim_char_t*)arg.c_str());//FIXME
-		}
-		setVariable(FIM_VID_LAST_SYSTEM_OUTPUT,res.c_str());
+			res+=fim_basename_of(args[i]);
 		return res;
 	}
 
-	fim::string CommandConsole::fcmd_pwd(const args_t& args)
+	fim_cxr CommandConsole::fcmd_pwd(const args_t& args)
 	{
 		/*
 		 * print working directory
 		 * */
-		fim::string cwd=FIM_CNS_EMPTY_STRING;
-#if HAVE_GET_CURRENT_DIR_NAME
-		/* default */
-		fim_char_t *p=get_current_dir_name();
-		if(p)
-			cwd=p;
-		else
-		       	cwd=FIM_CNS_EMPTY_STRING;
-		if(p)
-			fim_free(p);
-#else /* HAVE_GET_CURRENT_DIR_NAME */
-#if _BSD_SOURCE || _XOPEN_SOURCE >= 500
-		{
-			/* untested */
-			fim_char_t *buf[PATH_MAX];
-			getcwd(buf,PATH_MAX-1): 
-			buf[PATH_MAX-1]=FIM_SYM_CHAR_NUL;
-			cwd=buf;
-		}
-#endif /* _BSD_SOURCE || _XOPEN_SOURCE >= 500 */
-#endif /* HAVE_GET_CURRENT_DIR_NAME */
+		fim::string cwd = fim_getcwd();
 		return cwd;
 	}
 
-#ifndef FIM_NO_SYSTEM
-	fim::string CommandConsole::fcmd_system(const args_t& args)
+	fim_cxr CommandConsole::fcmd_system(const args_t& args)
 	{
+#if FIM_WANT_SYSTEM_CALLS
 		/*
 		 * executes the shell commands given in the arguments,
 		 * one by one, and returns the (collated) standard output
@@ -563,17 +445,17 @@ namespace fim
 #endif /* FIM_WANT_SYSTEM_CALL_DEBUG */
 #if FIM_WANT_SINGLE_SYSTEM_INVOCATION
 		/* 20110302 FIXME: inefficient */
-		fim::string cc=FIM_CNS_EMPTY_STRING;
+		fim::string scs=FIM_CNS_EMPTY_STRING;
 		for(size_t i=0;i<args.size();++i)
 		{
 			// FIXME: escaping the command (first argument) actually requires more than this
 #define FIM_WANT_SIMPLE_SHELL_ESCAPING 1
 #if FIM_WANT_SIMPLE_SHELL_ESCAPING
-			cc+=fim_shell_arg_escape(args[i]);
+			scs+=fim_shell_arg_escape(args[i]);
 #else /* FIM_WANT_SIMPLE_SHELL_ESCAPING */
-			cc+=args[i];
+			scs+=args[i];
 #endif /* FIM_WANT_SIMPLE_SHELL_ESCAPING */
-			cc+=" ";
+			scs+=" ";
 #if FIM_WANT_SYSTEM_CALL_DEBUG
 			is+=args[i];
 			is+=" ";
@@ -581,13 +463,13 @@ namespace fim
 		}
 #if FIM_WANT_SYSTEM_CALL_DEBUG
 		std::cerr << "received string: " << is << FIM_SYM_ENDL;	
-		std::cerr << "about to call popen on string: " << cc << FIM_SYM_ENDL;	
+		std::cerr << "about to call popen on string: " << scs << FIM_SYM_ENDL;	
 #endif /* FIM_WANT_SYSTEM_CALL_DEBUG */
 		if(args.size())
 		{
-			FILE* fd=popen(cc.c_str(),"r");
+			FILE* fd=popen(scs.c_str(),"r");
 		//	cout << readStdFileDescriptor(fd);
-			setVariable(FIM_VID_LAST_SYSTEM_OUTPUT,readStdFileDescriptor(fd).c_str());
+			setVariable(FIM_VID_LAST_SYSTEM_OUTPUT,readStdFileDescriptor(fd));
 			cout << getStringVariable(FIM_VID_LAST_SYSTEM_OUTPUT);
 		       	pclose(fd);
 		}
@@ -601,7 +483,7 @@ namespace fim
 			 * int fd=(int)popen("/bin/echo quit","r");
 			 */
 			//cout << readStdFileDescriptor(fd);
-			setVariable(FIM_VID_LAST_SYSTEM_OUTPUT,readStdFileDescriptor(fd).c_str());
+			setVariable(FIM_VID_LAST_SYSTEM_OUTPUT,readStdFileDescriptor(fd));
 			pclose(fd);
 		}
 #endif /* FIM_WANT_SINGLE_SYSTEM_INVOCATION */
@@ -611,11 +493,11 @@ namespace fim
 			std::system(args[i].c_str());
 		}
 #endif
+#endif /* FIM_WANT_SYSTEM_CALLS */
 		return FIM_CNS_EMPTY_RESULT;
 	}
-#endif /* FIM_NO_SYSTEM */
 	
-	fim::string CommandConsole::fcmd_status(const args_t &args)
+	fim_cxr CommandConsole::fcmd_status(const args_t& args)
 	{
 		/*
 		 * the status bar is updated with the given arguments collated.
@@ -627,17 +509,17 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_unalias(const args_t& args)
+	fim_cxr CommandConsole::fcmd_unalias(const args_t& args)
 	{
 		/*
 		 * removes the actions assigned to the specified aliases,
 		 */
 		if(args.size()<1)
-			return FIM_FLT_UNALIAS" : please specify an alias to remove or all (-a)!\n";
+			return FIM_FLT_UNALIAS ": please specify an alias to remove or all (-a)!\n";
 
 		if(args[0]==string("-a"))
 		{
-			/* FIXME : the lexer/parser is bugged and it takes -a as an expression if not between double quotes ("-a") */
+			/* the lexer/parser has limitation and it takes -a as a numerical expression if not between double quotes ("-a") */
 			aliases_.clear();
 			return FIM_CNS_EMPTY_RESULT;
 		}
@@ -647,79 +529,80 @@ namespace fim
 		{
 			aliases_.erase(args[i]);
 			return FIM_CNS_EMPTY_RESULT;
-			/* fim::string(FIM_FLT_UNALIAS" : \"")+args[i]+fim::string("\" successfully unaliased.\n"); */
+			/* fim::string(FIM_FLT_UNALIAS ": \"")+args[i]+fim::string("\" successfully unaliased.\n"); */
 		}
 		else
-		       	return fim::string(FIM_FLT_UNALIAS" : \"")+args[i]+fim::string("\" there is not such alias.\n");
+		       	return fim::string(FIM_FLT_UNALIAS ": \"")+args[i]+fim::string("\" there is not such alias.\n");
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_dump_key_codes(const args_t& args)
+	fim_cxr CommandConsole::fcmd_dump_key_codes(const args_t& args)
 	{
-		return do_dump_key_codes(args);
+		return do_dump_key_codes(DefRefMode);
 	}
 
-	fim::string CommandConsole::do_dump_key_codes(const args_t& args)const
+	fim::string CommandConsole::do_dump_key_codes(FimDocRefMode refmode)const
 	{
 		/*
-		 * all keyboard codes are dumped in the console.
+		 * dump all keysyms
 		 * */
-		fim::string acl;
+		std::ostringstream oss;
 		sym_keys_t::const_iterator ki;
-		acl+="There are ";
-		acl+=fim::string((int)(sym_keys_.size()));
-		acl+=" bindings (dumping them here, unescaped).\n";
+
+		if ( refmode == DefRefMode )
+			oss<<"There are " << fim::string((int)(sym_keys_.size())) << " bindings (dumping them here, unescaped).\n";
+
 		for( ki=sym_keys_.begin();ki!=sym_keys_.end();++ki)
 		{
-			acl+=((*ki).first);
-			acl+=" -> ";
-			acl+=fim::string((int)(((*ki).second)));
-			acl+="  ";
+			if ( refmode == DefRefMode )
+				oss << ((*ki).first);
+			else
+			{
+				if( ((*ki).first)=="\\")
+					oss << "\"\\\\\"";
+				else
+					oss << fim_auto_quote(ki->first,0);
+			}
+			if ( refmode == DefRefMode )
+				oss << " -> " << fim::string((int)(((*ki).second)));
+			oss << "  ";
 		}
-		acl+="\n";
-		return acl;
+		oss << "\n";
+		return oss.str();
 	}
 
 #ifdef FIM_RECORDING
-	fim::string CommandConsole::dump_record_buffer(const args_t &args)
+	fim::string CommandConsole::dump_record_buffer(const args_t& args)
 	{
-		return do_dump_record_buffer(args);
+		return do_dump_record_buffer(args,false);
 	}
 
-	fim::string CommandConsole::do_dump_record_buffer(const args_t &args)const
+	fim::string CommandConsole::do_dump_record_buffer(const args_t& args, bool headeronly)const
 	{
-		/*
-		 * the recorded commands are dumped in the console
-		 * */
-		fim::string res;
+		std::ostringstream oss;
+
+		if(headeronly)
+			oss << recorded_actions_.size() << " commands.\n";
+		else
 		for(size_t i=0;i<recorded_actions_.size();++i)
-		{
-			fim::string ss=(int)recorded_actions_[i].second;
-			/*
-			 * FIXME : fim::string+=<int> is bugful
-			 * */
-			res+=FIM_FLT_USLEEP" '";
-//			res+=(int)recorded_actions_[i].second;
-			res+=ss;
-			res+="';\n";
-			res+=recorded_actions_[i].first;
-			res+="\n";
-		}
-		return res;
+			oss << FIM_FLT_USLEEP " '" << recorded_actions_[i].second << "';\n"
+			       	<< recorded_actions_[i].first << "\n";
+		return oss.str();
 	}
 
-	fim::string CommandConsole::execute_record_buffer(const args_t &args)
+	fim::string CommandConsole::execute_record_buffer(const args_t& args)
 	{
 		/*
 		 * all of the commands in the record buffer are re-executed.
 		 * */
 		if(recordMode_==Normal)
 		{
+			cout << "Will execute " << do_dump_record_buffer(args_t(),true);
 			recordMode_=Playing;
 			execute_internal(dump_record_buffer(args).c_str(),FIM_X_NULL);
 			recordMode_=Normal;
 		}
-		/* for unknown reasons, the following code gives problems : image resizes don't work..
+		/* for unknown reasons, the following code gives problems: image resizes don't work..
 		 * but the present (above) doesn't support interruptions ...
 		 * */
 /*		fim::string res;
@@ -731,7 +614,7 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_eval(const args_t &args)
+	fim_cxr CommandConsole::fcmd_eval(const args_t& args)
 	{
 		/*
 		 * all of the commands given as arguments are executed.
@@ -745,10 +628,10 @@ namespace fim
 	}
 #endif /* FIM_RECORDING */
 
-	fim::string CommandConsole::repeat_last(const args_t &args)
+	fim::string CommandConsole::repeat_last(const args_t& args)
 	{
 		/*
-		 * WARNING : there is an intricacy concerning the semantics of this command :
+		 * WARNING: there are a few points concerning this command:
 		 * - This command should NOT be registered as last_command, nor any alias 
 		 *   triggering it. But this solution would require heavy parsing and very
 		 *   complicated machinery and information propagation... 
@@ -764,11 +647,16 @@ namespace fim
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_recording(const args_t &args)
+	fim_cxr CommandConsole::fcmd_recording(const args_t& args)
 	{
 		if(args.size()<1)
 		{
-			goto nop;
+			if(recordMode_==Normal)
+				return "Neither recording nor re-executing. Specify 'start' to start recording commands and then 'stop' to terminate, and 'dump' to see recorded commands.\n";
+			if(recordMode_==Recording)
+				return "Recording commands. Specify 'stop' to stop recording and then 'play' to execute them.\n";
+			if(recordMode_==Playing)
+				return "Executing recorded commands.\n";
 		}
 		else
 		{
@@ -777,70 +665,58 @@ namespace fim
 			if(args[0]=="stop")
 				return stop_recording();
 			if(args[0]=="dump")
-			{
-				args_t argsc(args);
-				argsc.erase(argsc.begin());
-				return dump_record_buffer(argsc);
-			}
-			if(args[0]=="execute")
-			{
-				args_t argsc(args);
-				argsc.erase(argsc.begin());
-				return execute_record_buffer(argsc);
-			}
+				return dump_record_buffer(args_t(args.begin()+1,args.end()));
+			if(args[0]=="execute" || args[0]=="play")
+				return execute_record_buffer(args_t(args.begin()+1,args.end()));
 			if(args[0]=="repeat_last")
-			{
-				args_t argsc(args);
-				argsc.erase(argsc.begin());
-				return repeat_last(argsc);
-			}
+				return repeat_last(args_t(args.begin()+1,args.end()));
 		}
-nop:
-		return FIM_CNS_EMPTY_RESULT;
+		return FIM_CMD_RET_HELP_MSG(FIM_FLT_RECORDING);
 	}
 
 	fim::string CommandConsole::start_recording(void)
 	{
-		/*
-		 * recording of commands starts here
-		 * */
+		std::ostringstream oss;
+
 		if(recordMode_==Normal)
 		{
 			recorded_actions_.clear();
 			recordMode_=Recording;
-			record_action("");
+			record_action(std::string()); // initialization
+			oss << "Starting recording.\n";
 		}
-		return FIM_CNS_EMPTY_RESULT;
+		return oss.str();
 	}
 
 	fim::string CommandConsole::stop_recording(void)
 	{
-		/*
-		 * since the last recorded action was stop_recording, we pop out the last command
-		 */
+		std::ostringstream oss;
+
 		if(recordMode_==Recording)
+		{
+			oss << "Recorded " << do_dump_record_buffer(args_t(),true);
 			recordMode_=Normal;
-		return FIM_CNS_EMPTY_RESULT;
+		}
+		return oss.str();
 	}
 
-	fim::string CommandConsole::fcmd_set(const args_t &args)
+	fim_cxr CommandConsole::fcmd_set(const args_t& args)
 	{
 		/*
 		 * with no arguments, prints out the variable names.
 		 * with one identifier as argument, prints out its value.
-		 * with two arguments, sets the first argument's value.
-		 *
-		 * NOTE : THIS IS NOT EXACTLY VIM'S BEHAVIOUR (FIXME)
+		 * with two arguments, sets the first argument's value and prints nothing.
+		 * otherwise, print help
 		 * */
 		if( ! args.size())
 			return get_variables_list();
 		if(1==args.size())
 			return getStringVariable(args[0]);
-		/*
-		 * warning!
-		 * */
 		if(2==args.size())
-			return setVariable(args[0],args [1].c_str());
+		{
+			setVariable(args[0],args [1]);
+			return FIM_CNS_EMPTY_RESULT;
+		}
 		else
 			return FIM_CMD_HELP_SET;
 	}
@@ -864,29 +740,28 @@ nop:
 		return FIM_CNS_EMPTY_RESULT;
 	}
 
-	fim::string CommandConsole::fcmd_clear(const args_t& args)
+	fim_cxr CommandConsole::fcmd_clear(const args_t& args)
 	{
 		displaydevice_->console_control(0x03);//experimental
 		return FIM_CNS_EMPTY_RESULT;
 	}
 #endif /* FIM_WANT_NO_OUTPUT_CONSOLE */
 
-	fim::string CommandConsole::fcmd_do_getenv(const args_t& args)
+	fim_cxr CommandConsole::fcmd_do_getenv(const args_t& args)
 	{
-		string help="usage : " FIM_FLT_GETENV " " FIM_CNS_EX_ID_STRING " will create a fim variable named " FIM_CNS_EX_ID_STRING " with value $" FIM_CNS_EX_ID_STRING " (if nonempty), from the current shell."
+		string help="usage: " FIM_CMD_HELP_GETENV;
 #ifndef HAVE_GETENV
-		" (note that getenv call was not available at build time, so it won't work)\n"
+		help="sorry, no getenv call was not available at build time\n";
 #endif /* HAVE_GETENV */
-		;
+
 		if( ! args.size())
 			return help;
 #ifdef HAVE_GETENV
 		if(1==args.size())
 		{
 			if(fim_getenv(args[0].c_str()))
-				return setVariable( fim::string("ENV_")+args[0], fim_getenv(args[0].c_str()) );
-			else
-				return FIM_CNS_EMPTY_RESULT;
+				setVariable( fim::string("ENV_")+args[0], fim_getenv(args[0].c_str()) );
+			return FIM_CNS_EMPTY_RESULT;
 		}
 		else
 			return help;
@@ -895,20 +770,54 @@ nop:
 #endif /* HAVE_GETENV */
 	}
 
-	fim::string CommandConsole::fcmd_desc(const args_t& args)
+#if FIM_WANT_PIC_CMTS_RELOAD
+	/* */
+#endif /* FIM_WANT_PIC_CMTS_RELOAD */
+
+	fim_cxr CommandConsole::fcmd_desc(const args_t& args)
 	{
 #if FIM_WANT_PIC_CMTS
 		fim_char_t sc = '\t';
+		const int aoc = fim_args_opt_count(args,'-');
 
-		if(2 > args.size() || args[0] != "load" )
+#if FIM_WANT_PIC_CMTS_RELOAD
+		if ( args.size()-aoc > 0 && args[aoc] == "reload" )
+		{
+			for (auto & dfp : cc.browser_.dfl_) 
+				cc.id_.fetch(dfp.first,dfp.second);
+			browser_.cache_.desc_update();
+		}
+#endif /* FIM_WANT_PIC_CMTS_RELOAD */
+
+		if(2 > args.size()-aoc )
+			goto err;
+
+		if ( args[aoc] == "load" )
+		{
+			if(2 < args.size()-aoc)
+				sc = *args[2+aoc].c_str();
+			this->id_.fetch(args[1+aoc],sc);
+			browser_.cache_.desc_update();
+		}
+		else
+		if ( args[aoc] == "save" )
+		{
+			const bool saveall = fim_args_opt_have(args,"-all");
+			const bool wappend = fim_args_opt_have(args,"-append");
+			const bool nooverw = fim_args_opt_have(args,"-nooverw");
+			const string sdn = args[aoc+1];
+
+			if(2 < args.size()-aoc)
+				sc = *args[2+aoc].c_str();
+
+			if(nooverw && is_file_nonempty(sdn))
+				goto err;
+			browser_.dump_desc(args[1+aoc].c_str(),sc,saveall,wappend);
+		}
+		else
 			goto err;
-		if(2 < args.size())
-			sc = *args[2].c_str();
-		this->id_.fetch(args[1],sc);
-		browser_.cache_.desc_update();
 err:
 #endif /* FIM_WANT_PIC_CMTS */
 		return FIM_CNS_EMPTY_RESULT;
 	}
-
-}
+} /* namespace fim */
