@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2016-03-14 11:17:40 +0100 (Mon, 14 Mar 2016) $ */
+/* $LastChangedDate: 2024-04-21 23:09:14 +0200 (Sun, 21 Apr 2024) $ */
 /*
  CommandConsole-init.cpp : Fim console initialization
 
- (c) 2010-2016 Michele Martone
+ (c) 2010-2024 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,15 +18,9 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
-
-/*
- * TODO:
- * 	framebufferdevice -> device
- * 	output methods should be moved in some other, new class
- * */
 #include "fim.h"
 #ifdef FIM_DEFAULT_CONFIGURATION
-#include "conf.h"
+#include "conf.h" /* FIM_DEFAULT_CONFIG_FILE_CONTENTS declared here */ 
 #endif /* FIM_DEFAULT_CONFIGURATION */
 #include <sys/time.h>
 #include <errno.h>
@@ -35,6 +29,8 @@
 #include "readline.h"
 #endif /* FIM_USE_READLINE */
 
+#include <type_traits>
+
 namespace fim
 {
 
@@ -57,7 +53,9 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 		do
 		{
 			btime=-getmilliseconds();
-			//fim_bench_video(bfp);
+#if 0 
+			fim_bench_video(NULL);
+#endif /* 0 */
 			bo->quickbench(qbi);
 			btime+=getmilliseconds();
 			++times;
@@ -73,23 +71,23 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 }
 #endif /* FIM_WANT_BENCHMARKS */
 
-
-
 	fim_err_t CommandConsole::init(fim::string device)
 	{
 		/*
-		 * TODO : move most of this stuff to the constructor, some day.
+		 * TODO: move most of this stuff to the constructor, some day.
 		 */
 		fim_int xres=0,yres=0;
 		bool device_failure=false;
 		int dosso=device.find(FIM_SYM_DEVOPTS_SEP_STR);
-		bool wcs = false;
+#ifdef FIM_USE_READLINE
+		bool wcs = false; /* want cookie stream (readline but no stdin) */
+#endif /* FIM_USE_READLINE */
 		std::string dopts;
-		/* new : prevents atof, sprintf and such conversion mismatches! */
-		setlocale(LC_ALL,"C");	/* portable (among Linux hosts) : should use dots for numerical radix separator */
+		/* prevents atof, sprintf and such conversion mismatches. */
+		setlocale(LC_ALL,"C");	/* portable (among Linux hosts): should use dots for numerical radix separator */
 		//setlocale(LC_NUMERIC,"en_US"); /* lame  */
 		//setlocale(LC_ALL,""); /* just lame */
-		displaydevice_=FIM_NULL;	/* TODO : is this really necessary ? */
+		assert(displaydevice_==FIM_NULL);
 
 		if(dosso>0)
 			dopts=device.substr(dosso+strlen(FIM_SYM_DEVOPTS_SEP_STR),device.length()).c_str();
@@ -107,20 +105,33 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 					mc_
 #endif /* FIM_WANT_NO_OUTPUT_CONSOLE */
 					);
-			if(!displaydevice_ || ((FramebufferDevice*)displaydevice_)->framebuffer_init()){cleanup();return FIM_ERR_GENERIC;}
+			const bool do_boz_patch = (dopts.size() && dopts[0]=='S')?0:(0xbabebabe==0xbabebabe);
+			if( displaydevice_)
+			{
+				ffdp=((FramebufferDevice*)displaydevice_);
+				if(default_fbdev)ffdp->set_fbdev(default_fbdev);
+				if(default_fbmode)ffdp->set_fbmode(default_fbmode);
+				if(default_vt!=-1)ffdp->set_default_vt(default_vt);
+				if(default_fbgamma!=-1.0)ffdp->set_default_fbgamma(default_fbgamma);
+				ffdp=NULL;
+			}
+			if(!displaydevice_ || ((FramebufferDevice*)displaydevice_)->framebuffer_init(do_boz_patch))
+			{
+				if(do_boz_patch) // if not, allow for reinit
+					cleanup();
+				displaydevice_=NULL;
+				if(ffdp)
+					delete ffdp;
+				return FIM_ERR_GENERIC;
+			}
 			ffdp=((FramebufferDevice*)displaydevice_);
-			setVariable(FIM_VID_DEVICE_DRIVER,FIM_DDN_VAR_FB);
-			if(default_fbdev)ffdp->set_fbdev(default_fbdev);
-			if(default_fbmode)ffdp->set_fbmode(default_fbmode);
-			if(default_vt!=-1)ffdp->set_default_vt(default_vt);
-			if(default_fbgamma!=-1.0)ffdp->set_default_fbgamma(default_fbgamma);
 			mangle_tcattr_=true;
 		}
 #endif	//#ifndef FIM_WITH_NO_FRAMEBUFFER
 
 
 		#ifdef FIM_WITH_LIBIMLIB2
-		if(device.find(FIM_DDN_VAR_IL2)==0)
+		if(device.find(FIM_DDN_INN_IL2)==0)
 		{
 			DisplayDevice *imld=FIM_NULL;
 			imld=new Imlib2Device(
@@ -133,14 +144,15 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 			if(imld && displaydevice_==FIM_NULL)
 			{
 				displaydevice_=imld;
-				setVariable(FIM_VID_DEVICE_DRIVER,FIM_DDN_VAR_IL2);
 				mangle_tcattr_=false;
 			}
 			else
 			{
 				device_failure=true;
 			}
-			wcs = true; /* FIXME: want cookie stream (for readline with no stdin) */
+#ifdef FIM_USE_READLINE
+			wcs = true;
+#endif /* FIM_USE_READLINE */
 		}
 		#endif /* FIM_WITH_LIBIMLIB2 */
 
@@ -162,36 +174,59 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 			if(sdld && displaydevice_==FIM_NULL)
 			{
 				displaydevice_=sdld;
-				setVariable(FIM_VID_DEVICE_DRIVER,FIM_DDN_VAR_SDL);
 				mangle_tcattr_=false;
 			}
 			else
 			{
 				device_failure=true;
 			}
-			wcs = true; /* FIXME: want cookie stream (for readline with no stdin) */
+#ifdef FIM_USE_READLINE
+			wcs = true;
+#endif /* FIM_USE_READLINE */
 		}
 		#endif /* FIM_WITH_LIBSDL */
 
-		#ifdef FIM_WITH_CACALIB
+		#ifdef FIM_WITH_LIBGTK
+		if(device.find(FIM_DDN_INN_GTK)==0)
+		{
+			DisplayDevice *gtkd=FIM_NULL;
+			gtkd=new GTKDevice(
+#ifndef FIM_WANT_NO_OUTPUT_CONSOLE
+					mc_,
+#endif /* FIM_WANT_NO_OUTPUT_CONSOLE */
+					dopts
+					);
+			if(gtkd && gtkd->initialize(sym_keys_)!=FIM_ERR_NO_ERROR){delete gtkd ; gtkd=FIM_NULL;}
+			if(gtkd && displaydevice_==FIM_NULL)
+			{
+				displaydevice_=gtkd;
+				mangle_tcattr_=false;
+			}
+			else
+				device_failure=true;
+		}
+		#endif /* FIM_WITH_LIBGTK */
+
+		#ifdef FIM_WITH_LIBCACA
 		if(device.find(FIM_DDN_INN_CACA)==0)
 		{
 			DisplayDevice *cacad=FIM_NULL;
 			cacad=new CACADevice(
 #ifndef FIM_WANT_NO_OUTPUT_CONSOLE
-					mc_
+					mc_,
 #endif /* FIM_WANT_NO_OUTPUT_CONSOLE */
-					); if(cacad && cacad->initialize(sym_keys_)!=FIM_ERR_NO_ERROR){delete cacad ; cacad=FIM_NULL;}
+					dopts
+					);
+			if(cacad && cacad->initialize(sym_keys_)!=FIM_ERR_NO_ERROR){delete cacad ; cacad=FIM_NULL;}
 			if(cacad && displaydevice_==FIM_NULL)
 			{
 				displaydevice_=cacad;
-				setVariable(FIM_VID_DEVICE_DRIVER,FIM_DDN_VAR_CACA);
 				mangle_tcattr_=false;
 			}
 			else
 				device_failure=true;
 		}
-		#endif /* FIM_WITH_CACALIB */
+		#endif /* FIM_WITH_LIBCACA */
 
 		#ifdef FIM_WITH_AALIB
 		if(device.find(FIM_DDN_INN_AA)==0)
@@ -208,16 +243,12 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 		if(aad_ && displaydevice_==FIM_NULL)
 		{
 			displaydevice_=aad_;
-			setVariable(FIM_VID_DEVICE_DRIVER,FIM_DDN_VAR_AA);
 #if FIM_WANT_SCREEN_KEY_REMAPPING_PATCH
 			/*
-			 * FIXME
-			 *
-			 * seems like the keymaps get shifted when running under screen
-			 * weird, isn't it ?
-			 * Regard this as a weird patch.
+			 * It seems like the keymaps get shifted when running under screen.
+			 * Regard this as a patch.
 			 * */
-			const fim_char_t * term = fim_getenv(FIM_CNS_TERM_VAR);
+			const fim_char_t * const term = fim_getenv(FIM_CNS_TERM_VAR);
 			if(term && string(term).re_match("screen"))
 			{
 				sym_keys_[FIM_KBD_LEFT]-=3072;
@@ -238,8 +269,8 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 
 		if( displaydevice_==FIM_NULL)
 		{
+			device=FIM_DDN_INN_DUMB;
 			displaydevice_=&dummydisplaydevice_;
-			setVariable(FIM_VID_DEVICE_DRIVER,FIM_DDN_VAR_DUMB);
 			if(device_failure)
 			{
 				std::cerr << "Failure using the \""<<device<<"\" display device driver string (wrong options to it ?)!\n";
@@ -248,7 +279,7 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 
 			}
 			else
-			if(device!=FIM_DDN_VAR_DUMB)
+			if(device!=FIM_DDN_INN_DUMB)
 			{
 				if( 0
 		#ifndef FIM_WITH_AALIB
@@ -257,15 +288,18 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 		#ifndef FIM_WITH_LIBSDL
 					|| device==FIM_DDN_INN_SDL
 		#endif /* FIM_WITH_LIBSDL */
+		#ifndef FIM_WITH_LIBGTK
+					|| device==FIM_DDN_INN_GTK
+		#endif /* FIM_WITH_LIBGTK */
 		#ifdef FIM_WITH_NO_FRAMEBUFFER
 					|| device==FIM_DDN_INN_FB
 		#endif /* FIM_WITH_NO_FRAMEBUFFER */
 		#ifndef FIM_WITH_LIBIMLIB2
-					|| device==FIM_DDN_VAR_IL2
+					|| device==FIM_DDN_INN_IL2
 		#endif /* FIM_WITH_LIBIMLIB2 */
-		#ifndef FIM_WITH_CACALIB
+		#ifndef FIM_WITH_LIBCACA
 					|| device==FIM_DDN_INN_CACA
-		#endif /* FIM_WITH_CACALIB */
+		#endif /* FIM_WITH_LIBCACA */
 				)
 					std::cerr << "Device string \""<<device<<"\" has been configured out (disabled) at build time.\n";
 				else
@@ -275,11 +309,10 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 			std::cerr << "Using the default \""<<FIM_DDN_INN_DUMB<<"\" display device instead.\n";
 		}
 
+		setVariable(FIM_VID_DEVICE_DRIVER,device);
+		mc_.setDisplayDevice(displaydevice_);
 		xres=displaydevice_->width(),yres=displaydevice_->height();
-
-		// textual console reformatting (should go to displaydevice some day)
-		displaydevice_->init_console();
-		// FIXME: shall check the error result
+		displaydevice_->format_console();
 
 #ifdef FIM_WINDOWS
 		/* true pixels if we are in framebuffer mode */
@@ -292,23 +325,21 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 
 		try
 		{
-			window_ = new FimWindow( *this, Rect(0,0,xres,yres) );
+			window_ = new FimWindow( *this, displaydevice_, Rect(0,0,xres,yres) );
 
 			if(window_)window_->setroot();
 		}
 		catch(FimException e)
 		{
-			if( e == FIM_E_NO_MEM || true ) quit(FIM_E_NO_MEM);
+			if( e == FIM_E_NO_MEM || true )
+				quit(FIM_E_NO_MEM);
 		}
 
-		/*
-		 * TODO: exceptions should be launched here in case ...
-		 * */
-		addCommand(new Command(fim::string(FIM_FLT_WINDOW),fim::string(FIM_CMD_HELP_WINDOW), window_,&FimWindow::fcmd_cmd));
+		addCommand(Command(FIM_FLT_WINDOW,FIM_CMD_HELP_WINDOW, window_,&FimWindow::fcmd_cmd));
 #else /* FIM_WINDOWS */
 		try
 		{
-			viewport_ = new Viewport(*this);
+			viewport_ = new Viewport(*this, displaydevice_);
 		}
 		catch(FimException e)
 		{
@@ -317,7 +348,7 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 #endif /* FIM_WINDOWS */
 #ifdef FIM_NAMESPACES
 		if(displaydevice_)
-			setVariable(FIM_VID_FIM_BPP ,(fim_int) displaydevice_->get_bpp());
+			setVariable(FIM_VID_FIM_BPP ,displaydevice_->get_bpp());
 #endif /* FIM_NAMESPACES */
 		setVariable(FIM_VID_SCREEN_WIDTH, xres);
 		setVariable(FIM_VID_SCREEN_HEIGHT,yres);
@@ -328,7 +359,7 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 	    	FIM_AUTOCMD_EXEC(FIM_ACM_PREHFIMRC,"");
 
   #ifndef FIM_WANT_NOSCRIPTING
-		if(preConfigCommand_!=fim::string(""))
+		if(!preConfigCommand_.empty())
 			execute_internal(preConfigCommand_.c_str(),FIM_X_HISTORY);
 
 		if(getIntVariable(FIM_VID_NO_DEFAULT_CONFIGURATION)==0 )
@@ -339,14 +370,13 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 			execute_internal(FIM_DEFAULT_CONFIG_FILE_CONTENTS,FIM_X_QUIET);
     #endif		/* FIM_DEFAULT_CONFIGURATION */
 		}
+	    	FIM_AUTOCMD_EXEC(FIM_ACM_POSTHFIMRC,""); 
   #endif /* FIM_WANT_NOSCRIPTING */
 
 #ifndef FIM_NOFIMRC
   #ifndef FIM_WANT_NOSCRIPTING
-		fim_char_t rcfile[FIM_PATH_MAX];
-		const fim_char_t *e = fim_getenv(FIM_CNS_HOME_VAR);
+		const std::string home_env_var = fim_getenv(FIM_CNS_HOME_VAR);
 
-	    	FIM_AUTOCMD_EXEC(FIM_ACM_POSTHFIMRC,""); 
 	    	FIM_AUTOCMD_EXEC(FIM_ACM_PREGFIMRC,""); 
 
 		if((getIntVariable(FIM_VID_LOAD_DEFAULT_ETC_FIMRC)==1 )
@@ -355,26 +385,32 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 		{
 			string ef=getStringVariable(FIM_VID_DEFAULT_ETC_FIMRC);
 			if(is_file(ef.c_str()))
-				if(FIM_ERR_NO_ERROR!=executeFile(ef.c_str()));
+				if(FIM_ERR_NO_ERROR!=executeFile(ef.c_str()))
+		    			std::cerr << "Problems loading " << ef << std::endl;
 		}
 		
 		/* execution of command line-set autocommands */
 	    	FIM_AUTOCMD_EXEC(FIM_ACM_POSTGFIMRC,""); 
 	    	FIM_AUTOCMD_EXEC(FIM_ACM_PREUFIMRC,""); 
+  #endif /* FIM_WANT_NOSCRIPTING */
+#endif /* FIM_NOFIMRC */
 
 		{
+#ifndef FIM_WANT_NOSCRIPTING
 			#include "grammar.h"
 			setVariable(FIM_VID_FIM_DEFAULT_GRAMMAR_FILE_CONTENTS,FIM_DEFAULT_GRAMMAR_FILE_CONTENTS);
+#endif /* FIM_WANT_NOSCRIPTING */
 		}
 
-		if(e && strlen(e)<FIM_PATH_MAX-8)//strlen("/" FIM_CNS_USR_RC_FILEPATH)+2
+#ifndef FIM_NOFIMRC
+  #ifndef FIM_WANT_NOSCRIPTING
+		if(home_env_var.size())//strlen("/" FIM_CNS_USR_RC_FILEPATH)+2
 		{
-			strcpy(rcfile,e);
-			strcat(rcfile,"/" FIM_CNS_USR_RC_FILEPATH);
+			const std::string rcfile = home_env_var + "/" FIM_CNS_USR_RC_FILEPATH;
 			if(getIntVariable(FIM_VID_NO_RC_FILE)!=1 )
 			{
 				if(
-					(!is_file(rcfile) || FIM_ERR_NO_ERROR!=executeFile(rcfile))
+					(!is_file(rcfile) || FIM_ERR_NO_ERROR!=executeFile(rcfile.c_str()))
 	//			&&	(!is_file(FIM_CNS_SYS_RC_FILEPATH) || FIM_ERR_NO_ERROR!=executeFile(FIM_CNS_SYS_RC_FILEPATH))
 				  )
   #endif /* FIM_WANT_NOSCRIPTING */
@@ -394,15 +430,17 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 		}
   #endif		/* FIM_WANT_NOSCRIPTING */
 #endif		/* FIM_NOFIMRC */
+		setVariable(FIM_VID_ALL_FILE_LOADERS,Var(fim_loaders_to_string()));
 	    	FIM_AUTOCMD_EXEC(FIM_ACM_POSTUFIMRC,""); 
 	    	FIM_AUTOCMD_EXEC(FIM_ACM_POSTCONF,"");
 #ifndef FIM_WANT_NOSCRIPTING
-		for(size_t i=0;i<scripts_.size();++i) executeFile(scripts_[i].c_str());
+		for(size_t i=0;i<scripts_.size();++i)
+			executeFile(scripts_[i].c_str());
 #endif		/* FIM_WANT_NOSCRIPTING */
 #ifdef FIM_AUTOCMDS
-		if(postInitCommand_!=fim::string(""))
+		if(!postInitCommand_.empty())
 			autocmd_add(FIM_ACM_PREEXECUTIONCYCLE,"",postInitCommand_.c_str());
-		if(postExecutionCommand_!=fim::string(""))
+		if(!postExecutionCommand_.empty())
 			autocmd_add(FIM_ACM_POSTEXECUTIONCYCLE,"",postExecutionCommand_.c_str());
 #endif /* FIM_AUTOCMDS */
 
@@ -410,12 +448,72 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 		rl::initialize_readline( displaydevice_==FIM_NULL, wcs );
 		load_or_save_history(true);
 #endif /* FIM_USE_READLINE */
+
+		static_assert(std::is_signed<fim_int>(),FIM_EMSG_ITE);
+		static_assert(std::is_floating_point<fim_scale_t>(),FIM_EMSG_ITE);
+		static_assert(std::is_floating_point<fim_angle_t>(),FIM_EMSG_ITE);
+		static_assert(std::is_floating_point<fim_float_t>(),FIM_EMSG_ITE);
+		static_assert(std::is_enum<fim_redraw_t>(),FIM_EMSG_ITE);
+		static_assert(std::is_class<Var>(),FIM_EMSG_ITE);
+		static_assert(std::is_abstract<DisplayDevice>(),FIM_EMSG_ITE);
+		static_assert(std::is_polymorphic<DisplayDevice>(),FIM_EMSG_ITE);
+		static_assert(std::is_polymorphic<Namespace>(),FIM_EMSG_ITE);
+		static_assert(std::is_polymorphic<MiniConsole>(),FIM_EMSG_ITE);
+		static_assert(*FIM_SYM_UNKNOWN_STRING==FIM_SYM_UNKNOWN_CHAR,FIM_EMSG_ICE);
 		if(getIntVariable(FIM_VID_SANITY_CHECK)==1 )
 		{
+			fim_err_t errval = FIM_ERR_NO_ERROR;
+			assert( fim::string("-file").re_match("^-"));
+			assert( fim::string("file ").re_match(" "));
+			assert(!fim::string("file").re_match(" "));
+			assert(!fim::string("file").re_match("^-"));
+			assert(!fim::string("-file").re_match(FIM_CNS_PIPEABLE_PATH_RE));
+			assert(!fim::string("file ").re_match(FIM_CNS_PIPEABLE_PATH_RE));
+			assert( fim::string("file-").re_match(FIM_CNS_PIPEABLE_PATH_RE));
+			if ( fim_common_test() != 0 )
+				errval = FIM_ERR_GENERIC;
+			{
+				fim_char_t s[] = {'\\','x','4','1','\\','a','a','\\','x','4','2','b','\\','b',0};
+				trec(s,"ab","cd");
+				if ( strcmp(s, "AcaBbd") )
+					errval = FIM_ERR_GENERIC;
+			}
+			for (int si = 0; si < 3; ++si)
+			{
+				fim_stream stream(si);
+				const fim_str_t t = 1;
+				const fim_char_t*s = "s";
+				const fim_byte_t b = 'b';
+				const fim_byte_t S [] = { 'b', FIM_SYM_CHAR_NUL };
+				const fim_char_t c = 'c';
+				stream << s    << " ";
+				stream << b    << " ";
+				stream << S    << " ";
+				stream << t    << " ";
+				stream << 1    << " ";
+				stream << 1U   << " ";
+				stream << c    << " ";
+				stream << 99.f << " ";
+				stream << "\n";
+			}
 #if FIM_WANT_BENCHMARKS
 			fim_bench_subsystem(displaydevice_);
+			fim_bench_subsystem(&browser_);
 			fim_bench_subsystem(this);
+			{
+#ifndef FIM_WANT_NO_OUTPUT_CONSOLE
+				const int tbs = 2 * mc_.byte_size();
+				std::vector<char> hs(tbs,'?');
+				hs[tbs-1] = FIM_SYM_CHAR_NUL;
+				CommandConsole::status_screen(hs.data()); // TODO: this is to check if DebugConsole coped with that huge string, so a status check would be in order here.
+				displaydevice_->fb_status_screen_new("clear",false,0x01); // TODO: need to connect these to a command
+				displaydevice_->fb_status_screen_new("scroll_down",false,0x02);
+				displaydevice_->fb_status_screen_new("scroll_up",false,0x03);
+#endif /* FIM_WANT_NO_OUTPUT_CONSOLE */
+			}
 #endif /* FIM_WANT_BENCHMARKS */
+			if ( errval == FIM_ERR_GENERIC )
+				return_code_ = -1;
 			quit(return_code_);
 			exit(return_code_);
 		}
@@ -430,20 +528,25 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 
 	string CommandConsole::get_bresults_string(fim_int qbi, fim_int qbtimes, fim_fms_t qbttime)const
 	{
-		string msg=FIM_CNS_EMPTY_STRING;
+		std::ostringstream oss;
 		switch(qbi)
 		{
 			case 0:
-			msg+="fim variables set/get test";
-			msg+=" : ";
-			msg+=string((float)(((fim_fms_t)qbtimes)/((qbttime)*1.e-3)));
-			msg+=" set/get /s\n";
+				oss << "fim console random variables set/get test: " << ((float)(((fim_fms_t)qbtimes)/((qbttime)*1.e-3))) << " set/get /s\n";
 		}
-		return msg;
+		return oss.str();
 	}
 
 	void CommandConsole::quickbench_init(fim_int qbi)
 	{
+		std::ostringstream oss;
+		switch(qbi)
+		{
+			case 0:
+				oss << "fim console check";
+				std::cout << oss.str() << ": " << "please be patient\n";
+			break;
+		}
 	}
 
 	void CommandConsole::quickbench_finalize(fim_int qbi)
@@ -452,10 +555,14 @@ static fim_err_t fim_bench_subsystem(Benchmarkable * bo)
 
 	void CommandConsole::quickbench(fim_int qbi)
 	{
-		const fim_int max_sq=1024*1024;
-		/* FIXME: may be enhanced with sequence numbers ... */
-		cc.setVariable(fim_rand()%(max_sq),fim_rand());
-		cc.getIntVariable(fim_rand()%max_sq);
+		switch(qbi)
+		{
+			case 0:
+				FIM_CONSTEXPR fim_int max_sq=1024*1024;
+				cc.setVariable(fim_rand()%(max_sq),fim_rand());
+				cc.getIntVariable(fim_rand()%max_sq);
+			break;
+		}
 	}
 #endif /* FIM_WANT_BENCHMARKS */
 
