@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2015-12-19 01:44:41 +0100 (Sat, 19 Dec 2015) $ */
+/* $LastChangedDate: 2024-03-18 15:17:10 +0100 (Mon, 18 Mar 2024) $ */
 /*
  CommandConsole-var.h : CommandConsole variables store
 
- (c) 2013-2015 Michele Martone
+ (c) 2013-2024 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,17 +18,16 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
-
 #include "fim.h"
 
 namespace fim
 {
 
-	fim::string CommandConsole::rnid(const fim::string & varname)const
+	fim_var_id CommandConsole::rnid(const fim_var_id& varname)const
 	{
 		fim::string id;
 #ifdef FIM_NAMESPACES
-		if( varname[1]==FIM_SYM_NAMESPACE_SEP && varname.length() > 2 )
+		if( varname.length() > 2 && varname.at(1) == FIM_SYM_NAMESPACE_SEP )
 		{
 			if( varname.length() > 2 )
 				id = varname.c_str()+2;
@@ -39,17 +38,17 @@ namespace fim
 		return id;
 	}
 
-	Namespace * CommandConsole::rns(const fim::string varname)
+	Namespace * CommandConsole::rns(const fim_var_id varname)
 	{
 		Namespace * nsp = FIM_NULL;
 #ifdef FIM_NAMESPACES
-		if( varname[1]==FIM_SYM_NAMESPACE_SEP )
+		if( varname.length() > 2 && varname.at(1) == FIM_SYM_NAMESPACE_SEP )
 		{
 			try
 			{
 			//a specific namespace was selected!
-			fim_char_t ns = varname[0];
-			fim::string id = varname.c_str()+2;
+			const fim_char_t ns = varname.at(0);
+			const fim_var_id id = varname.c_str()+2;
 
 			if( ns == FIM_SYM_NAMESPACE_WINDOW_CHAR )
 #ifdef FIM_WINDOWS
@@ -71,7 +70,15 @@ namespace fim
 			if( ns == FIM_SYM_NAMESPACE_IMAGE_CHAR )
 			{
 				//image variable
-				nsp = (Image*) browser_.c_image(); /* FIXME: porcata assurda */
+#if FIM_WINDOWS
+				if(window_ &&
+				   window_->current_viewportp() && 
+				   window_->current_viewportp()->getImage())
+					nsp = window_->current_viewportp()->getImage();
+#else /* FIM_WINDOWS */
+				if ( viewport_->getImage() )
+					nsp = viewport_->getImage();
+#endif /* FIM_WINDOWS */
 				goto err;
 			}
 			else
@@ -94,7 +101,7 @@ namespace fim
 				goto err;
 			}
 			}
-			catch(FimException e){}
+			catch(FimException){}
 		}
 #endif /* FIM_NAMESPACES */
 		nsp = this;
@@ -102,17 +109,17 @@ err:
 		return nsp;
 	}
 
-	const Namespace * CommandConsole::c_rns(const fim::string varname)const
+	const Namespace * CommandConsole::c_rns(const fim_var_id varname)const
 	{
 		const Namespace * nsp = FIM_NULL;
 #ifdef FIM_NAMESPACES
-		if( varname[1]==FIM_SYM_NAMESPACE_SEP )
+		if( varname.length() > 2 && varname.at(1) == FIM_SYM_NAMESPACE_SEP )
 		{
 			try
 			{
 			//a specific namespace was selected!
-			fim_char_t ns = varname[0];
-			fim::string id = varname.c_str()+2;
+			const fim_char_t ns = varname.at(0);
+			const fim_var_id id = varname.c_str()+2;
 
 			if( ns == FIM_SYM_NAMESPACE_WINDOW_CHAR )
 #ifdef FIM_WINDOWS
@@ -134,7 +141,7 @@ err:
 			if( ns == FIM_SYM_NAMESPACE_IMAGE_CHAR )
 			{
 				//image variable
-				nsp = (Image*) browser_.c_image(); /* FIXME: porcata assurda */
+				nsp = browser_.c_getImage();
 				goto err;
 			}
 			else
@@ -165,88 +172,89 @@ err:
 		return nsp;
 	}
 
-	fim_int CommandConsole::setVariable(const fim::string& varname,fim_int value)
+	fim_int CommandConsole::setVariable(const fim_var_id& varname,fim_int value)
+	{
+		if( Namespace *nsp = rns(varname) )
+			nsp->setVariable(rnid(varname),value);
+		return value;
+	}
+
+#if FIM_WANT_LONG_INT
+	int CommandConsole::setVariable(const fim_var_id& varname,int value)
+	{
+		return setVariable(varname,static_cast<fim_int>(value));
+	}
+
+	unsigned int CommandConsole::setVariable(const fim_var_id& varname,unsigned int value)
+	{
+		return setVariable(varname,static_cast<fim_int>(value));
+	}
+#endif /* FIM_WANT_LONG_INT */
+
+	fim_float_t CommandConsole::setVariable(const fim_var_id& varname,fim_float_t value)
+	{
+		if( Namespace *nsp = rns(varname) )
+			nsp->setVariable(rnid(varname),value);
+		return value;
+	}
+
+	const fim_char_t* CommandConsole::setVariable(const fim_var_id& varname,const fim_char_t*value)
+	{
+		if( Namespace *nsp = rns(varname) )
+			nsp->setVariable(rnid(varname),value);
+		return value;
+	}
+
+	const Var & CommandConsole::setVariable(const fim_var_id varname,const Var&value)
+	{
+		if( Namespace *nsp = rns(varname) )
+			nsp->setVariable(rnid(varname),value);
+		return value;
+	}
+
+	const fim::string& CommandConsole::setVariable(const fim_var_id varname,const fim::string&value)
+	{
+		if( Namespace *nsp = rns(varname) )
+			nsp->setVariable(rnid(varname),value);
+		return value;
+	}
+
+	fim_bool_t CommandConsole::isSetVar(const fim_var_id& varname)const
+	{
+		if( const Namespace *nsp = c_rns(varname) )
+			return nsp->isSetVar(rnid(varname));
+		return false;
+	}
+
+	fim_int CommandConsole::getIntVariable(const fim_var_id& varname)const
 	{
 		fim_int retval = 0;
-		Namespace *nsp = rns(varname);
-
-		if(nsp)
-			retval = nsp->setVariable(rnid(varname),value);
-err:
-		return retval;
-	}
-
-	fim_float_t CommandConsole::setVariable(const fim::string& varname,fim_float_t value)
-	{
-		fim_float_t retval = FIM_CNS_EMPTY_FP_VAL;
-		Namespace *nsp = rns(varname);
-
-		if(nsp)
-			retval = nsp->setVariable(rnid(varname),value);
-err:
-		return retval;
-	}
-
-	fim_int CommandConsole::setVariable(const fim::string& varname,const fim_char_t*value)
-	{
-		fim_int retval = 0;
-		Namespace *nsp = rns(varname);
-
-		if(nsp)
-			retval = nsp->setVariable(rnid(varname),value);
-err:
-		return retval;
-	}
-
-	Var CommandConsole::setVariable(const fim::string varname,const Var&value)
-	{
-		Var retval = (fim_int)0;
-		Namespace *nsp = rns(varname);
-
-		if(nsp)
-			retval = nsp->setVariable(rnid(varname),value);
-err:
-		return retval;
-	}
-
-	fim_int CommandConsole::getIntVariable(const fim::string &varname)const
-	{
-		fim_int retval = 0;
-		const Namespace *nsp = c_rns(varname);
-
-		if(nsp)
+		if( const Namespace *nsp = c_rns(varname) )
 			retval = nsp->getIntVariable(rnid(varname));
-err:
 		return retval;
 	}
 
-	fim_float_t CommandConsole::getFloatVariable(const fim::string &varname)const
+	fim_float_t CommandConsole::getFloatVariable(const fim_var_id& varname)const
 	{
 		fim_float_t retval = FIM_CNS_EMPTY_FP_VAL;
-		const Namespace *nsp = c_rns(varname);
-
-		if(nsp)
+		if( const Namespace *nsp = c_rns(varname) )
 			retval = nsp->getFloatVariable(rnid(varname));
 		return retval;
 	}
 
-	fim::string CommandConsole::getStringVariable(const fim::string &varname)const
+	fim::string CommandConsole::getStringVariable(const fim_var_id& varname)const
 	{
 		fim::string retval = FIM_CNS_EMPTY_RESULT;
-		const Namespace *nsp = c_rns(varname);
-
-		if(nsp)
+		if( const Namespace *nsp = c_rns(varname) )
 			retval = nsp->getStringVariable(rnid(varname));
 		return retval;
 	}
 
-	Var CommandConsole::getVariable(const fim::string &varname)const
+	const Var CommandConsole::getVariable(const fim_var_id& varname)const
 	{
-		const Namespace *nsp = c_rns(varname);
-
-		if(nsp)
+		if( const Namespace *nsp = c_rns(varname) )
 			return nsp->getVariable(rnid(varname));
 		else
-			return Var();
+			return {};
 	}
-}
+} /* namespace fim */
