@@ -1,4 +1,4 @@
-/* $LastChangedDate: 2024-04-07 19:28:17 +0200 (Sun, 07 Apr 2024) $ */
+/* $LastChangedDate: 2024-05-12 11:54:03 +0200 (Sun, 12 May 2024) $ */
 /*
  DisplayDevice.h : virtual device Fim driver header file
 
@@ -28,6 +28,8 @@
 #define FIM_WANT_MOUSE_PAN FIM_WANT_MOUSE /* experimental mouse support */
 #define FIM_WANT_MOUSE_WHEEL_SCROLL FIM_WANT_MOUSE /* experimental mouse support */
 
+#define FIM_DD_HURRY_QUIT() cc.execute(FIM_FLT_QUIT, {}, false, false, true); // calling exit() requires a clean terminal
+
 #if FIM_WANT_PROOF_OF_CONCEPT_MOUSE_SUPPORT
 #define FIM_MOUSE_LBUTTON_MASK 1
 #define FIM_MOUSE_MBUTTON_MASK 2
