@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2017-04-09 13:01:15 +0200 (Sun, 09 Apr 2017) $ */
+/* $LastChangedDate: 2023-04-09 16:02:31 +0200 (Sun, 09 Apr 2023) $ */
 /*
  FbiStuffJpeg.cpp : fbi functions for JPEG files, modified for fim
 
- (c) 2007-2017 Michele Martone
+ (c) 2007-2023 Michele Martone
  (c) 1998-2006 Gerd Knorr <kraxel@bytesex.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -55,6 +55,12 @@ extern "C"
 // we require C linkage for these symbols
 #include <jpeglib.h>
 }
+#ifndef HAVE_BOOLEAN
+#ifndef TRUE
+#define TRUE 1
+#endif
+#endif /* HAVE_BOOLEAN */
+#define FIM_JPEGLIB_TRUE TRUE /* jpeglib 8.0 uses 'boolean' */
 
 
 /*
@@ -71,14 +77,14 @@ extern "C"
 /*								*/
 
 #ifdef USE_X11
-# include <X11/Xlib.h>
-# include <X11/Intrinsic.h>
-# include <Xm/Xm.h>
-# include <Xm/Text.h>
-# include <Xm/SelectioB.h>
-# include "RegEdit.h"
-# include "ida.h"
-# include "viewer.h"
+ #include <X11/Xlib.h>
+ #include <X11/Intrinsic.h>
+ #include <Xm/Xm.h>
+ #include <Xm/Text.h>
+ #include <Xm/SelectioB.h>
+ #include "RegEdit.h"
+ #include "ida.h"
+ #include "viewer.h"
 #endif /* USE_X11 */
 namespace fim
 {
@@ -103,6 +109,8 @@ struct jpeg_state {
     /* thumbnail */
     fim_byte_t  *thumbnail;
     unsigned int   tpos, tsize;
+
+    struct ida_image_info *i; /* here for load_free_extras */
 };
 
 /* ---------------------------------------------------------------------- */
@@ -172,13 +180,13 @@ static void fim_ExifContentForeachEntryFunc (ExifEntry *entry, void * user_data)
 	fim_char_t buffer[FIM_EXIF_BUFSIZE];
 	Namespace *nsp = (Namespace*) ((void**)user_data)[0];
 	ExifContent *c =  (ExifContent *) ((void**)user_data)[1];
-	const fim_char_t *value = exif_entry_get_value(entry, buffer, sizeof(buffer));
+	const fim_char_t * const value = exif_entry_get_value(entry, buffer, sizeof(buffer));
 	ExifIfd ifd = exif_content_get_ifd (c);
-	const fim_char_t *title = exif_tag_get_name_in_ifd (entry->tag,ifd); /* exif_tag_get_name() is deprecated */
+	const fim_char_t * const title = exif_tag_get_name_in_ifd (entry->tag,ifd); /* exif_tag_get_name() is deprecated */
 
 	if(nsp)
 		/* std::cout << "|" << title << "|\n", */
-		nsp->setVariable(string("EXIF_") + title,value);
+		nsp->setVariable(string("EXIF_") + string(title),value);
 }
 
 static void fim_ExifDataForeachContentFunc (ExifContent *content, void * user_data)
@@ -388,6 +396,7 @@ jpeg_init(FILE *fp, const fim_char_t *filename, unsigned int page,
     if(!h) goto oops;
 
     h->infile = fp;
+    h->i = i;
 
     h->jerr.error_exit=FIM_NULL; // ?
     h->cinfo.err = jpeg_std_error(&h->jerr);	/* FIXME : should use an error manager of ours (this one exits the program!) */
@@ -404,8 +413,8 @@ jpeg_init(FILE *fp, const fim_char_t *filename, unsigned int page,
     if(fim_jerr /*&& h->jerr.msg_code*/)goto oops;
     jpeg_stdio_src(&h->cinfo, h->infile);
     if(fim_jerr /*&& h->jerr.msg_code*/)goto oops;
-//    if(jpeg_read_header(&h->cinfo, FIM_FBI_TRUE)==0)	goto oops;
-    jpeg_read_header(&h->cinfo, FIM_FBI_TRUE);
+//    if(jpeg_read_header(&h->cinfo, FIM_JPEGLIB_TRUE)==0)	goto oops;
+    jpeg_read_header(&h->cinfo, FIM_JPEGLIB_TRUE);
     if(fim_jerr /*&& h->jerr.msg_code*/)goto oops;
 //    if(h->jerr.msg_code)goto oops;	// this triggers with apparently good file
 
@@ -432,6 +441,7 @@ jpeg_init(FILE *fp, const fim_char_t *filename, unsigned int page,
 #else /* FIM_EXPERIMENTAL_IMG_NMSPC */
 			dump_exif(stdout,ed,FIM_NULL);
 #endif /* FIM_EXPERIMENTAL_IMG_NMSPC */
+		exif_data_unref(ed);
 	}
 #endif /* FIM_WITH_LIBEXIF */
 #endif /* HAVE_NEW_EXIF */
@@ -491,7 +501,7 @@ jpeg_init(FILE *fp, const fim_char_t *filename, unsigned int page,
         if(fim_jerr)goto oops;
   //      if(h->jerr.msg_code)goto oops;
 	h->cinfo.src = &thumbnail_mgr;
-	jpeg_read_header(&h->cinfo, FIM_FBI_TRUE);
+	jpeg_read_header(&h->cinfo, FIM_JPEGLIB_TRUE);
         if(fim_jerr)goto oops;
 //        if(h->jerr.msg_code)goto oops;
     }
@@ -517,6 +527,7 @@ jpeg_init(FILE *fp, const fim_char_t *filename, unsigned int page,
     return h;
 oops:
 std::cerr << "OOPS: problems decoding "<< filename <<"...\n";
+    if( h ) jpeg_destroy_decompress (&h->cinfo);
     if( h && h->thumbnail) fim_free(h->thumbnail);
     if( h ) fim_free(h);
     fim_jerr=0;/* ready for the next */
@@ -543,6 +554,7 @@ jpeg_done(void *data)
 	fim_fclose(h->infile);
     if (h->thumbnail)
 	fim_free(h->thumbnail);
+    load_free_extras(h->i);
     fim_free(h);
 }
 
@@ -556,7 +568,7 @@ static struct ida_loader jpeg_loader = {
     /*done:*/  jpeg_done,
 };
 
-static void __init init_rd(void)
+static void fim__init init_rd(void)
 {
     fim_load_register(&jpeg_loader);
 }
@@ -623,8 +635,8 @@ jpeg_write(FILE *fp, struct ida_image *img)
     cinfo.input_components = 3;
     cinfo.in_color_space = JCS_RGB;
     jpeg_set_defaults(&cinfo);
-    jpeg_set_quality(&cinfo, jpeg_quality, FIM_FBI_TRUE);
-    jpeg_start_compress(&cinfo, FIM_FBI_TRUE);
+    jpeg_set_quality(&cinfo, jpeg_quality, FIM_JPEGLIB_TRUE);
+    jpeg_start_compress(&cinfo, FIM_JPEGLIB_TRUE);
 
     for (i = 0, line = img->data; i < img->i.height; i++, line += img->i.width*3)
         jpeg_write_scanlines(&cinfo, &line, 1);
@@ -641,7 +653,7 @@ struct ida_writer jpeg_writer = {
     /*conf: */   jpeg_conf,
 };
 
-static void __init init_wr(void)
+static void fim__init init_wr(void)
 {
     fim_write_register(&jpeg_writer);
 }
