@@ -1,8 +1,8 @@
-/* $Id: FbiStuffLoader.h 116 2008-02-02 13:39:37Z dezperado $ */
+/* $LastChangedDate: 2022-11-20 02:50:16 +0100 (Sun, 20 Nov 2022) $ */
 /*
  FbiStuffLoader.h : fbi functions for loading files, modified for fim
 
- (c) 2008 Michele Martone
+ (c) 2008-2022 Michele Martone
  (c) 1998-2006 Gerd Knorr <kraxel@bytesex.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -24,29 +24,29 @@
  * */
 
 
-#ifndef FBI_STUFF_LOADER_FBI_H
-#define FBI_STUFF_LOADER_FBI_H
-
-#include "fim.h"
+#ifndef FIM_STUFF_LOADER_H
+#define FIM_STUFF_LOADER_H
 
+#include "fim_string.h"
+#include "fim_types.h"
 #include "FbiStuffList.h"
 
 //#include "list.h"
 #ifdef USE_X11
 # include <X11/Intrinsic.h>
-#endif
+#endif /* USE_X11 */
 
 namespace fim
 {
 
 enum ida_extype {
     EXTRA_COMMENT = 1,
-    EXTRA_EXIF    = 2,
+    EXTRA_EXIF    = 2
 };
 
 struct ida_extra {
     enum ida_extype   type;
-    unsigned char     *data;
+    fim_byte_t     *data;
     unsigned int      size;
     struct ida_extra  *next;
 };
@@ -58,15 +58,21 @@ struct ida_image_info {
     unsigned int      dpi;
     unsigned int      npages;
     struct ida_extra  *extra;
+#ifdef FIM_EXPERIMENTAL_ROTATION
+    unsigned int      fim_extra_flags;/* FIXME : unclean: regard this as a hack (flag set on a rotated image) */
+#endif /* FIM_EXPERIMENTAL_ROTATION */
 
     int               thumbnail;
     unsigned int      real_width;
     unsigned int      real_height;
+#if FIM_EXPERIMENTAL_IMG_NMSPC
+    fim::Namespace *nsp;
+#endif /* FIM_EXPERIMENTAL_IMG_NMSPC */
 };
 
 struct ida_image {
     struct ida_image_info  i;
-    unsigned char          *data;
+    fim_byte_t          *data;
 };
 struct ida_rect {
     int x1,y1,x2,y2;
@@ -74,43 +80,48 @@ struct ida_rect {
 
 /* load image files */
 struct ida_loader {
-    char  *magic;
+    //const fim_byte_t  *magic;
+    const fim_char_t  *magic;
     int   moff;
     int   mlen;
-    char  *name;
-    void* (*init)(FILE *fp, char *filename, unsigned int page,
+    const fim_char_t  *name;
+    void* (*init)(FILE *fp, const fim_char_t *filename, unsigned int page,
 		  struct ida_image_info *i, int thumbnail);
-    void  (*read)(unsigned char *dst, unsigned int line, void *data);
+    void  (*read)(fim_byte_t *dst, unsigned int line, void *data);
     void  (*done)(void *data);
     struct list_head list;
 };
 
 /* filter + operations */
 struct ida_op {
-    char  *name;
-    void* (*init)(struct ida_image *src, struct ida_rect *rect,
+    const fim_char_t  *name;
+    void* (*init)(const struct ida_image *src, struct ida_rect *rect,
 		  struct ida_image_info *i, void *parm);
-    void  (*work)(struct ida_image *src, struct ida_rect *rect,
-		  unsigned char *dst, int line,
+    void  (*work)(const struct ida_image *src, struct ida_rect *rect,
+		  fim_byte_t *dst, int line,
 		  void *data);
     void  (*done)(void *data);
 };
 
+#if FIM_WANT_OBSOLETE
 void* op_none_init(struct ida_image *src, struct ida_rect *rect,
 		   struct ida_image_info *i, void *parm);
+#endif /* FIM_WANT_OBSOLETE */
 void  op_none_done(void *data);
+#if FIM_WANT_OBSOLETE
 void  op_free_done(void *data);
+#endif /* FIM_WANT_OBSOLETE */
 
 #ifdef USE_X11
 /* save image files */
 struct ida_writer {
-    char  *label;
-    char  *ext[8];
+    const fim_char_t  *label;
+    const fim_char_t  *ext[8];
     int   (*write)(FILE *fp, struct ida_image *img);
     int   (*conf)(Widget widget, struct ida_image *img);
     struct list_head list;
 };
-#endif
+#endif /* USE_X11 */
 
 /* ----------------------------------------------------------------------- */
 /* resolution                                                              */
@@ -122,16 +133,16 @@ struct ida_writer {
 /* ----------------------------------------------------------------------- */
 
 /* helpers */
-void load_bits_lsb(unsigned char *dst, unsigned char *src, int width,
+void load_bits_lsb(fim_byte_t *dst, fim_byte_t *src, int width,
 		   int on, int off);
-void load_bits_msb(unsigned char *dst, unsigned char *src, int width,
+void load_bits_msb(fim_byte_t *dst, fim_byte_t *src, int width,
 		   int on, int off);
-void load_gray(unsigned char *dst, unsigned char *src, int width);
-void load_graya(unsigned char *dst, unsigned char *src, int width);
-void load_rgba(unsigned char *dst, unsigned char *src, int width);
+void load_gray(fim_byte_t *dst, fim_byte_t *src, int width);
+void load_graya(fim_byte_t *dst, fim_byte_t *src, int width);
+void load_rgba(fim_byte_t *dst, fim_byte_t *src, int width);
 
 int load_add_extra(struct ida_image_info *info, enum ida_extype type,
-		   unsigned char *data, unsigned int size);
+		   fim_byte_t *data, unsigned int size);
 struct ida_extra* load_find_extra(struct ida_image_info *info,
 				  enum ida_extype type);
 int load_free_extras(struct ida_image_info *info);
@@ -139,23 +150,25 @@ int load_free_extras(struct ida_image_info *info);
 /* ----------------------------------------------------------------------- */
 
 /* other */
-extern int debug;
+//extern int debug;
 extern struct ida_loader ppm_loader;
 extern struct ida_loader sane_loader;
 extern struct ida_writer ps_writer;
 extern struct ida_writer jpeg_writer;
 
 /* lists */
-#define __init __attribute__ ((constructor))
-#define __fini __attribute__ ((destructor))
+#define fim__init __attribute__ ((constructor))
+#define fim__fini __attribute__ ((destructor))
 
 extern struct list_head loaders;
 extern struct list_head writers;
 
-void load_register(struct ida_loader *loader);
-void write_register(struct ida_writer *writer);
+void fim_load_register(struct ida_loader *loader);
+void fim_write_register(struct ida_writer *writer);
+void fim_loaders_to_stderr(FILE * stream);
 
+string fim_loaders_to_string(void);
 }
-#endif
+#endif /* FIM_STUFF_LOADER_H */
 
 
