@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2015-12-19 01:44:41 +0100 (Sat, 19 Dec 2015) $ */
+/* $LastChangedDate: 2024-03-22 16:34:38 +0100 (Fri, 22 Mar 2024) $ */
 /*
  FbiStuffTiff.cpp : fbi functions for TIFF files, modified for fim
 
- (c) 2007-2015 Michele Martone
+ (c) 2007-2024 Michele Martone
  (c) 1998-2006 Gerd Knorr <kraxel@bytesex.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 
 
 #include <stdio.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <string.h>
 #include <inttypes.h>
 #include <tiffio.h>
@@ -33,9 +33,10 @@
 #include "FbiStuff.h"
 #include "FbiStuffLoader.h"
 #ifdef USE_X11
-# include "viewer.h"
+ #include "viewer.h"
 #endif /* USE_X11 */
 
+#define FIM_TIFF_ERRVAL -1
 namespace fim
 {
 
@@ -44,11 +45,11 @@ struct tiff_state {
     //char           emsg[FIM_LIBERR_BUFSIZE];
     tdir_t         ndirs;     /* Number of directories                     */
                               /* (could be interpreted as number of pages) */
-    uint32         width,height;
-    uint16         config,nsamples,depth,fillorder,photometric;
-    uint32*        row;
-    uint32*        image;
-    uint16         resunit;
+    uint32_t         width,height;
+    uint16_t         config,nsamples,depth,fillorder,photometric;
+    uint32_t*        row;
+    uint32_t*        image;
+    uint16_t         resunit;
     float          xres,yres;
 };
 
@@ -74,15 +75,21 @@ tiff_init(FILE *fp, const fim_char_t *filename, unsigned int page,
     /* Select requested directory (page) */
     if (!TIFFSetDirectory(h->tif, (tdir_t)page))
         goto oops;
-    
-    TIFFGetField(h->tif, TIFFTAG_IMAGEWIDTH,      &h->width);
-    TIFFGetField(h->tif, TIFFTAG_IMAGELENGTH,     &h->height);
-    TIFFGetField(h->tif, TIFFTAG_PLANARCONFIG,    &h->config);
-    TIFFGetField(h->tif, TIFFTAG_SAMPLESPERPIXEL, &h->nsamples);
-    TIFFGetField(h->tif, TIFFTAG_BITSPERSAMPLE,   &h->depth);
-    TIFFGetField(h->tif, TIFFTAG_FILLORDER,       &h->fillorder);
-    TIFFGetField(h->tif, TIFFTAG_PHOTOMETRIC,     &h->photometric);
-    h->row = (uint32*)fim_malloc(TIFFScanlineSize(h->tif));
+    if(FIM_TIFF_ERRVAL==TIFFGetField(h->tif, TIFFTAG_IMAGEWIDTH,      &h->width))
+			goto oops;
+    if(FIM_TIFF_ERRVAL==TIFFGetField(h->tif, TIFFTAG_IMAGELENGTH,     &h->height))
+			goto oops;
+    if(FIM_TIFF_ERRVAL==TIFFGetField(h->tif, TIFFTAG_PLANARCONFIG,    &h->config))
+			goto oops;
+    if(FIM_TIFF_ERRVAL==TIFFGetField(h->tif, TIFFTAG_SAMPLESPERPIXEL, &h->nsamples))
+			goto oops;
+    if(FIM_TIFF_ERRVAL==TIFFGetField(h->tif, TIFFTAG_BITSPERSAMPLE,   &h->depth))
+			goto oops;
+    if(FIM_TIFF_ERRVAL==TIFFGetField(h->tif, TIFFTAG_FILLORDER,       &h->fillorder))
+			goto oops;
+    if(FIM_TIFF_ERRVAL==TIFFGetField(h->tif, TIFFTAG_PHOTOMETRIC,     &h->photometric))
+			goto oops;
+    h->row = (uint32_t*)fim_malloc(TIFFScanlineSize(h->tif));
     if(!h->row)goto oops;
     if (FbiStuff::fim_filereading_debug())
 #ifndef FIM_PRId32
@@ -106,14 +113,11 @@ tiff_init(FILE *fp, const fim_char_t *filename, unsigned int page,
 	 * progressive loading and decode everything here */
 	if (FbiStuff::fim_filereading_debug())
 	    FIM_FBI_PRINTF("tiff: reading whole image [TIFFReadRGBAImage]\n");
-	h->image=(uint32*)fim_malloc(4*h->width*h->height);
+	h->image=(uint32_t*)fim_malloc(4*h->width*h->height);
         if(!h->image)goto oops;
-	TIFFReadRGBAImage(h->tif, h->width, h->height, h->image, 0);
     } else {
 	if (FbiStuff::fim_filereading_debug())
 	    FIM_FBI_PRINTF("tiff: reading scanline by scanline\n");
-	h->row = (uint32*)fim_malloc(TIFFScanlineSize(h->tif));
-        if(!h->row)goto oops;
     }
 
     i->width  = h->width;
@@ -153,7 +157,7 @@ tiff_read(fim_byte_t *dst, unsigned int line, void *data)
 
     if (h->image) {
 	/* loaded whole image using TIFFReadRGBAImage() */
-	uint32 *row = h->image + h->width * (h->height - line -1);
+	uint32_t *row = h->image + h->width * (h->height - line -1);
 	load_rgba(dst,(fim_byte_t*)row,h->width);
 	return;
     }
@@ -231,7 +235,7 @@ static struct ida_loader tiff2_loader = {
     /*done:*/  tiff_done,
 };
 
-static void __init init_rd(void)
+static void fim__init init_rd(void)
 {
     fim_load_register(&tiff1_loader);
     fim_load_register(&tiff2_loader);
@@ -284,7 +288,7 @@ static struct ida_writer tiff_writer = {
     /*  write:*/  tiff_write,
 };
 
-static void __init init_wr(void)
+static void fim__init init_wr(void)
 {
     fim_write_register(&tiff_writer);
 }
