@@ -1,8 +1,7 @@
-/* $LastChangedDate: 2016-09-15 18:20:27 +0200 (Thu, 15 Sep 2016) $ */
-/*
- FimWindow.cpp : Fim's own windowing system
+/* $LastChangedDate: 2022-11-16 02:27:16 +0100 (Wed, 16 Nov 2022) $ */
+/* FimWindow.cpp : Fim's own windowing system 
 
- (c) 2007-2016 Michele Martone
+ (c) 2007-2022 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,25 +17,21 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
-
-/*
- *	This code is still experimental and programmed in great hurry.
- *	FIXME : there are bugs.
- */
-
 #include "fim.h"
 
 #ifdef FIM_WINDOWS
 
+#define FIM_WINDOW_PROBLEM "FimWindow Internal Error"
+
 namespace fim
 {
-        fim::string FimWindow::fcmd_cmd(const std::vector<fim::string> &args)
+        fim_cxr FimWindow::fcmd_cmd(const args_t&args)
         {
+#if FIM_DISABLE_WINDOW_SPLITTING
+		return "Warning: window control (splitting, etc.) is disabled. It may be re-enabled in a future version.\n";
+#else /* FIM_DISABLE_WINDOW_SPLITTING */
 		unsigned int i=0;
 		fim_err_t rc=0;/*return code*/
-#if FIM_DISABLE_WINDOW_SPLITTING
-		return "Warning: window control (splitting, etc.) is disabled. It shall be re-enabled in a future version.\n";
-#endif /* FIM_DISABLE_WINDOW_SPLITTING */
 #ifdef FIM_AUTOCMDS
 		fim::string c=getGlobalIntVariable(FIM_VID_FILENAME);
 		// note that an autocommand on a transient object is lethal
@@ -47,8 +42,9 @@ namespace fim
 		{
 		while(i<args.size())
                 {
-			string cmd=args[i];
-			const fim_coo_t es=FIM_CNS_WENLARGE_STEPS_DEFAULT;
+			fim_cmd_id cmd=args[i];
+			FIM_CONSTEXPR fim_coo_t es = FIM_CNS_WENLARGE_STEPS_DEFAULT;
+
 			if(cmd == "split" || cmd == "hsplit")
 			{
 				hsplit();
@@ -119,14 +115,15 @@ namespace fim
 		// note that an autocommand on a transient object is lethal
 		if(amroot_)
 		{ FIM_AUTOCMD_EXEC(FIM_ACM_POSTWINDOW,c); }
+#endif /* FIM_DISABLE_WINDOW_SPLITTING */
                 return FIM_CNS_EMPTY_RESULT;
         }
 
-	FimWindow::FimWindow(CommandConsole &c,const Rect& corners, Viewport* vp):
+	FimWindow::FimWindow(CommandConsole& c,DisplayDevice *displaydevice,const Rect& corners, Viewport* vp):
 #ifdef FIM_NAMESPACES
 	Namespace(&c,FIM_SYM_NAMESPACE_WINDOW_CHAR),
 #endif /* FIM_NAMESPACES */
-	corners_(corners),focus_(false),first_(FIM_NULL),second_(FIM_NULL),amroot_(false)
+	displaydevice_(displaydevice),corners_(corners),focus_(false),first_(FIM_NULL),second_(FIM_NULL),amroot_(false)
 	,viewport_(FIM_NULL),
 	commandConsole_(c)
 	{
@@ -143,17 +140,19 @@ namespace fim
 
 		}
 		else
-			viewport_=new Viewport( commandConsole_,  corners_ );
+			viewport_=new Viewport( commandConsole_, displaydevice,  corners_ );
 
 		if( viewport_ == FIM_NULL )
 		       	throw FIM_E_NO_MEM;
 	}
 
 //#ifdef FIM_UNDEFINED
-	FimWindow::FimWindow(const FimWindow & root):
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	FimWindow::FimWindow(const FimWindow& root):
 #ifdef FIM_NAMESPACES
 			Namespace(root),
 #endif /* FIM_NAMESPACES */
+		displaydevice_(root.displaydevice_),
 		corners_(root.corners_),focus_(root.focus_),first_(root.first_),second_(root.second_),amroot_(false), viewport_(FIM_NULL),commandConsole_(root.commandConsole_)
 	{
 		/*
@@ -162,13 +161,15 @@ namespace fim
 		 *
 		 *  Note : this member function is useless, and should be kept private :D
 		 */
-		viewport_=new Viewport( commandConsole_, corners_ );
+		viewport_=new Viewport( commandConsole_, root.displaydevice_, corners_ );
 
 		if( viewport_ == FIM_NULL )
 		       	throw FIM_E_NO_MEM;
 	}
 //#endif
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::issplit(void)const
 	{
 		/*
@@ -176,7 +177,9 @@ namespace fim
 		 * */
 		return ( first_ && second_ ) ;
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::isleaf(void)const
 	{
 		/*
@@ -198,7 +201,9 @@ namespace fim
 		 */
 		return ( ! first_ && ! second_ ) ;
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::isvalid(void)const
 	{	
 		/*
@@ -206,7 +211,9 @@ namespace fim
 		 * */
 		return !(( first_ && ! second_ ) || ( ! first_ && second_ ) );
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::ishsplit(void)const
 	{
 		/*
@@ -219,7 +226,9 @@ namespace fim
 		 */
 		return ( issplit() && focused().corners_.x_==shadowed().corners_.x_ ) ;
 	}
+#endif
 	
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::isvsplit(void)const
 	{
 		/*
@@ -233,7 +242,7 @@ namespace fim
 		return ( issplit() && focused().corners_.y_==shadowed().corners_.y_ ) ;
 	}
 	
-	const FimWindow & FimWindow::c_focused(void)const
+	const FimWindow& FimWindow::c_focused(void)const
 	{
 		/*
 		 * return a const reference to the focused window
@@ -245,8 +254,10 @@ namespace fim
 			return first_->c_focused();
 		else return second_->c_focused();
 	}
+#endif
 
-	FimWindow & FimWindow::focused(void)const
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	FimWindow& FimWindow::focused(void)const
 	{
 		/*
 		 * return a reference to the focused window
@@ -258,8 +269,10 @@ namespace fim
 			return *first_;
 		else return *second_;
 	}
+#endif
 
-	FimWindow & FimWindow::upper(void)
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	FimWindow& FimWindow::upper(void)
 	{
 		/*
 		 * return a reference to the upper window
@@ -269,7 +282,7 @@ namespace fim
 		return *first_;
 	}
 
-	FimWindow & FimWindow::lower(void)
+	FimWindow& FimWindow::lower(void)
 	{
 		/*
 		 * return a reference to the lower window
@@ -279,7 +292,7 @@ namespace fim
 		return *second_;
 	}
 
-	FimWindow & FimWindow::left(void)
+	FimWindow& FimWindow::left(void)
 	{
 		/*
 		 * return a reference to the left window
@@ -289,7 +302,7 @@ namespace fim
 		return *first_;
 	}
 
-	FimWindow & FimWindow::right(void)
+	FimWindow& FimWindow::right(void)
 	{
 		/*
 		 * return a reference to the right window
@@ -298,8 +311,10 @@ namespace fim
 		if(!isvsplit())/* temporarily, for security reasons */throw FIM_E_WINDOW_ERROR;
 		return *second_;
 	}
+#endif
 
-	FimWindow & FimWindow::shadowed(void)const
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	FimWindow& FimWindow::shadowed(void)const
 	{
 		/*
 		 * return a const reference to the right window
@@ -311,8 +326,10 @@ namespace fim
 			return *first_;
 		else return *second_;
 	}
+#endif
 
-	const FimWindow & FimWindow::c_shadowed(void)const
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	const FimWindow& FimWindow::c_shadowed(void)const
 	{
 		/*
 		 * return a const reference to the shadowed window
@@ -324,6 +341,7 @@ namespace fim
 			return first_->c_shadowed();
 		else return second_->c_shadowed();
 	}
+#endif
 
 	void FimWindow::setroot(void)
 	{
@@ -333,6 +351,7 @@ namespace fim
 		amroot_=true;
 	}
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	void FimWindow::split(void)
 	{
 		/*
@@ -340,6 +359,7 @@ namespace fim
 		 * */
 		hsplit();
 	}
+#endif
 
 #if 0
 	void FimWindow::print_focused(void)
@@ -368,6 +388,7 @@ namespace fim
 	}
 #endif
 	
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	void FimWindow::hsplit(void)
 	{
 		/*
@@ -381,8 +402,8 @@ namespace fim
 		 * */
 		if(isleaf())
 		{
-			first_  = new FimWindow( commandConsole_, this->corners_.hsplit(Rect::Upper),viewport_);
-			second_ = new FimWindow( commandConsole_, this->corners_.hsplit(Rect::Lower),viewport_);
+			first_  = new FimWindow( commandConsole_, displaydevice_, this->corners_.hsplit(Rect::Upper),viewport_);
+			second_ = new FimWindow( commandConsole_, displaydevice_, this->corners_.hsplit(Rect::Lower),viewport_);
 			if(viewport_ && first_ && second_)
 			{
 #define FIM_COOL_WINDOWS_SPLITTING 0
@@ -409,8 +430,8 @@ namespace fim
 		 * */
 		if(isleaf())
 		{
-			first_  = new FimWindow( commandConsole_, this->corners_.vsplit(Rect::Left ),viewport_);
-			second_ = new FimWindow( commandConsole_, this->corners_.vsplit(Rect::Right),viewport_);
+			first_  = new FimWindow( commandConsole_, displaydevice_, this->corners_.vsplit(Rect::Left ),viewport_);
+			second_ = new FimWindow( commandConsole_, displaydevice_, this->corners_.vsplit(Rect::Right),viewport_);
 			if(viewport_ && first_ && second_)
 			{
 #if     FIM_COOL_WINDOWS_SPLITTING
@@ -422,7 +443,9 @@ namespace fim
 		}
 		else focused().vsplit();
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::swap(void)
 	{
 		/*
@@ -562,7 +585,9 @@ namespace fim
 			return Up;
 		return move;
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	FimWindow::Moves FimWindow::move_focus(Moves move)
 	{
 		/*
@@ -649,6 +674,7 @@ namespace fim
 		 */
 		return focus_ = !focus_;
 	}
+#endif
 
 	fim_coo_t Rect::height(void)const
 	{
@@ -660,6 +686,7 @@ namespace fim
 		return h_ ;
 	}
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	fim_coo_t Rect::setwidth(fim_coo_t w)
 	{
 		/*
@@ -680,6 +707,7 @@ namespace fim
 		 */
 		return h_=h;
 	}
+#endif
 
 	fim_coo_t Rect::width(void)const
 	{
@@ -692,6 +720,7 @@ namespace fim
 		return w_ ;
 	}
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	fim_coo_t Rect::setxorigin(fim_coo_t x)
 	{
 		/*
@@ -711,6 +740,7 @@ namespace fim
 		 */
 		return y_=y ;
 	}
+#endif
 
 	fim_coo_t Rect::xorigin(void)const
 	{
@@ -732,7 +762,8 @@ namespace fim
 		return y_ ;
 	}
 
-	bool FimWindow::can_vgrow(const FimWindow & window, fim_coo_t howmuch)
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	bool FimWindow::can_vgrow(const FimWindow& window, fim_coo_t howmuch)const
 	{
 		/*
 		 * Assuming that the argument window is a contained one, 
@@ -748,7 +779,7 @@ namespace fim
 		return window.corners_.height() + howmuch + vspacing  < corners_.height();
 	}
 
-	bool FimWindow::can_hgrow(const FimWindow & window, fim_coo_t howmuch)
+	bool FimWindow::can_hgrow(const FimWindow& window, fim_coo_t howmuch)const
 	{
 		/*
 		 * Assuming that the argument window is a contained one, 
@@ -762,7 +793,9 @@ namespace fim
 		 * +--------+
 		 */		return window.corners_.width() + howmuch + hspacing   < corners_.width();
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::operator==(const FimWindow&window)const
 	{
 		/*
@@ -772,7 +805,9 @@ namespace fim
 		 */
 		return corners_==window.corners_;
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	int FimWindow::count_hdivs(void)const
 	{
 		/*
@@ -795,7 +830,9 @@ namespace fim
 		 * */
 		return (isleaf()|| !isvsplit())?1: first_->count_vdivs()+ second_->count_vdivs();
 	}
+#endif /* FIM_DISABLE_WINDOW_SPLITTING */
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::normalize(void)
 	{
 		/*
@@ -910,7 +947,9 @@ namespace fim
 			return FIM_ERR_NO_ERROR;
 		}
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	fim_err_t FimWindow::venlarge(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT)
 	{
 #if FIM_BUGGED_ENLARGE
@@ -960,7 +999,9 @@ namespace fim
 			}
 			return FIM_ERR_NO_ERROR;
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	fim_err_t FimWindow::henlarge(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT)
 	{
 		/*
@@ -1036,8 +1077,9 @@ namespace fim
 			// isleaf(void)
 			return FIM_ERR_NO_ERROR;
 	}
+#endif
 
-
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	fim_err_t FimWindow::vlgrow(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT)   {  return corners_.vlgrow(  units); } 
 	fim_err_t FimWindow::vlshrink(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT) {  return corners_.vlshrink(units); }
 	fim_err_t FimWindow::vugrow(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT)   {  return corners_.vugrow(  units); } 
@@ -1047,6 +1089,7 @@ namespace fim
 	fim_err_t FimWindow::hlshrink(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT) {  return corners_.hlshrink(units); }
 	fim_err_t FimWindow::hrgrow(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT)   {  return corners_.hrgrow(  units); } 
 	fim_err_t FimWindow::hrshrink(fim_coo_t units=FIM_CNS_WGROW_STEPS_DEFAULT) {  return corners_.hrshrink(units); }
+#endif /* FIM_DISABLE_WINDOW_SPLITTING */
 
 #if 0
 	void FimWindow::draw(void)const
@@ -1078,25 +1121,31 @@ namespace fim
 		bool re=false;//really redisplayed ? sometimes fim guesses it is not necessary
 		try
 		{
+#if !FIM_DISABLE_WINDOW_SPLITTING
 		if(isleaf())
+#endif
 		{
 			if(viewport_)
 				re=viewport_->redisplay();
 		}
+#if !FIM_DISABLE_WINDOW_SPLITTING
 		else
 		{
 			re |= focused().recursive_redisplay();
 			re |= shadowed().recursive_redisplay();
 		}
+#endif
 		}
 		catch(FimException e)
 		{
-			if( e != FIM_E_WINDOW_ERROR) ;// this would be bad..
+			if( e != FIM_E_WINDOW_ERROR) // this would be bad..
+		    		std::cerr << FIM_WINDOW_PROBLEM << std::endl;
 		}
 		return re;
 	}
 
 	// WARNING : SHOULD BE SURE VIEWPORT IS CORRECTLY INITIALIZED
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool FimWindow::recursive_display(void)const
 	{
 		/*
@@ -1118,10 +1167,12 @@ namespace fim
 		}
 		catch(FimException e)
 		{
-			if( e != FIM_E_WINDOW_ERROR) ;// this would be bad..
+			if( e != FIM_E_WINDOW_ERROR) // this would be bad..
+		   		std::cerr << FIM_WINDOW_PROBLEM << std::endl;
 		}
 		return re;
 	}
+#endif
 
 	Viewport * FimWindow::current_viewportp(void)const
 	{
@@ -1135,13 +1186,16 @@ namespace fim
 		 * ||   ||     |
 		 * +#===#+-----+
 		 */
+#if !FIM_DISABLE_WINDOW_SPLITTING
 		if(!isleaf()) 
 			return focused().current_viewportp();
+#endif
 
 		return viewport_;
 	}	
 
-	Viewport & FimWindow::current_viewport(void)const
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	Viewport& FimWindow::current_viewport(void)const
 	{
 		/*
 		 * returns a reference to the current window's viewport_.
@@ -1154,21 +1208,27 @@ namespace fim
 		 * ||   ||     |
 		 * +#===#+-----+
 		 */
+#if !FIM_DISABLE_WINDOW_SPLITTING
 		if(!isleaf())
 			return focused().current_viewport();
+#endif
 
-		if(!viewport_)/* temporarily, for security reasons throw FIM_E_TRAGIC*/; // isleaf(void)
+		if(!viewport_)/* temporarily, for security reasons throw FIM_E_TRAGIC*/ // isleaf(void)
+		    	std::cerr << FIM_WINDOW_PROBLEM << std::endl;
 
 		return *viewport_;
 	}	
+#endif
 
-	const Image *FimWindow::getImage(void)const
+#if 0
+	const Image* FimWindow::getImage(void)const
 	{
 		if( current_viewportp() )
 			return current_viewportp()->getImage();
 		else
 			return FIM_NULL;
 	}
+#endif
 
 	FimWindow::~FimWindow(void)
 	{
@@ -1194,12 +1254,14 @@ namespace fim
 		return bs;
 	}
 
-	FimWindow & FimWindow ::operator= (const FimWindow &w){return *this;/* a disabled assignment */}
+#if !FIM_DISABLE_WINDOW_SPLITTING
+	FimWindow& FimWindow ::operator= (const FimWindow& w){return *this;/* a disabled assignment */}
 
 	fim_bool_t FimWindow::need_redraw(void)const
 	{
 	       	return ( viewport_ && viewport_->need_redraw() );
 	}
+#endif
 
 	void FimWindow::should_redraw(enum fim_redraw_t sr)
        	{
@@ -1209,6 +1271,7 @@ namespace fim
 
 	/* Rect stuff */
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	Rect Rect::hsplit(Splitmode s){return split(s);}
 	Rect Rect::vsplit(Splitmode s){return split(s);}
 	Rect Rect::split(Splitmode s)
@@ -1235,6 +1298,7 @@ namespace fim
 	{
 		std::cout << x_ <<" " << y_  << " "<< w_ << " " << h_  << "\n";
 	}
+#endif
 
 	Rect::Rect(fim_coo_t x,fim_coo_t y,fim_coo_t w,fim_coo_t h):
 	x_(x), y_(y), w_(w), h_(h)
@@ -1244,6 +1308,7 @@ namespace fim
 
 	Rect::Rect(const Rect& rect): x_(rect.x_), y_(rect.y_), w_(rect.w_), h_(rect.h_){}
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	bool Rect::operator==(const Rect&rect)const
 	{
 		return x_==rect.x_ &&
@@ -1251,7 +1316,9 @@ namespace fim
 		w_==rect.w_ &&
 		h_==rect.h_;
 	}
+#endif
 
+#if !FIM_DISABLE_WINDOW_SPLITTING
 	fim_err_t Rect::vlgrow(fim_coo_t units)   { h_+=units; return FIM_ERR_NO_ERROR; } 
 	fim_err_t Rect::vlshrink(fim_coo_t units) { h_-=units; return FIM_ERR_NO_ERROR; }
 	fim_err_t Rect::vugrow(fim_coo_t units)   { y_-=units; h_+=units ; return FIM_ERR_NO_ERROR; } 
@@ -1261,6 +1328,7 @@ namespace fim
 	fim_err_t Rect::hrshrink(fim_coo_t units) { w_-=units; return FIM_ERR_NO_ERROR; }
 	fim_err_t Rect::hrgrow(fim_coo_t units)   { w_+=units; return FIM_ERR_NO_ERROR; } 
 	fim_err_t Rect::hlshrink(fim_coo_t units) { x_+=units; w_-=units ; return FIM_ERR_NO_ERROR; }
+#endif
 }
 #if 0
 /*
@@ -1299,4 +1367,3 @@ int main(void)
 }
 #endif
 #endif /* FIM_WINDOWS */
-
