@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2016-01-12 18:49:21 +0100 (Tue, 12 Jan 2016) $ */
+/* $LastChangedDate: 2022-10-02 20:11:31 +0200 (Sun, 02 Oct 2022) $ */
 /*
  FontServer.h : Font Server code from fbi, adapted for fim.
 
- (c) 2008-2016 Michele Martone
+ (c) 2008-2022 Michele Martone
  (c) 1998-2006 Gerd Knorr <kraxel@bytesex.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -29,17 +29,17 @@
 
 #include "fim.h"
 
-# include <errno.h>
+# include <cerrno>
 # include <unistd.h> //for FIM_NULL
-# include <stdio.h> //for stderr..
-# include <stdlib.h> //for exit..
-# include <string.h> //for strlen..
+# include <cstdio> //for stderr..
+# include <cstdlib> //for exit..
+# include <cstring> //for strlen..
 namespace fim
 {
 
-
 #ifdef FIM_USE_X11_FONTS
 # include <FSlib.h>
+    int swidth(void)const{return /*_FSXCharInfo::fim_fmf_**/width;}
 
 struct fs_font {
     Font               font;
@@ -59,7 +59,9 @@ struct fs_font {
 
 #else /* FIM_USE_X11_FONTS */
 
+#if FIM_FONT_MAGNIFY_FACTOR >  0
 #define fim_fmf FIM_FONT_MAGNIFY_FACTOR
+#endif /* FIM_FONT_MAGNIFY_FACTOR */
 
 typedef struct _FSXCharInfo {
     short       left;
@@ -68,7 +70,12 @@ typedef struct _FSXCharInfo {
     short       ascent;
     short       descent;
     /*unsigned short      attributes;*/
-    int swidth(void)const{return fim_fmf*width;}
+#if ( FIM_FONT_MAGNIFY_FACTOR <= 0 )
+    int fmf(void)const{extern fim_int fim_fmf_;return fim_fmf_;}
+#else /* FIM_FONT_MAGNIFY_FACTOR */
+    int fmf(void)const{return FIM_FONT_MAGNIFY_FACTOR;}
+#endif /* FIM_FONT_MAGNIFY_FACTOR */
+    int swidth(void)const{return fmf()*width;}
 } FSXCharInfo;
 
 typedef struct _FSXFontInfoHeader {
@@ -91,8 +98,13 @@ struct fs_font {
     int                maxenc,width,height;
     FSXCharInfo        **eindex;
     fim_byte_t      **gindex;
-    int swidth(void)const{return fim_fmf*width;}
-    int sheight(void)const{return fim_fmf*height;}
+#if ( FIM_FONT_MAGNIFY_FACTOR <= 0 )
+    int fmf(void)const{extern fim_int fim_fmf_;return fim_fmf_;}
+#else /* FIM_FONT_MAGNIFY_FACTOR */
+    int fmf(void)const{return FIM_FONT_MAGNIFY_FACTOR;}
+#endif /* FIM_FONT_MAGNIFY_FACTOR */
+    int swidth(void)const{return fmf()*width;}
+    int sheight(void)const{return fmf()*height;}
 };
 
 #endif  /* FIM_USE_X11_FONTS */
@@ -110,7 +122,7 @@ static const unsigned fs_masktab[] = {
 
 fim::string get_default_font_list(void);
 
-class FontServer
+class FontServer FIM_FINAL
 {
 
 
@@ -119,15 +131,15 @@ public:
 
 #if 1
 /* 20080507 unused, as default_font ? */
-static void fb_text_init1(const fim_char_t *font, struct fs_font **_f);
+static void fb_text_init1(const fim_char_t *font, struct fs_font **_f, fim_int vl=1);
 
-static struct fs_font* fs_consolefont(const fim_char_t **filename);
+static struct fs_font* fs_consolefont(const fim_char_t **filename, fim_int vl=1);
 #endif
 
 
 
 };
-void fim_free_fs_font(struct fs_font *f_); /* FIXME: temporarily here */
+void fim_free_fs_font(struct fs_font *f_);
 
 }
 
