@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2016-01-29 17:23:12 +0100 (Fri, 29 Jan 2016) $ */
+/* $LastChangedDate: 2024-04-08 00:08:57 +0200 (Mon, 08 Apr 2024) $ */
 /*
- common.cpp : Miscellaneous stuff..
+ common.cpp : Miscellaneous stuff.
 
- (c) 2007-2016 Michele Martone
+ (c) 2007-2024 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -64,7 +64,21 @@ void fim_touppers(fim_char_t *s)
 }
 */
 
-fim::string fim_dirname(const fim::string & arg)
+static fim::string fim_my_dirname(const fim::string& arg)
+{
+	size_t lds = arg.rfind(FIM_CNS_DIRSEP_CHAR);
+	if (lds != std::string::npos)
+	{
+		while (lds > 0 && arg[lds-1] == FIM_CNS_DIRSEP_CHAR)
+			--lds;
+		fim::string dir(arg);
+		dir.erase(lds);
+		return dir;
+	}
+	return arg;
+}
+
+fim::string fim_dirname(const fim::string& arg)
 {
 #ifdef HAVE_LIBGEN_H
 	fim_char_t buf[FIM_PATH_MAX];
@@ -72,20 +86,109 @@ fim::string fim_dirname(const fim::string & arg)
 	buf[FIM_PATH_MAX-1]='\0';
 	return dirname(buf);
 #else /* HAVE_LIBGEN_H */
-	return "";//FIXME
+	return fim_my_dirname(arg);
 #endif /* HAVE_LIBGEN_H */
 }
-	fim::string fim_shell_arg_escape(const fim::string & arg)
+
+fim::string fim_auto_quote(const fim::string& arg, int quoted)
+{
+	char qc = 0;
+	fim::string ear=arg;
+	fim::string res=FIM_CNS_EMPTY_STRING;
+
+	if (quoted == 0)
 	{
-		// FIXME: this escaping function is NOT safe; this code shall only serve as a placeholder for a better one.
-		fim::string ear=arg;
-		fim::string res=FIM_CNS_EMPTY_STRING;
-		res+="'";
-		ear.substitute("'","'\\''");
-		res+=ear;
-		res+="'";
+		quoted = 2;
+		if (arg.size() == 1)
+			if ( arg[0] == '\"' )
+				quoted = 1;
+		// ...
+	}
+	if (quoted == 1)
+		qc = '\'';
+	if (quoted == 2)
+		qc = '"';
+	if (quoted == 3)
+	{
+		const char * as = arg.c_str();
+		const auto sqi = strchr(as,'\'');
+		const auto dqi = strchr(as,'"');
+		
+		if (!sqi || ( sqi && dqi && sqi< dqi) )
+			qc = '\'';
+		if (!dqi || ( sqi && dqi && sqi>=dqi) )
+			qc = '"';
+		res = qc + ear + qc;
 		return res;
 	}
+	if(qc)
+		res+=qc;
+	if(qc == '\'')
+		ear.substitute("'","\\'");
+	if(qc == '\"')
+		ear.substitute("\"","\\\"");
+	res+=ear;
+	if(qc)
+		res+=qc;
+	return res;
+}
+
+fim::string fim_shell_arg_escape(const fim::string& arg, bool quoted)
+{
+	fim::string ear=arg;
+	fim::string res=FIM_CNS_EMPTY_STRING;
+
+	if(quoted)
+		res+="'";
+	ear.substitute("'","\\'");
+	res+=ear;
+	if(quoted)
+		res+="'";
+	return res;
+}
+
+fim::string fim_key_escape(const fim::string uk)
+{
+	// escape for display
+	std::ostringstream oss;
+	oss << "[" << uk << "]";
+	return oss.str();
+}
+
+fim::string fim_text_to_man(const fim::string ts)
+{
+	// this is *not* the reverse of fim_man_to_text
+	auto ms {ts};
+#if FIM_USE_CXX_REGEX
+	ms.substitute("\\\\'", "\\e'", 0); // without this, literal \' would get translated as ´ (acute accent) 
+#else /* FIM_USE_CXX_REGEX */
+	ms.substitute("\\'", "\\e'", 0); // without this, literal \' would get translated as ´ (acute accent) 
+#endif /* FIM_USE_CXX_REGEX */
+	return ms;
+}
+
+fim::string fim_man_to_text(const fim::string ms, bool keep_nl)
+{
+	fim::string ts = ms;
+#if FIM_USE_CXX_REGEX
+	ts.substitute("(\\\\fB)(.*?)(\\\\fP)", "$2", 0);
+	ts.substitute("(\n\\.B)(.*?)(\n)", "$2 ", 0);
+	ts.substitute("(\\\\fR\\\\fI)(.*?)(\\\\fR)", "$2", 0);
+	if(!keep_nl)
+		ts.substitute("(\n)", " ", 0);
+#else /* FIM_USE_CXX_REGEX */
+	/* workaround */
+	ts.substitute("\\\\fB", "", 0);
+	ts.substitute("\\\\fP", "", 0);
+	ts.substitute("\\\\fR", "", 0);
+	ts.substitute("\\\\fI", "", 0);
+	ts.substitute("\\.B", " ", 0);
+	if(!keep_nl)
+		ts.substitute("(\n)", " ");
+#endif /* FIM_USE_CXX_REGEX */
+	ts.substitute("\\\\:", "");
+	return ts;
+}
 
 void fim_perror(const fim_char_t *s)
 {
@@ -104,6 +207,7 @@ size_t fim_strlen(const fim_char_t *str)
 	return strlen(str);
 }
 
+#if 0
 void trhex(fim_char_t *str)
 {
 	/*	
@@ -148,6 +252,7 @@ void trhex(fim_char_t *str)
 ret:
 	return;
 }
+#endif
 
 void trec(fim_char_t *str,const fim_char_t *f,const fim_char_t*t)
 {
@@ -158,6 +263,7 @@ void trec(fim_char_t *str,const fim_char_t *f,const fim_char_t*t)
 	 * 
 	 *	this function could be optimized.
 	 *	20090520 hex translation in
+	 *	20221204 not sure about whether hex translation is useful
 	 */
 	int tl;
 	fim_char_t*_p=FIM_NULL;
@@ -182,19 +288,13 @@ void trec(fim_char_t *str,const fim_char_t *f,const fim_char_t*t)
 			 && _p[2] && isxdigit(toupper(_p[2])) 
 			 && _p[3] && isxdigit(toupper(_p[3]))  )
 		{
-			unsigned int hc;
-			fim_char_t hb[3];
-			fim_char_t *pp;
-			hb[2]=0;
-			hb[0]=toupper(_p[2]);
-			hb[1]=toupper(_p[3]);
-			hc=(fim_byte_t)strtol(hb,FIM_NULL,FIM_PRINTINUM_BUFSIZE);
+			unsigned int hc = (toupper(_p[2])-'0')*16 + (toupper(_p[3])-'0');
 			*_p=hc;
 			/*	
 				\xFF
 				^-_p^-_p+4
 			*/
-			pp=_p+4;
+			fim_char_t *pp=_p+4;
 			while(*pp){pp[-3]=*pp;++pp;}
 			pp[-3]='\0';
 		}
@@ -202,7 +302,7 @@ void trec(fim_char_t *str,const fim_char_t *f,const fim_char_t*t)
 #endif
 		while(*fp)
 		{
-			//  if the following charcter is backslash-escaped and is in our from-list ..
+			//  if the following character is backslash-escaped and is in our from-list ..
 			if( *_p == '\\' && *(_p+1) == *fp )
 			{
 				fim_char_t*pp;
@@ -230,6 +330,7 @@ ret:
 	return;
 }
 
+#if FIM_SHALL_BUFFER_STDIN
 	fim_byte_t* slurp_binary_FD(FILE* fd, size_t  *rs)
 	{
 			/*
@@ -246,7 +347,7 @@ ret:
 				fim_byte_t *tb;
 				// if(nrb==inc) a full read. let's try again
 				// else we assume this is the last read (could not be true, of course)
-				tb=(fim_byte_t*)realloc(buf,rb+=nrb);
+				tb=(fim_byte_t*)fim_realloc(buf,rb+=nrb);
 				if(tb!=FIM_NULL)
 					buf=tb;
 				else
@@ -259,16 +360,14 @@ ret:
 ret:
 			return buf;
 	}
+#endif /* FIM_SHALL_BUFFER_STDIN */
 
 	fim_char_t* slurp_binary_fd(int fd,int *rs)
 	{
 			/*
-			 * If badly tuned, this code is a true allocator grinder :)
-			 *
-			 * slurps a binary file (possibly a stream) and returns the allocated memory.
-			 *
-			 * FIXME : use stat if possible.
-			 * FIXME : it is not throughly tested
+			 * slurps a string.
+			 * puts length of non-NUL bytes in *rs.
+			 * TODO: rename to slurp_string().
 			 * */
 			fim_char_t	*buf=FIM_NULL;
 			int	inc=FIM_FILE_BUF_SIZE,rb=0,nrb=0;
@@ -280,11 +379,11 @@ ret:
 				fim_char_t *tb;
 				// if(nrb==inc) a full read. let's try again
 				// else we assume this is the last read (could not be true, of course)
-				tb=(fim_char_t*)realloc(buf,rb+=nrb);
+				tb=(fim_char_t*)fim_realloc(buf,(rb+=nrb)+1);
 				if(tb!=FIM_NULL)
 					buf=tb;
 				else
-					{rb-=nrb;continue;}
+					{rb-=nrb;buf[rb]=FIM_SYM_CHAR_NUL;continue;}
 			}
 			if(rs)
 				*rs=rb;
@@ -313,19 +412,48 @@ ret:
 		return fim::string(file.c_str());
 	}
 
-#if 0
-	void append_to_file(fim::string filename, fim::string lines)
+	bool write_to_file(fim::string filename, fim::string lines, bool append)
 	{
 		std::ofstream fs;
-		fs.open(filename.c_str(),std::ios::app|std::ios::binary);
+
+		fs.open(filename.c_str(), std::ios::out | ( append ? std::ios::app : std::ios::binary ));
+
+		if(fs.rdstate()!=std::ios_base::goodbit)
+		{
+			cout << "Error opening file " << filename << " for writing !\n";
+			goto err;
+		}
+
 		if(fs.is_open())
 		{
-			fs << "\""<< lines.c_str()<<"\"" ;
+			fs << lines.c_str();
+
+			if(fs.rdstate()!=std::ios_base::goodbit)
+			{
+				cout << "Error writing to file " << filename << " !\n";
+				goto err;
+			}
+
+			fs.close();
+
+			if(fs.rdstate()!=std::ios_base::goodbit)
+			{
+				cout << "Error closing file " << filename << " !\n";
+				goto err;
+			}
 		}
-		fs.close();
+		else 
+		{
+			cout << "Error after opening file " << filename << " !\n";
+			goto err;
+		}
+#if HAVE_SYNC
 		sync();
+#endif /* HAVE_SYNC */
+		return true;
+err:
+		return false;
 	}
-#endif
 
 /*
  * Turns newline characters in NULs.
@@ -344,36 +472,39 @@ void chomp(fim_char_t *s)
  * */
 void sanitize_string_from_nongraph_except_newline(fim_char_t *s, int c)
 {	
-	int n=c;
+	const int n=c;
 	if(s)
 		while(*s && (c--||!n))
+		{
 			if(!isgraph(*s)&&*s!='\n')
-			{
-				*s=' ';
+				*s=' ',
 				++s;
-			}
 			else
 			       	++s;
+		}
 	return;
 }
 
 /*
  * cleans the input string terminating it when some non printable character is encountered
  * */
+
+#if FIM_WANT_OBSOLETE
 void sanitize_string_from_nongraph(fim_char_t *s, int c)
-{	
-	int n=c;
+{
+	const int n=c;
 	if(s)
 		while(*s && (c--||!n))
+		{
 			if(!isgraph(*s)||*s=='\n')
-			{
-				*s=' ';
+				*s=' ',
 				++s;
-			}
 			else
 			       	++s;
+		}
 	return;
 }
+#endif /* FIM_WANT_OBSOLETE */
 
 /*
  *	Allocation of a small string for storing the 
@@ -405,10 +536,7 @@ fim_char_t * dupnstr (fim_int n)
 	//allocation of a single string
 	fim_char_t *r = (fim_char_t*) fim_malloc (FIM_PRINTINUM_BUFSIZE);
 	if(!r){/*assert(r);*/throw FIM_E_NO_MEM;}
-	if(sizeof(fim_int)==sizeof(int))
-		sprintf(r,"%d",(int)n);
-	else
-		sprintf(r,"%lld",(long long int)n);
+	fim_snprintf_fim_int(r,n);
 	return (r);
 }
 
@@ -444,70 +572,98 @@ fim_char_t * dupsqstr (const fim_char_t* s)
  *	Allocation and duplication of a single string (not necessarily terminating)
  */
 #ifdef HAVE_FGETLN
+#if FIM_USE_CXX14
+auto                dupstrn (const fim_char_t* s, size_t l) /* Return type inference could be useful in a few places; this is just an example reminder. */
+#else /* FIM_USE_CXX14 */
 static fim_char_t * dupstrn (const fim_char_t* s, size_t l)
+#endif /* FIM_USE_CXX14 */
 {
 	fim_char_t *r = (fim_char_t*) fim_malloc (l + 1);
+	if(!r){/*assert(r);*/throw FIM_E_NO_MEM;}
 	strncpy(r,s,l);
 	r[l]='\0';
 	return (r);
 }
 #endif /* HAVE_FGETLN */
 
-static int pick_word(const fim_char_t *f, unsigned int *w)
+template<typename T>
+static int pick_word(const fim_char_t *f, T*wp)
 {
-	/*
-		FIXME : what is this ? :)
-	*/
 	int fd = open(f,O_RDONLY);
+
 	if(fd==-1)
 	       	goto ret;
-	if(read(fd,w,sizeof(int))==sizeof(int))
-		fd=0;
+
+	if(read(fd,wp,sizeof(T))==sizeof(T))
+		wp=FIM_NULL; // success
+	close(fd);
+	if(wp==FIM_NULL)
+		fd=0; // success
+	else
+		fd=-1;
 ret:
 	return fd;
 }
 
-/*
- * Will be improved, if needed.
- * */
 fim_int fim_rand(void)
 {
 	/*
 	 * Please don't use Fim random numbers for cryptographical purposes ;)
 	 * Note that we use /dev/urandom because it will never block on reading.
-	 * Reading from     /dev/random could instead block.
+	 * Reading from     /dev/random may block.
 	 * */
-	unsigned int w,r;
-	if(pick_word(FIM_LINUX_RAND_FILE,&w)==0)
-	       	r = (w%RAND_MAX);// TODO: are we sure that RAND_MAX corresponds to FIM_LINUX_RAND_FILE ?
+	fim_int w;
+	unsigned int u;
+
+	if(pick_word(FIM_LINUX_RAND_FILE,&u)==0)
+	       	w = (u%RAND_MAX);
 	else
 	{
 		srand(clock());
-		r = rand();
+		w = rand();
 	}
-	return (fim_int) r; /* FIXME: shall document this limitation  */
+	return w;
 }
 
-	bool regexp_match(const fim_char_t*s, const fim_char_t*r, int ignorecase, int ignorenewlines)
+	bool regexp_match(const fim_char_t*s, const fim_char_t*r, int ignorecase, int ignorenewlines, int globexception)
 	{
 		/*
 		 *	given a string s, and a Posix regular expression r, this
 		 *	member function returns true if there is match. false otherwise.
 		 */
+		bool match=false;
+		match=true;
+#if FIM_USE_CXX_REGEX
+		/*
+		 * we allow for the default match, in case of null regexp
+		 */
+		if(!r || !*r || !strlen(r))
+			goto ret;
+
+		/* fixup code for a mysterious bug (TODO: check if still relevant) */
+		if(globexception && *r=='*')
+		{
+			match = false;
+			goto ret;
+		}
+
+		try {
+			match = std::regex_search(s, std::regex(r, ignorecase ? ( std::regex_constants::extended | std::regex_constants::icase) : std::regex_constants::extended ));
+		} catch (const std::exception &) { match = false; }
+#else /* FIM_USE_CXX_REGEX */
+#if HAVE_REGEX_H
 		regex_t regex;		//should be static!!!
-		const int nmatch=1;	// we are satisfied with the first match, aren't we ?
+		FIM_CONSTEXPR int nmatch=1;	// we are satisfied with the first match, aren't we ?
 		regmatch_t pmatch[nmatch];
-		bool match=true;
 
 		/*
 		 * we allow for the default match, in case of null regexp
 		 */
-		if(!r || !strlen(r))
+		if(!r || !*r || !strlen(r))
 			goto ret;
 
-		/* fixup code for a mysterious bug
-		 */
-		if(*r=='*')
+		/* fixup code for a mysterious bug (TODO: check if still relevant) */
+		if(globexception && *r=='*')
 		{
 			match = false;
 			goto ret;
@@ -515,6 +671,7 @@ fim_int fim_rand(void)
 
 		if(ignorenewlines)
 		{
+			/* TODO: unfinished, maybe obsolete */
 			fim::string aux;
 			aux=s;
 		}
@@ -548,10 +705,13 @@ fim_int fim_rand(void)
 		};
 		regfree(&regex);
 		match = false;
+#endif /* HAVE_REGEX_H */
+#endif /* FIM_USE_CXX_REGEX */
 ret:
 		return match;
 	}
 
+#if FIM_WANT_OBSOLETE
 int strchr_count(const fim_char_t*s, int c)
 {
 	int n=0;
@@ -575,6 +735,7 @@ int newlines_count(const fim_char_t*s)
 		++c;
 	return c;
 }
+#endif /* FIM_WANT_OBSOLETE */
 
 const fim_char_t* next_row(const fim_char_t*s, int cols)
 {
@@ -588,7 +749,8 @@ const fim_char_t* next_row(const fim_char_t*s, int cols)
 	 * next_row("12")     -> \0
 	 * next_row("1234")   ->  4
 	 * */
-	const fim_char_t *b=s;int l=strlen(s);
+	const fim_char_t *b=s;
+	int l=strlen(s);
 	if(!s)
 		return FIM_NULL;
 	if((s=strchr(s,'\n'))!=FIM_NULL)
@@ -616,8 +778,10 @@ int lines_count(const fim_char_t*s, int cols)
 	 * */
 	if(cols<=0)
 		return -1;
+#if FIM_WANT_OBSOLETE
 	if(cols==0)
 		return newlines_count(s);
+#endif /* FIM_WANT_OBSOLETE */
 
 	int n=0;
 	const fim_char_t*b;
@@ -636,37 +800,20 @@ int lines_count(const fim_char_t*s, int cols)
 		n+=s>b?(s-1-b)/cols:0;	/* extra lines due to the excessive line width (if s==b we can't expect any wrapping, of course )	*/
 		++n;	// the \n is counted as a new line
 		b=++s;	// if now *s==NUL, strchr simply will fail
-	};
+	}
 	//printf("n:%d\n",n);
 	s=b;//*b==NUL or *b points to the last substring non newline terminated
 	n+=(strlen(s))/cols;	// we count the last substring chars (with no wrapping exceptions)
 	return n;
 }
 
-int fim_common_test(void)
-{	
-	/*
-	 * this function should test the correctness of the functions in this file.
-	 * it should be used for debug purposes, for Fim maintainance.
-	 * */
-	printf("%d\n",0==lines_count("" ,6));
-	printf("%d\n",0==lines_count("aab" ,6));
-	printf("%d\n",1==lines_count("aaaaba\nemk" ,6));
-	printf("%d\n",2==lines_count("aaaaba\nemk\n" ,6));
-	printf("%d\n",3==lines_count("aaaabaa\nemk\n" ,6));
-	printf("%d\n",*next_row("123\n",3)=='\0');
-	printf("%d\n",*next_row("123\n4",3)=='4');
-	printf("%d\n",*next_row("12",3)=='\0');
-	printf("%d\n",*next_row("1234",3)=='4');
-	return 0;
-}
-
 int swap_bytes_in_int(int in)
 {
 	// to Most Significant Byte First
 	// FIXME : this function should be optimized
+	const int b=sizeof(int);
 	int out=0;
-	int b=sizeof(int),i=-1;
+	int i=-1;
 	while(i++<b/2)
 	{
 		((fim_byte_t*)&out)[i]=((fim_byte_t*)&in)[b-i-1];
@@ -675,36 +822,67 @@ int swap_bytes_in_int(int in)
 	return out;
 }
 
+int fim_common_test(void)
+{	
+	/*
+	 * this function should test the correctness of the functions in this file.
+	 * it should be used for debug purposes, for Fim maintainance.
+	 * */
+	int score = 0;
+	score += ( 0==lines_count("" ,6));
+	score += ( 0==lines_count("aab" ,6));
+	score += ( 1==lines_count("aaaaba\nemk" ,6));
+	score += ( 2==lines_count("aaaaba\nemk\n" ,6));
+	score += ( 3==lines_count("aaaabaa\nemk\n" ,6));
+	score += ( *next_row("123\n",3)=='\0');
+	score += ( *next_row("123\n4",3)=='4');
+	score += ( *next_row("12",3)=='\0');
+	score += ( *next_row("1234",3)=='4');
+	score += ( fim_my_dirname("media/file")  == "media" );
+	score += ( fim_my_dirname("media//file") == "media" );
+	score += ( fim_dirname("media/file")  == "media" );
+	score += ( fim_dirname("media//file") == "media" );
+	score += ( swap_bytes_in_int(0x01020304) == 0x04030201 );
+	return score - 14;
+}
+
+#if FIM_WANT_OBSOLETE
 int int2lsbf(int in)
 {
-	int one=0x01;
+	const int one=0x01;
 	if( 0x01 & (*(fim_byte_t*)(&one)) )/*true on msbf (like ppc), false on lsbf (like x86)*/
 		return swap_bytes_in_int(in);
 	return in;
 }
+#endif /* FIM_WANT_OBSOLETE */
 
 int int2msbf(int in)
 {
-	int one=0x01;
+	const int one=0x01;
 	if( 0x01 & (*(fim_byte_t*)(&one)) )/*true on msbf (like ppc), false on lsbf (like x86)*/
 		return in;
 	return swap_bytes_in_int(in);
 }
 
+double getmicroseconds(void)
+{
+	double dt=0.0;
+        struct timeval tv;
+        /*err=*/gettimeofday(&tv, FIM_NULL);
+	dt+=tv.tv_usec;
+	dt+=tv.tv_sec *1000 * 1000;
+	// note : we ignore err!
+	return dt;
+}
+
 double getmilliseconds(void)
 {
 	/*
          * For internal usage: returns with milliseconds precision the current clock time.
          * NOTE : this function is NOT essential.
          */
-	int err;//t,pt in ms; d in us
-	double dt=0.0;
-        struct timeval tv;
-        err=gettimeofday(&tv, FIM_NULL);
-	dt+=tv.tv_usec/1000;
-	dt+=tv.tv_sec *1000;
-	// note : we ignore err!
-	return dt;
+	//int err;//t,pt in ms; d in us
+	return getmicroseconds() / 1000;
 }
 
 #if 0
@@ -730,6 +908,12 @@ const fim_char_t * fim_getenv(const fim_char_t * name)
 #endif /* HAVE_GETENV */
 }
 
+const bool fim_getenv_is_nonempty(const fim_char_t * name)
+{
+	const fim_char_t * ev = fim_getenv(name);
+	return ev && (*ev != FIM_SYM_CHAR_NUL);
+}
+
 FILE * fim_fread_tmpfile(FILE * fp)
 {
 	/*
@@ -737,11 +921,13 @@ FILE * fim_fread_tmpfile(FILE * fp)
 	*/
 	FILE *tfd=FIM_NULL;
 
+	errno=0; // shield form previous errors'
 	if( ( tfd=tmpfile() )!=FIM_NULL )
 	{
 		/* todo : read errno in case of error and print some report.. */
 		const size_t buf_size=FIM_STREAM_BUFSIZE;
-		fim_char_t buf[buf_size];size_t rc=0,wc=0;/* on some systems fwrite has attribute warn_unused_result */
+		fim_char_t buf[buf_size];
+		size_t rc=0,wc=0;/* on some systems fwrite has attribute warn_unused_result */
 		while( (rc=fim_fread(buf,1,buf_size,fp))>0 )
 		{
 			wc=fwrite(buf,1,rc,tfd);
@@ -755,11 +941,11 @@ FILE * fim_fread_tmpfile(FILE * fp)
 		if(errno)
 		{
 			// FIXME: need to handle errors properly.
-			std::cout << "Error while reading temporary file: errno has now value "<<errno<<":"<<
+			std::cerr << "Error while reading temporary file: errno has now value "<<errno<<":"<<
 #if HAVE_STRERROR
 				strerror(errno)<<"\n";
 #else
-				sys_errlist[errno]<<"\n";
+				/*sys_errlist[errno]<< */"\n"; /* sys_errlist is deprecated in favour of strerror */
 #endif
 		}
 		/*
@@ -804,17 +990,30 @@ double fim_atof(const fim_char_t *nptr)
 	return sign?-n:n;
 }
 
-ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream)
+ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream, int delim)
 {
-	/*
-	 * WARNING : untested!
-	 */
+#ifdef HAVE_GETDELIM
+	if (!n)
+		free(*lineptr), *lineptr = FIM_NULL;
+	else
+		return getdelim(lineptr,n,delim,stream);
+	return EINVAL;
+#else /* HAVE_GETDELIM */
+	/* the man page depends on this */
+#endif /* HAVE_GETDELIM */
 #ifdef HAVE_GETLINE
-	return getline(lineptr,n,stream);
+	if (!n)
+		free(*lineptr), *lineptr = FIM_NULL;
+	else
+		return getline(lineptr,n,stream);
+	return EINVAL;
 #endif /* HAVE_GETLINE */
 #ifdef HAVE_FGETLN
+	if (!n)
+		fim_free(*lineptr), *lineptr = FIM_NULL;
+	else
 	{	
-		/* for BSD (in stdlib.h) */
+		/* for BSD (in cstdlib) */
 		fim_char_t *s,*ns;
 		size_t len=0;
 		s=fgetln(stream,&len);
@@ -830,6 +1029,7 @@ ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream)
 
 	bool is_dir(const fim::string nf)
 	{
+#if HAVE_SYS_STAT_H
 		struct stat stat_s;
 		/*	if the directory doesn't exist, return */
 		if(-1==stat(nf.c_str(),&stat_s))
@@ -837,6 +1037,9 @@ ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream)
 		if( ! S_ISDIR(stat_s.st_mode))
 			return false;
 		return true;
+#else /* HAVE_SYS_STAT_H */
+		return false;
+#endif /* HAVE_SYS_STAT_H */
 	}
 
 	bool is_file(const fim::string nf)
@@ -845,6 +1048,7 @@ ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream)
 #if 0
 		return !is_dir(nf);
 #else
+#if HAVE_SYS_STAT_H
 		struct stat stat_s;
 		/*	if the file (it can be a device, but not a directory) doesn't exist, return */
 		if(-1==stat(nf.c_str(),&stat_s))
@@ -852,6 +1056,7 @@ ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream)
 		if( S_ISDIR(stat_s.st_mode))
 			return false;
 		/*if(!S_IFREG(stat_s.st_mode))return false;*/
+#endif /* HAVE_SYS_STAT_H */
 		return true;
 #endif
 	}
@@ -862,6 +1067,7 @@ ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream)
 #if 0
 		return !is_dir(nf);
 #else
+#if HAVE_SYS_STAT_H
 		struct stat stat_s;
 		/*	if the file (it can be a device, but not a directory) doesn't exist, return */
 		if(-1==stat(nf.c_str(),&stat_s))
@@ -871,6 +1077,7 @@ ssize_t fim_getline(fim_char_t **lineptr, size_t *n, FILE *stream)
 		/*if(!S_IFREG(stat_s.st_mode))return false;*/
 		if( stat_s.st_size == 0 )
 			return false;
+#endif /* HAVE_SYS_STAT_H */
 		return true;
 #endif
 	}
@@ -936,6 +1143,22 @@ int fim_fgetc(FILE *stream)
 #endif /* FIM_WANT_ZLIB */
 }
 
+int fim_snprintf_fim_int(char *r, fim_int n)
+{
+	// this does not pass g++ -pedantic-errors
+	if(FIM_LARGE_FIM_INT)
+#if FIM_USE_OLDCXX
+		return sprintf(r,"%ld",(long int)n);
+#else
+		return sprintf(r,"%lld",(long long int)n);
+#endif
+		//return sprintf(r,"%jd",(intmax_t)n);
+		//return sprintf(r,"%zd",(signed size_t)n);
+		//return sprintf(r,"%lld",(long long int)n);
+	else
+		return sprintf(r,"%d",(int)n);
+}
+
 int fim_snprintf_XB(char *str, size_t size, size_t q)
 {
 	/* result fits in 5 bytes */
@@ -957,7 +1180,11 @@ int fim_snprintf_XB(char *str, size_t size, size_t q)
 	if(q/d<10)
 		src = snprintf(str, size, "%1.1f%c%c",((float)q)/((float)d),u,b);
 	else
+#if FIM_USE_OLDCXX
+		src = snprintf(str, size, "%d%c%c",(int)(q/d),u,b);
+#else
 		src = snprintf(str, size, "%zd%c%c",q/d,u,b);
+#endif
 	return src;
 }
 
@@ -969,7 +1196,7 @@ fim_byte_t * fim_pm_alloc(unsigned int width, unsigned int height, bool want_cal
 	nmemb *= 3;
 	/* FIXME: shall implement overflow checks here */
 	if(want_calloc)
-		return (fim_byte_t*)fim_calloc(nmemb, 1);
+		return (fim_byte_t*)fim_calloc(nmemb, size);
 	else
 		return (fim_byte_t*)fim_malloc(nmemb);
 }
@@ -988,7 +1215,7 @@ const fim_char_t * fim_basename_of(const fim_char_t * s)
 				return s+sl+1;
 		}
 #else
-		const fim_char_t *bn = strrchr(s,'/');
+		const fim_char_t * const bn = strrchr(s,'/');
 		if(bn)
 			s=bn+1;
 #endif
@@ -996,19 +1223,68 @@ const fim_char_t * fim_basename_of(const fim_char_t * s)
 	return s;
 }
 
+fim::string fim_getcwd(void)
+{
+		fim::string cwd;
+#if HAVE_GET_CURRENT_DIR_NAME
+		/* default */
+		if( fim_char_t *p = get_current_dir_name() )
+		{
+			cwd=p;
+			free(p);
+		}
+#else /* HAVE_GET_CURRENT_DIR_NAME */
+#if _BSD_SOURCE || _XOPEN_SOURCE >= 500
+		{
+			/* untested */
+			fim_char_t *buf[PATH_MAX];
+			getcwd(buf,PATH_MAX-1): 
+			buf[PATH_MAX-1]=FIM_SYM_CHAR_NUL;
+			cwd=buf;
+		}
+#endif /* _BSD_SOURCE || _XOPEN_SOURCE >= 500 */
+#endif /* HAVE_GET_CURRENT_DIR_NAME */
+		return cwd;
+}
+
+static fim_int fim_util_atoi_kmX(const fim_char_t *nptr, int base)
+{
+	// from rsb__util_atoi_kmX
+	fim_int v = fim_atoi(nptr);
+
+	if(*nptr=='-')
+		++nptr;
+	while(isdigit(*nptr))
+		++nptr;
+	if(*nptr && tolower(*nptr)=='g')
+		v *= base * base * base;
+	if(*nptr && tolower(*nptr)=='m')
+		v *= base * base;
+	if(*nptr && tolower(*nptr)=='k')
+		v *= base;
+
+	return v;
+}
+
+fim_int fim_util_atoi_km2(const fim_char_t *nptr)
+{
+	// from rsb__util_atoi_km2
+	return fim_util_atoi_kmX(nptr, 1024);
+}
+
 fim_int fim_atoi(const char*s)
 {
-	if(sizeof(fim_int)==sizeof(int))
-		return atoi(s);
-	else
+	if(FIM_LARGE_FIM_INT)
 		return atoll(s);
+	else
+		return atoi(s);
 }
 
 size_t fim_maxrss(void)
 {
 #if HAVE_SYS_RESOURCE_H
 	struct rusage usage;
-	int gru = getrusage(RUSAGE_SELF,&usage);
+	/* int gru = */getrusage(RUSAGE_SELF,&usage);// TODO: shall report in case of error.
 	// printf("ru_maxrss: %ld (maximum resident set size -- MB)\n",usage.ru_maxrss / 1024);
 	// return quantity in B
 	return usage.ru_maxrss * 1024;
@@ -1017,3 +1293,33 @@ size_t fim_maxrss(void)
 #endif /* HAVE_SYS_RESOURCE_H */
 }
 
+fim_bool_t fim_is_id(const char*s)
+{
+	/* is fim variable identifier ? */
+	if(s)
+	{
+		if(*s != '_' && ! isalpha(*s) )
+			goto no;
+		for(; *s == '_' || isalnum(*s); ++s )
+			;
+		if(!*s)
+			return true;
+	}
+no:
+	return false;
+}
+
+fim_bool_t fim_is_id_char(const char c)
+{
+	// valid only within desc file ids
+	return isalpha(c) || c == '_';
+}
+
+std::string fim_get_int_as_hex(const int i)
+{
+	fim_char_t buf[2*FIM_CHARS_FOR_INT];
+	snprintf(buf,FIM_CHARS_FOR_INT-1,"0x%x", i);
+	buf[FIM_CHARS_FOR_INT-1]='\0';
+	return buf;
+}
+
