@@ -1,4 +1,4 @@
-/* $LastChangedDate: 2024-04-09 21:08:29 +0200 (Tue, 09 Apr 2024) $ */
+/* $LastChangedDate: 2024-04-29 14:03:11 +0200 (Mon, 29 Apr 2024) $ */
 /*
  fim.cpp : Fim main program and accessory functions
 
@@ -311,9 +311,10 @@ FIM_MAN_fB("'b'") " hides the menu bar, "
 FIM_MAN_fB("'e'") " starts with empty menus, "
 FIM_MAN_fB("'" FIM_CNS_GTK_MNRBCHR_STR "'") " rebuilds the menus, and "
 FIM_MAN_fB("'" FIM_CNS_GTK_MNDLCHR_STR "'") " removes the menus."
-" " FIM_MAN_fB("Warning") ": the \\fB" "gtk" "\\fP mode is still work in progress."
+" " FIM_MAN_fB("Note") ": the \\fB" "gtk" "\\fP mode is a recent addition and may have defects."
 #endif /* FIM_WANT_SDL_OPTIONS_STRING FIM_WITH_LIBGTK */
 //#ifdef FIM_WITH_LIBIMLIB2
+" The " FIM_MAN_fB(FIM_DDN_INN_SDL) " mode works best with libsdl-2; libsdl-1.2 support is being discontinued.\n"
 " The " FIM_MAN_fB(FIM_DDN_INN_IL2) " option requests imlib2 and is unfinished: do not use it.\n"
 " The " FIM_MAN_fB(FIM_DDN_INN_DUMB) " test mode is there only for test purposes and is not interactive.\n"
 //#endif /* FIM_WITH_LIBIMLIB2 */
@@ -1129,7 +1130,7 @@ FIM_MAN_fB("fim")
 
 ".TP 15\n"
 ".B " FIM_CNS_HIST_COMPLETE_FILENAME "\n"
-"File where to load from or save. See"
+"File where to load from or save command history. See"
 " (man " FIM_MAN_fR("fimrc") "(5),"
 "  man " FIM_MAN_fR("readline") "(3))."
 "\n"
@@ -1268,13 +1269,19 @@ ret:
 			return 0;
 }
 
-void fim_args_from_desc_file(args_t& argsc, const fim_fn_t& dfn, const fim_char_t sc)
+bool fim_args_from_desc_file(args_t& argsc, const fim_fn_t& dfn, const fim_char_t sc)
 {
 	/* dfn: descriptions file name */
 	/* sc: separator char */
-	std::ifstream mfs (dfn.c_str(),std::ios::app);
+	std::ifstream mfs (dfn.c_str(),std::ios::in);
 	std::string ln;
 
+	if( !mfs.is_open() )
+	{
+		std::cerr << "Description file " << dfn << " could not be opened!" << std::endl;
+		return false; // a proper FIM_ERR.. would be better
+	}
+
 	while( std::getline(mfs,ln))
 	{
 		std::stringstream  ls(ln);
@@ -1287,6 +1294,8 @@ void fim_args_from_desc_file(args_t& argsc, const fim_fn_t& dfn, const fim_char_
 			if(std::getline(ls,fn,sc))
 				argsc.push_back(fn);
 	}
+	mfs.close();
+	return true;
 }
 
 std::string guess_output_device(void)const
@@ -1892,8 +1901,11 @@ std::string guess_output_device(void)const
 		    break;
 #if FIM_WANT_PIC_LISTUNMARK
 		case 0x6d666466:
-		    fim_args_from_desc_file(argsc,optarg,g_sc);
-		    break;
+		    if(!fim_args_from_desc_file(argsc,optarg,g_sc))
+		    {
+			retcode=FIM_PERR_BAD_PARAMS;
+			goto ret;
+		    }
 #endif /* FIM_WANT_PIC_LISTUNMARK */
 #if FIM_WANT_PIC_CMTS
 		case /*0x69646673*/'S': /* FIM_OSW_IMG_DSC_FILE_SEPC */
@@ -1901,7 +1913,12 @@ std::string guess_output_device(void)const
 			    g_sc = *optarg;
 		    break;
 		case /*0x6c696466*/'D': /* FIM_OSW_LOAD_IMG_DSC_FILE */
-		    cc.id_.fetch(optarg,g_sc);
+		    if(!cc.id_.fetch(optarg,g_sc))
+		    {
+			std::cerr << "Descriptions file could not be opened!" << std::endl;
+			retcode=FIM_PERR_BAD_PARAMS;
+			goto ret;
+		    }
 #if FIM_WANT_PIC_CMTS_RELOAD
 		    cc.browser_.dfl_.push_back({optarg,g_sc});
 #endif /* FIM_WANT_PIC_CMTS_RELOAD */
@@ -2191,7 +2208,7 @@ extern "C" {
 			    " )"
 	#else /* SVN_REVISION */
 	/* obsolete */
-	# define FIM_REPOSITORY_VERSION  "$LastChangedDate: 2024-04-09 21:08:29 +0200 (Tue, 09 Apr 2024) $"
+	# define FIM_REPOSITORY_VERSION  "$LastChangedDate: 2024-04-29 14:03:11 +0200 (Mon, 29 Apr 2024) $"
 	# ifdef FIM_REPOSITORY_VERSION 
 			    " ( repository version "
 		FIM_REPOSITORY_VERSION 	    
