@@ -1,8 +1,8 @@
-/* $LastChangedDate: 2015-02-11 08:43:28 +0100 (Wed, 11 Feb 2015) $ */
+/* $LastChangedDate: 2022-10-22 00:37:29 +0200 (Sat, 22 Oct 2022) $ */
 /*
  fim_stream.h : Textual output facility
 
- (c) 2007-2013 Michele Martone
+ (c) 2007-2022 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,18 +39,19 @@ namespace fim
 
 		fim_stream& operator<<(const  fim_char_t* s);
 
+		fim_stream& operator<<(const  fim_char_t c);
+
 		fim_stream& operator<<(const fim_byte_t* s);
 
 		fim_stream& operator<<(const  fim::string&s);
 
 		fim_stream& operator<<(float f);
-
 #if FIM_WANT_LONG_INT
-		fim_stream& operator<<(int i);
+		fim_stream& operator<<(fim_int i);
 #endif /* FIM_WANT_LONG_INT */
 
-		fim_stream& operator<<(fim_int i);
-
+		fim_stream& operator<<(int i);
+		fim_stream& operator<<(unsigned int i);
 	};
 }
 #endif /* FIM_FIM_STREAM_H */
